Package: bvarnet
Title: Bayesian Estimation of Dynamic VAR Models using STAN
Version: 1.0.0
Authors@R: person("Florian", "Metwaly", , "f.j.metwaly@uva.nl", role = c("aut", "cre", "cph"), comment = c(ORCID = "0009-0001-8357-7870"))
Author: Florian Metwaly [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0001-8357-7870>)
Maintainer: Florian Metwaly <f.j.metwaly@uva.nl>
Description: Bayesian estimation of multilevel Vector Autoregression (VAR) models
    using Stan. Supports Gaussian, Binary, and Ordinal (adjacent category) outcome
    variables with random effects and customizable priors.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://flo1met.github.io/bvarnet/,
        https://github.com/flo1met/bvarnet
BugReports: https://github.com/flo1met/bvarnet/issues
RoxygenNote: 7.3.3
Additional_repositories: https://mc-stan.org/r-packages/
Imports: instantiate, logspline, mvtnorm, posterior
Suggests: cmdstanr, bayesplot, knitr, qgraph, rmarkdown, testthat (>=
        3.0.0)
Config/testthat/edition: 3
Config/Needs/website: pkgdown
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: yes
Packaged: 2026-05-19 10:29:47 UTC; florianmetwaly
Repository: CRAN
Date/Publication: 2026-05-27 09:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(format,bvarnet_prior)
S3method(print,bvarnet)
S3method(print,bvarnet_prior)
S3method(print,bvarnet_priors)
S3method(print,summary.bvarnet)
S3method(summary,bvarnet)
export(bf_table)
export(bvar)
export(compare_to_truth)
export(extract_draws)
export(extract_network_matrix)
export(extract_param)
export(extract_random_effects)
export(extract_temporal)
export(get_default_priors)
export(prior)
export(set_priors)
export(sim_var)
importFrom(instantiate,stan_package_model)
importFrom(stats,complete.cases)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)

# bvarnet

<!-- badges: start -->
[![R-CMD-check](https://github.com/flo1met/bvarnet/actions/workflows/R-CMD-check.yml/badge.svg)](https://github.com/flo1met/bvarnet/actions/workflows/R-CMD-check.yml)
[![codecov](https://codecov.io/gh/flo1met/bvarnet/graph/badge.svg)](https://app.codecov.io/gh/flo1met/bvarnet)
[![pkgdown](https://github.com/flo1met/bvarnet/actions/workflows/pkgdown.yml/badge.svg)](https://flo1met.github.io/bvarnet/)
<!-- badges: end -->

## Bayesian Estimation of Multilevel Vector Autoregressive Networks using STAN

The `bvarnet` package allows user to estimate Bayesian multilevel Vector Auto Regressive (VAR) models for binary, ordinal and continuous outcome variables. Missing data is handled through listwise deletion and a skip-lag mechanism, which skips the estimation of the temporal structure when there is a gap between two timepoints.
Further, we provide functionality to conduct hypothesis test.

## Installation

You can install the development version of `bvarnet` from [GitHub](https://github.com/flo1met/bvarnet) with:

``` r
if(!requireNamespace("remotes")) {
  install.packages("remotes")
}
remotes::install_github("flo1met/bvarnet")
```

## Getting Started

The best place to start learning how to use this package to estimate Bayesian (multilevel) Vector Autoregression is the [Getting Started Vignette](https://flo1met.github.io/bvarnet/articles/bvarnet.html).
This vignette covers the basic model syntax, how to specify priors and how to extract the relevant parameters.

## Feature Requests and Contributions

- Predictions
- Cross-sectional Networks
- Correlated Random Effects
- Hierarchical Prior Distributions
- Performance Optimisation

## Roadmap

`bvarnet` is actively being developed. While the core functionality is stable, we have several features planned for future releases. For bug reports or feature request, please visit our [Issue Tracker](https://github.com/flo1met/bvarnet/issues).

libs <- file.path(R_PACKAGE_DIR, "libs", R_ARCH)
dir.create(libs, recursive = TRUE, showWarnings = FALSE)
for (file in c("symbols.rds", Sys.glob(paste0("*", SHLIB_EXT)))) {
  if (file.exists(file)) {
    file.copy(file, file.path(libs, file))
  }
}
inst_stan <- file.path("..", "inst", "stan")
if (dir.exists(inst_stan)) {
  warning(
    "Stan models in inst/stan/ are deprecated in {instantiate} ",
    ">= 0.0.4.9001 (2024-01-03). Please put them in src/stan/ instead."
  )
  if (file.exists("stan")) {
    warning("src/stan/ already exists. Not copying models from inst/stan/.")
  } else {
    message("Copying inst/stan/ to src/stan/.")
    fs::dir_copy(path = inst_stan, new_path = "stan")
  }
}
bin <- file.path(R_PACKAGE_DIR, "bin")
if (!file.exists(bin)) {
  dir.create(bin, recursive = TRUE, showWarnings = FALSE)
}
bin_stan <- file.path(bin, "stan")
fs::dir_copy(path = "stan", new_path = bin_stan)
callr::r(
  func = function(bin_stan) {
    instantiate::stan_package_compile(
      models = instantiate::stan_package_model_files(path = bin_stan),
      include_paths = bin_stan
    )
  },
  args = list(bin_stan = bin_stan),
  show = TRUE,
  stderr = "2>&1"
)

#' Extract temporal (VAR lag) effects
#'
#' Returns a data frame of autoregressive and/or cross-lagged parameter
#' summaries with convergence diagnostics, filtered by lag and effect type.
#'
#' @param object A \code{bvarnet} object returned by \code{\link{bvar}}.
#' @param lag Integer or \code{NULL}. If specified, only effects from this
#'   lag are returned. Default \code{NULL} (all lags).
#' @param effect Character. One of \code{"all"} (default), \code{"ar"}
#'   (autoregressive only), or \code{"cl"} (cross-lagged only).
#' @param bayes_factor Logical; if \code{TRUE}, append BF columns.
#'   Default \code{FALSE}.
#' @param null_value Numeric; null hypothesis for BF. Default 0.
#'
#' @return A data frame with columns \code{type}, \code{predictor},
#'   \code{outcome}, \code{mean}, \code{median}, \code{q5}, \code{q95},
#'   \code{rhat}, \code{ess_bulk}, \code{ess_tail}, and optionally
#'   \code{BF01}, \code{BF10}.
#'
#' @export
extract_temporal <- function(object,
                             lag = NULL,
                             effect = c("all", "ar", "cl"),
                             bayes_factor = FALSE,
                             null_value = 0) {
  stopifnot(inherits(object, "bvarnet"))
  effect <- match.arg(effect)

  tab <- extract_param(object,
                        bayes_factor = bayes_factor,
                        null_value   = null_value)

  # Filter to temporal types
  type_filter <- switch(effect,
    ar  = "Autoregressive",
    cl  = "Cross-lagged",
    all = c("Autoregressive", "Cross-lagged")
  )
  out <- tab[tab$type %in% type_filter, , drop = FALSE]

  # Optionally filter by lag
  if (!is.null(lag)) {
    stopifnot(is.numeric(lag), length(lag) == 1L, lag >= 1L)
    lag_pattern <- paste0("^lag", as.integer(lag), "_")
    out <- out[grepl(lag_pattern, out$predictor), , drop = FALSE]
  }

  rownames(out) <- NULL
  out
}


#' Extract random-effect summaries
#'
#' Returns random-effect standard deviations (group-level variance),
#' subject-level posterior means, or the full posterior draws of the
#' subject-level random effects \code{u}.
#'
#' @param object A \code{bvarnet} object returned by \code{\link{bvar}}.
#' @param what Character. What to extract:
#'   \describe{
#'     \item{\code{"sd"}}{Data frame of random-effect SD summaries
#'       (from \code{extract_param}).}
#'     \item{\code{"mean_u"}}{3D array \code{[node, subject, re]} of
#'       posterior means of subject-level effects.}
#'     \item{\code{"draws_u"}}{4D array \code{[draw, node, subject, re]}
#'       of full posterior draws.}
#'   }
#'
#' @return Depends on \code{what}; see above.
#'
#' @export
extract_random_effects <- function(object,
                                   what = c("sd", "mean_u", "draws_u")) {
  stopifnot(inherits(object, "bvarnet"))
  what <- match.arg(what)

  if (object$standata$n_re == 0L)
    stop("No random effects in this model (n_re = 0).", call. = FALSE)

  switch(what,
    sd = {
      tab <- extract_param(object)
      out <- tab[tab$type == "Random Effect SD", , drop = FALSE]
      rownames(out) <- NULL
      out
    },
    mean_u = .posterior_mean_u(object),
    draws_u = .extract_u_draws(object)
  )
}


#' Extract a network matrix of temporal coefficients
#'
#' Returns a named \code{p x p} matrix of posterior summary statistics for
#' the VAR lag coefficients at a chosen lag, suitable for network
#' visualisation (e.g., with \pkg{igraph} or \pkg{qgraph}).
#'
#' @param object A \code{bvarnet} object returned by \code{\link{bvar}}.
#' @param lag Integer. Which lag block. Default 1.
#' @param stat Character. Summary statistic to fill the matrix with:
#'   \code{"mean"} (default), \code{"median"}, \code{"q5"}, or \code{"q95"}.
#'
#' @return A named \code{p x p} numeric matrix. Element \code{[i, j]}
#'   gives the effect of variable \code{i} (lagged) on variable \code{j}
#'   (outcome). Row and column names are the outcome variable names.
#'
#' @export
extract_network_matrix <- function(object,
                                   lag = 1L,
                                   stat = c("mean", "median", "q5", "q95")) {
  stopifnot(inherits(object, "bvarnet"))
  stat <- match.arg(stat)

  sd <- object$standata
  p  <- sd$p
  K  <- sd$K
  stopifnot(lag >= 1L, lag <= K)

  nm <- get_param_names(sd)

  # Extract phi draws and build summary
  draws_phi <- extract_draws(object, "phi")
  phi_tab   <- build_summary_table(draws_phi, nm$b, nm$y, "phi")

  # Filter to the requested lag block
  lag_pattern <- paste0("^lag", as.integer(lag), "_")
  lag_rows <- grepl(lag_pattern, phi_tab$predictor)
  sub <- phi_tab[lag_rows, , drop = FALSE]

  # Build p x p matrix: sub rows cycle through predictors (lagged vars)
  # for each outcome column.
  mat <- matrix(sub[[stat]], nrow = p, ncol = p)
  y_names <- nm$y
  rownames(mat) <- y_names
  colnames(mat) <- y_names
  mat
}

# ──────────────────────────────────────────────────────────────────────────────
# test-summary-extractors.R — tests for summary.bvarnet(), extract_temporal(),
# extract_random_effects(), extract_network_matrix(), and exported extract_draws()
# ──────────────────────────────────────────────────────────────────────────────


# ═══════════════════════════════════════════════════════════════════════════════
# §1 — summary.bvarnet()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("summary.bvarnet returns summary.bvarnet class", {
  obj <- make_mock_bvarnet("gaussian")
  s   <- summary(obj)
  expect_s3_class(s, "summary.bvarnet")
})

test_that("summary.bvarnet contains expected elements", {
  obj <- make_mock_bvarnet("bernoulli")
  s   <- summary(obj)
  expect_true(all(c("table", "family", "p", "K", "n",
                     "rhat_max", "n_divergences") %in% names(s)))
})

test_that("summary.bvarnet table is a data.frame with correct columns", {
  obj <- make_mock_bvarnet("gaussian")
  s   <- summary(obj)
  expect_true(is.data.frame(s$table))
  expected_cols <- c("type", "predictor", "outcome",
                     "mean", "median", "q5", "q95",
                     "rhat", "ess_bulk", "ess_tail")
  expect_true(all(expected_cols %in% names(s$table)))
})

test_that("summary.bvarnet prints without error", {
  obj <- make_mock_bvarnet("gaussian")
  s   <- summary(obj)
  expect_output(print(s), "BVAR Network Summary")
  expect_output(print(s), "gaussian")
  expect_output(print(s), "Autoregressive")
})

test_that("summary.bvarnet warns for high Rhat", {
  obj <- make_mock_bvarnet()
  obj$convergence$rhat <- rep(1.05, nrow(obj$convergence))
  s <- summary(obj)
  expect_output(print(s), "WARNING")
})

test_that("summary.bvarnet with bayes_factor = TRUE includes BF10", {
  obj <- make_mock_bvarnet("gaussian")
  s   <- summary(obj, bayes_factor = TRUE)
  expect_true("BF10" %in% names(s$table))
})

test_that("print.summary.bvarnet returns object invisibly", {
  obj <- make_mock_bvarnet()
  s   <- summary(obj)
  ret <- withVisible(print(s))
  expect_false(ret$visible)
  expect_identical(ret$value, s)
})

test_that("print.summary.bvarnet truncates groups and shows footer", {
  obj <- make_mock_bvarnet("gaussian")
  s   <- summary(obj)
  # max_rows=1 should truncate almost every group
  out <- capture.output(print(s, max_rows = 1))
  txt <- paste(out, collapse = "\n")
  expect_true(grepl("more rows", txt))
  expect_true(grepl("extract_param\\(\\)", txt))
  expect_true(grepl("extract_temporal", txt))
  expect_true(grepl("extract_network_matrix\\(\\)", txt))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 — extract_temporal()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_temporal returns only temporal types", {
  obj <- make_mock_bvarnet("bernoulli")
  out <- extract_temporal(obj)
  expect_true(is.data.frame(out))
  expect_true(all(out$type %in% c("Autoregressive", "Cross-lagged")))
})

test_that("extract_temporal effect='ar' returns only AR rows", {
  obj <- make_mock_bvarnet("bernoulli")
  out <- extract_temporal(obj, effect = "ar")
  expect_true(all(out$type == "Autoregressive"))
  expect_true(nrow(out) > 0)
})

test_that("extract_temporal effect='cl' returns only CL rows", {
  obj <- make_mock_bvarnet("bernoulli")
  out <- extract_temporal(obj, effect = "cl")
  expect_true(all(out$type == "Cross-lagged"))
  expect_true(nrow(out) > 0)
})

test_that("extract_temporal lag filter works", {
  obj <- make_mock_bvarnet("bernoulli")  # K=1
  out <- extract_temporal(obj, lag = 1)
  expect_true(nrow(out) > 0)
  expect_true(all(grepl("^lag1_", out$predictor)))
})

test_that("extract_temporal has convergence columns", {
  obj <- make_mock_bvarnet("bernoulli")
  out <- extract_temporal(obj)
  expect_true(all(c("rhat", "ess_bulk", "ess_tail") %in% names(out)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 — extract_random_effects()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_random_effects errors when n_re = 0", {
  obj <- make_mock_bvarnet("bernoulli", n_re = 0L)
  expect_error(extract_random_effects(obj), "n_re = 0")
})

test_that("extract_random_effects what='sd' returns data.frame", {
  obj <- make_mock_bvarnet("bernoulli", n_re = 2L, J = 5L)
  out <- extract_random_effects(obj, what = "sd")
  expect_true(is.data.frame(out))
  expect_true(all(out$type == "Random Effect SD"))
  expect_equal(nrow(out), obj$standata$p * obj$standata$n_re)
})

test_that("extract_random_effects what='mean_u' returns 3D array", {
  obj <- make_mock_bvarnet("bernoulli", n_re = 2L, J = 5L)
  out <- extract_random_effects(obj, what = "mean_u")
  expect_true(is.array(out))
  expect_equal(length(dim(out)), 3L)
  expect_equal(dim(out), c(obj$standata$p, obj$standata$J, obj$standata$n_re))
})

test_that("extract_random_effects what='draws_u' returns 4D array", {
  obj <- make_mock_bvarnet("bernoulli", n_re = 2L, J = 5L)
  out <- extract_random_effects(obj, what = "draws_u")
  expect_true(is.array(out))
  expect_equal(length(dim(out)), 4L)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 — extract_network_matrix()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_network_matrix returns named p x p matrix", {
  obj <- make_mock_bvarnet("bernoulli")
  mat <- extract_network_matrix(obj)
  expect_true(is.matrix(mat))
  expect_equal(dim(mat), c(obj$standata$p, obj$standata$p))
  expect_false(is.null(rownames(mat)))
  expect_false(is.null(colnames(mat)))
  expect_equal(rownames(mat), colnames(mat))
})

test_that("extract_network_matrix respects stat argument", {
  obj  <- make_mock_bvarnet("gaussian")
  m1 <- extract_network_matrix(obj, stat = "mean")
  m2 <- extract_network_matrix(obj, stat = "median")
  # They should both be p x p but may have different values
  expect_equal(dim(m1), dim(m2))
})

test_that("extract_network_matrix errors for invalid lag", {
  obj <- make_mock_bvarnet("bernoulli")  # K=1
  expect_error(extract_network_matrix(obj, lag = 2))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §5 — extract_draws() (now exported)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_draws is accessible as an exported function", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_draws(obj, "beta")
  expect_true(is.matrix(res))
  expect_true(ncol(res) > 0)
})

# Extracted from test-bvarnet-object.R:134

# test -------------------------------------------------------------------------
n_iter <- 20L
n_chains <- 2L
obj    <- make_mock_bvarnet("bernoulli", n_iter = n_iter, n_chains = n_chains)
result <- bvarnet:::extract_draws(obj, "beta")

#' Summary method for bvarnet objects
#'
#' Returns a labelled posterior summary table grouped by parameter type,
#' with convergence diagnostics and optional Bayes factors. Wraps
#' \code{\link{extract_param}}.
#'
#' @param object A \code{bvarnet} object returned by \code{\link{bvar}}.
#' @param bayes_factor Logical; if \code{TRUE}, append Savage-Dickey BF
#'   columns. Default \code{FALSE}.
#' @param null_value Numeric scalar; null hypothesis value for BF computation.
#'   Default 0.
#' @param ... Ignored.
#'
#' @return An object of class \code{"summary.bvarnet"} (a list) with elements:
#'   \describe{
#'     \item{table}{Data frame from \code{extract_param()}.}
#'     \item{family}{Model family.}
#'     \item{p}{Number of outcome variables.}
#'     \item{K}{AR order.}
#'     \item{n}{Number of observations.}
#'     \item{rhat_max}{Maximum Rhat across all parameters.}
#'     \item{n_divergences}{Total divergent transitions.}
#'   }
#'
#' @export
summary.bvarnet <- function(object, bayes_factor = FALSE, null_value = 0, ...) {
  stopifnot(inherits(object, "bvarnet"))

  tab <- extract_param(object,
                        bayes_factor = bayes_factor,
                        null_value   = null_value)

  sd   <- object$standata
  conv <- object$convergence
  diag <- object$diagnostics

  rhat_max <- if (!is.null(conv) && "rhat" %in% names(conv))
    max(conv$rhat, na.rm = TRUE) else NA_real_
  n_div <- if (!is.null(diag) && "num_divergent" %in% names(diag))
    sum(diag$num_divergent) else 0L

  structure(
    list(
      table         = tab,
      family        = object$family,
      p             = sd$p,
      K             = sd$K,
      n             = sd$n_obs,
      rhat_max      = rhat_max,
      n_divergences = n_div
    ),
    class = "summary.bvarnet"
  )
}


#' Print a bvarnet summary
#'
#' Pretty-prints the output of \code{\link{summary.bvarnet}}, grouping
#' parameters by type and displaying convergence information.
#' Each group is truncated to \code{max_rows} rows; use \code{extract_param()}
#' or dedicated extractors to see full output.
#'
#' @param x A \code{summary.bvarnet} object.
#' @param digits Number of decimal digits for numeric columns. Default 3.
#' @param max_rows Maximum number of rows to print per parameter group.
#'   Default 10.
#' @param ... Ignored.
#'
#' @return \code{x} invisibly.
#' @export
print.summary.bvarnet <- function(x, digits = 3, max_rows = 10, ...) {
  cat("BVAR Network Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("Family: %s | p=%d | K=%d | n=%d\n",
              .format_family(x$family), x$p, x$K, x$n))
  cat(sprintf("Rhat max: %.3f | Divergences: %d\n",
              x$rhat_max, x$n_divergences))

  if (x$rhat_max > 1.01)
    cat("  WARNING: Rhat > 1.01 \u2014 chains may not have converged.\n")
  if (x$n_divergences > 0)
    cat("  WARNING: divergent transitions detected \u2014 check model/priors.\n")

  tab <- x$table

  # Determine which columns to print
  display_cols <- c("predictor", "outcome", "mean", "median",
                    "q5", "q95", "rhat", "ess_bulk", "ess_tail")
  if ("BF10" %in% names(tab))
    display_cols <- c(display_cols, "BF10")
  display_cols <- intersect(display_cols, names(tab))

  # Group by type and print each section
  types <- unique(tab$type)
  type_order <- c("Intercept", "Fixed Effect", "Autoregressive",
                  "Cross-lagged", "Random Effect SD", "Residual SD",
                  "Threshold")
  types <- intersect(type_order, types)

  for (tp in types) {
    cat(sprintf("\n--- %s ---\n", tp))
    sub <- tab[tab$type == tp, display_cols, drop = FALSE]
    rownames(sub) <- NULL
    n_total <- nrow(sub)
    truncated <- n_total > max_rows

    if (truncated) sub <- sub[seq_len(max_rows), , drop = FALSE]

    # Round numeric columns
    num_cols <- vapply(sub, is.numeric, logical(1))
    sub[num_cols] <- lapply(sub[num_cols], round, digits = digits)

    cat(.fmt_df(sub, right = FALSE), sep = "\n")
    cat("\n")

    if (truncated) {
      hint <- switch(tp,
        "Autoregressive"   = "extract_temporal(fit, effect = \"ar\")",
        "Cross-lagged"     = "extract_temporal(fit, effect = \"cl\")",
        "Random Effect SD" = "extract_random_effects(fit)",
        "Intercept"        = "extract_param(fit, type = \"Intercept\")",
        "Fixed Effect"     = "extract_param(fit, type = \"Fixed Effect\")",
        "Residual SD"      = "extract_param(fit, type = \"Residual SD\")",
        "Threshold"        = "extract_param(fit, type = \"Threshold\")",
        "extract_param(fit)")
      cat(sprintf("... %d more rows. Use %s for full output.\n",
                  n_total - max_rows, hint))
    }
  }

  cat("\n", strrep("=", 50), "\n", sep = "")
  cat("Use extract_param() or extract_param(fit, type = \"...\") for the full parameter table.\n")
  cat("Use extract_network_matrix() for the temporal network matrix.\n")
  invisible(x)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom instantiate stan_package_model
#' @importFrom stats complete.cases quantile rbinom rnorm runif sd setNames
## usethis namespace: end
NULL

# ──────────────────────────────────────────────────────────────────────────────
# helper-fixtures.R — session-cached fixtures & reusable test utilities
#
# This file is automatically sourced by testthat before any test file.
# Follows the bgms pattern of lazy-cached fixtures and shared helpers.
# ──────────────────────────────────────────────────────────────────────────────

# ── Load package functions ────────────────────────────────────────────────────
# When running via devtools::test() or test_check() the package is loaded
# automatically. When running via test_dir() or Rscript we source all R files
# so that the in-development versions are used instead of any installed copy.
# We always source to ensure local edits take precedence over the installed pkg.
local({
  here <- tryCatch(dirname(sys.frame(1)$ofile), error = function(e) ".")
  pkg_root <- normalizePath(file.path(here, "..", ".."))
  r_dir    <- file.path(pkg_root, "R")
  if (dir.exists(r_dir)) {
    for (f in list.files(r_dir, pattern = "\\.R$", full.names = TRUE)) source(f)
  }
})

# ── Session cache environment ────────────────────────────────────────────────
.test_cache <- new.env(parent = emptyenv())

# ── Fixture getters (each cached once per test session) ──────────────────────

#' Get a simulated bernoulli dataset (cached)
get_sim_bernoulli <- function() {

  if (!exists("sim_bernoulli", envir = .test_cache)) {
    .test_cache$sim_bernoulli <- sim_var(
      N = 10, T_obs = 50, p = 3, K = 1,
      family = "bernoulli", q = 2, seed = 42
    )
  }
  .test_cache$sim_bernoulli
}

#' Get a simulated gaussian dataset (cached)
get_sim_gaussian <- function() {
  if (!exists("sim_gaussian", envir = .test_cache)) {
    .test_cache$sim_gaussian <- sim_var(
      N = 10, T_obs = 50, p = 3, K = 1,
      family = "gaussian", q = 2, seed = 43
    )
  }
  .test_cache$sim_gaussian
}

#' Get a simulated ordinal dataset (cached)
get_sim_ordinal <- function() {
  if (!exists("sim_ordinal", envir = .test_cache)) {
    .test_cache$sim_ordinal <- sim_var(
      N = 10, T_obs = 50, p = 3, K = 1,
      family = "ordinal", q = 2, C = 5, seed = 44
    )
  }
  .test_cache$sim_ordinal
}

#' Get stan data for a bernoulli simulation (cached)
get_standata_bernoulli <- function() {
  if (!exists("standata_bernoulli", envir = .test_cache)) {
    sim <- get_sim_bernoulli()
    .test_cache$standata_bernoulli <- to_stan_data(
      data     = sim$data,
      family   = "bernoulli",
      id_col   = "id",
      time_col = "t",
      y_cols   = paste0("y_", 1:3),
      x_cols   = paste0("x_", 1:2),
      K        = 1
    )
  }
  .test_cache$standata_bernoulli
}

#' Get stan data for a gaussian simulation (cached)
get_standata_gaussian <- function() {
  if (!exists("standata_gaussian", envir = .test_cache)) {
    sim <- get_sim_gaussian()
    .test_cache$standata_gaussian <- to_stan_data(
      data     = sim$data,
      family   = "gaussian",
      id_col   = "id",
      time_col = "t",
      y_cols   = paste0("y_", 1:3),
      x_cols   = paste0("x_", 1:2),
      K        = 1
    )
  }
  .test_cache$standata_gaussian
}

#' Get stan data for an ordinal simulation (cached)
get_standata_ordinal <- function() {
  if (!exists("standata_ordinal", envir = .test_cache)) {
    sim <- get_sim_ordinal()
    .test_cache$standata_ordinal <- to_stan_data(
      data     = sim$data,
      family   = "ordinal",
      id_col   = "id",
      time_col = "t",
      y_cols   = paste0("y_", 1:3),
      x_cols   = paste0("x_", 1:2),
      K        = 1
    )
  }
  .test_cache$standata_ordinal
}


# ── Reusable test helpers ────────────────────────────────────────────────────

#' Check if all values lie within [lo, hi]
values_in_range <- function(x, lo, hi) {
  all(x >= lo & x <= hi, na.rm = TRUE)
}

#' Check if a matrix is symmetric
is_symmetric <- function(m, tol = .Machine$double.eps^0.5) {
  is.matrix(m) && nrow(m) == ncol(m) && all(abs(m - t(m)) < tol)
}

#' Build a minimal long-format data frame for testing to_stan_data
make_test_df <- function(N = 5, T_obs = 20, p = 2, q = 0,
                         family = "bernoulli", seed = 123) {
  set.seed(seed)
  df <- expand.grid(t = 1:T_obs, id = 1:N)
  df <- df[order(df$id, df$t), ]

  for (j in seq_len(p)) {
    col <- paste0("y_", j)
    if (family == "bernoulli") {
      df[[col]] <- rbinom(nrow(df), 1, 0.5)
    } else if (family == "gaussian") {
      df[[col]] <- rnorm(nrow(df))
    } else if (family == "ordinal") {
      df[[col]] <- sample(1:5, nrow(df), replace = TRUE)
    }
  }
  for (j in seq_len(q)) {
    df[[paste0("x_", j)]] <- rnorm(nrow(df))
  }
  df
}

#' Build a mock bvarnet object without running Stan
#'
#' Returns a minimal bvarnet list with a synthetic 3D draws array, a matching
#' summary data.frame, and stubbed diagnostics/timing/metadata fields.
#' p=2 outcomes, K=1 lag, n_fe=2 (Intercept + x_1).
#' For ordinal: C=3 (2 cutpoints).  For gaussian: sigma included.
#' When \code{n_re > 0}, \code{sd_u} and \code{u} draw columns are added
#' with \code{J} subjects and \code{n_re} random-effect columns.
#'
#' \code{family} can be a scalar (applied to all p nodes) or a character
#' vector of length p for mixed-family mocks. When mixed, sigma columns are
#' only generated for gaussian nodes and kappa columns only for ordinal nodes.
#' Ordinal nodes get \code{beta[1,j] = NA_real_} sentinel (D4).
make_mock_bvarnet <- function(family   = "bernoulli",
                               n_iter   = 20L,
                               n_chains = 2L,
                               n_re     = 0L,
                               J        = 5L) {
  p <- 2L; K <- 1L

  # Normalise family to a named vector of length p
  y_cols <- c("y_1", "y_2")
  if (length(family) == 1L) {
    family_vec <- setNames(rep(family, p), y_cols)
  } else {
    stopifnot(length(family) == p)
    family_vec <- setNames(family, y_cols)
  }

  # Pure ordinal: no Intercept, n_fe = 1; otherwise n_fe = 2
  is_pure_ordinal <- all(family_vec == "ordinal")
  n_fe <- if (is_pure_ordinal) 1L else 2L

  if (is_pure_ordinal) {
    beta_nm <- c("beta[1,1]", "beta[1,2]")
  } else {
    beta_nm <- c("beta[1,1]", "beta[2,1]", "beta[1,2]", "beta[2,2]")
  }
  phi_nm  <- c("phi[1,1]",  "phi[2,1]",  "phi[1,2]",  "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm)

  # sigma only for gaussian nodes
  gauss_idx <- which(family_vec == "gaussian")
  if (length(gauss_idx) > 0L) {
    sigma_nm <- paste0("sigma[", gauss_idx, "]")
    par_nms <- c(par_nms, sigma_nm)
  }

  # kappa only for ordinal nodes (C=3 → 2 cutpoints each)
  ord_idx <- which(family_vec == "ordinal")
  if (length(ord_idx) > 0L) {
    kappa_nm <- character(0)
    for (node in ord_idx)
      for (k in 1:2)
        kappa_nm <- c(kappa_nm, sprintf("kappa[%d,%d]", node, k))
    par_nms <- c(par_nms, kappa_nm)
  }

  # sd_u parameters: sd_u[node,re]
  if (n_re > 0L) {
    sd_u_nm <- character(0)
    for (node in seq_len(p))
      for (re in seq_len(n_re))
        sd_u_nm <- c(sd_u_nm, sprintf("sd_u[%d,%d]", node, re))
    par_nms <- c(par_nms, sd_u_nm)
  }

  # u parameters: array[p] matrix[J, n_re] → u[node, subject, re]
  u_nm <- character(0)
  if (n_re > 0L) {
    for (node in seq_len(p))
      for (subj in seq_len(J))
        for (re in seq_len(n_re))
          u_nm <- c(u_nm, sprintf("u[%d,%d,%d]", node, subj, re))
    par_nms <- c(par_nms, u_nm)
  }

  n_par <- length(par_nms)
  set.seed(42L)
  draws <- array(
    rnorm(n_iter * n_chains * n_par),
    dim      = c(n_iter, n_chains, n_par),
    dimnames = list(NULL, NULL, par_nms)
  )

  # Make sigma draws positive
  if (length(gauss_idx) > 0L) {
    sigma_idx_arr <- grep("^sigma\\[", par_nms)
    draws[, , sigma_idx_arr] <- abs(draws[, , sigma_idx_arr]) + 0.1
  }

  # Make kappa draws ordered (ascending per node)
  if (length(ord_idx) > 0L) {
    for (node in ord_idx) {
      k1 <- paste0("kappa[", node, ",1]")
      k2 <- paste0("kappa[", node, ",2]")
      draws[, , k1] <- -1 + runif(n_iter * n_chains, -0.2, 0.2)
      draws[, , k2] <-  1 + runif(n_iter * n_chains, -0.2, 0.2)
    }
  }

  # Set ordinal beta[1,j] to NA sentinel (D4) — only for mixed-family
  if (!is_pure_ordinal) for (j in ord_idx) {
    nm <- paste0("beta[1,", j, "]")
    draws[, , nm] <- NA_real_
  }

  # Make sd_u draws positive (half-prior)
  if (n_re > 0L) {
    sd_u_idx <- grep("^sd_u\\[", par_nms)
    draws[, , sd_u_idx] <- abs(draws[, , sd_u_idx])
  }

  smry <- data.frame(
    variable  = par_nms,
    rhat      = 1.001,
    ess_bulk  = 3000,
    ess_tail  = 2800,
    stringsAsFactors = FALSE
  )

  n_obs <- 10L
  Y <- matrix(0L, n_obs, p, dimnames = list(NULL, c("y_1", "y_2")))
  B <- matrix(0,  n_obs, p * K,
              dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  # Pure ordinal models have no Intercept in X; mixed/other families do
  is_pure_ordinal <- all(family_vec == "ordinal")
  if (is_pure_ordinal) {
    X <- matrix(0, n_obs, 1L, dimnames = list(NULL, "x_1"))
    n_fe <- 1L
  } else {
    X <- matrix(0, n_obs, n_fe, dimnames = list(NULL, c("Intercept", "x_1")))
  }

  if (n_re > 0L) {
    re_colnames <- paste0("z_", seq_len(n_re))
    Z <- matrix(rnorm(n_obs * n_re), n_obs, n_re,
                dimnames = list(NULL, re_colnames))
  } else {
    Z <- matrix(0, n_obs, 0L)
  }

  sd_list <- list(
    p = p, K = K, n_fe = n_fe, n_re = n_re, n = n_obs, n_obs = n_obs,
    J = J, Y = Y, X = X, B = B, Z = Z,
    id = rep(seq_len(J), length.out = n_obs),
    id_levels = as.character(seq_len(J)),
    x_center_means = NULL,
    fe_interaction_terms = list(),
    design_spec = list(
      id_col   = "id",
      time_col = "t",
      y_cols   = c("y_1", "y_2"),
      x_cols   = "x_1",
      center_x = FALSE,
      fe_interactions = NULL,
      re_interactions = NULL,
      re_cols    = character(0),
      re_temporal = FALSE,
      K          = K,
      skip_lag   = TRUE,
      na_action  = "listwise"
    )
  )
  if (length(ord_idx) > 0L) sd_list$C <- 3L
  if (length(ord_idx) > 0L) {
    C_per_node <- setNames(rep(NA_integer_, p), y_cols)
    C_per_node[ord_idx] <- 3L
    sd_list$C_per_node <- C_per_node
  }

  # Compute priors_needed for this mock
  priors_needed <- c("beta", "phi")
  if (n_re > 0L) priors_needed <- c(priors_needed, "sd_u")
  if (any(family_vec == "gaussian")) priors_needed <- c(priors_needed, "sigma")
  if (any(family_vec == "ordinal"))  priors_needed <- c(priors_needed, "kappa")

  structure(
    list(
      draws         = draws,
      convergence   = smry,
      diagnostics   = data.frame(
        num_divergent     = integer(n_chains),
        num_max_treedepth = integer(n_chains),
        ebfmi             = rep(1.0, n_chains)
      ),
      timing        = list(total = 5.0),
      metadata      = list(),
      return_codes  = rep(0L, n_chains),
      family        = family_vec,
      standata      = sd_list,
      priors        = set_priors(),
      priors_needed = priors_needed
    ),
    class = "bvarnet"
  )
}

# ──────────────────────────────────────────────────────────────────────────────
# test-to_stan_data.R — data preparation / design matrix tests
#
# Tests the full pipeline from long-format data to Stan-ready data list.
# Covers: dimensions, intercept handling, B matrix (lags), Z matrix (RE),
#         interaction terms, gap handling, centering, and edge cases.
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 Basic dimensions
# ═══════════════════════════════════════════════════════════════════════════════

test_that("stan data has correct p, J, K, n_obs for all families", {
  for (fam in c("bernoulli", "gaussian", "ordinal")) {
    N <- 5; T_obs <- 20; p <- 3; q <- 2; K <- 1

    df <- make_test_df(N = N, T_obs = T_obs, p = p, q = q, family = fam)

    sd <- to_stan_data(
      data = df, family = fam,
      id_col = "id", time_col = "t",
      y_cols = paste0("y_", 1:p), x_cols = paste0("x_", 1:q),
      K = K
    )

    expect_equal(sd$p, p, info = paste("p for", fam))
    expect_equal(sd$J, N, info = paste("J for", fam))
    expect_equal(sd$K, K, info = paste("K for", fam))
    expect_equal(sd$n_obs, N * (T_obs - K), info = paste("n_obs for", fam))
  }
})


test_that("Y matrix has correct dimensions", {
  df <- make_test_df(N = 4, T_obs = 25, p = 3, family = "gaussian")

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:3), character(0), K = 1)

  expect_equal(dim(sd$Y), c(4 * (25 - 1), 3))
  expect_true(all(colnames(sd$Y) == paste0("y_", 1:3)))
})


test_that("B matrix has p*K columns", {
  df <- make_test_df(N = 3, T_obs = 30, p = 4, family = "bernoulli")
  K <- 2

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:4), character(0), K = K)

  expect_equal(ncol(sd$B), 4 * K)  # p * K
  expect_equal(nrow(sd$B), sd$n_obs)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 Intercept handling
# ═══════════════════════════════════════════════════════════════════════════════

test_that("bernoulli and gaussian X matrix includes intercept column", {
  for (fam in c("bernoulli", "gaussian")) {
    df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = fam)

    sd <- to_stan_data(df, fam, "id", "t",
                       paste0("y_", 1:2), "x_1", K = 1)

    expect_true("Intercept" %in% colnames(sd$X),
                info = paste("Intercept column for", fam))
    expect_equal(sd$n_fe, 2)  # Intercept + x_1

    # Intercept column should be all 1s
    expect_true(all(sd$X[, "Intercept"] == 1),
                info = paste("Intercept values for", fam))
  }
})


test_that("ordinal X matrix does NOT include intercept column", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = "ordinal")

  sd <- to_stan_data(df, "ordinal", "id", "t",
                     paste0("y_", 1:2), "x_1", K = 1)

  expect_false("Intercept" %in% colnames(sd$X))
  expect_equal(sd$n_fe, 1)  # just x_1
})


test_that("no-covariate model: X is intercept-only (bernoulli/gaussian)", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 0, family = "bernoulli")

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_equal(sd$n_fe, 1)
  expect_equal(ncol(sd$X), 1)
  expect_equal(colnames(sd$X), "Intercept")
})


test_that("no-covariate ordinal: X has zero columns", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 0, family = "ordinal")

  sd <- to_stan_data(df, "ordinal", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_equal(sd$n_fe, 0)
  expect_equal(ncol(sd$X), 0)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 Lag matrix B
# ═══════════════════════════════════════════════════════════════════════════════

test_that("B column names follow lag_y pattern", {
  df <- make_test_df(N = 3, T_obs = 20, p = 3, family = "bernoulli")
  K <- 2

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:3), character(0), K = K)

  expected_names <- c(
    paste0("lag1_y_", 1:3),
    paste0("lag2_y_", 1:3)
  )
  expect_equal(colnames(sd$B), expected_names)
})


test_that("B values match actual lagged Y values (no gaps)", {
  N <- 2; T_obs <- 10; p <- 2; K <- 1
  df <- make_test_df(N = N, T_obs = T_obs, p = p, family = "gaussian")

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:p), character(0), K = K)

  # For the first subject, the first modeled row (t=2) should have
  # B = [y1[t=1], y2[t=1]]
  sub1 <- df[df$id == 1, ]
  expect_equal(unname(sd$B[1, 1]), sub1$y_1[1])
  expect_equal(unname(sd$B[1, 2]), sub1$y_2[1])
})


test_that("K = 1 and K = 2 produce different n_obs per subject", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, family = "bernoulli")

  sd1 <- to_stan_data(df, "bernoulli", "id", "t",
                      paste0("y_", 1:2), character(0), K = 1)
  sd2 <- to_stan_data(df, "bernoulli", "id", "t",
                      paste0("y_", 1:2), character(0), K = 2)

  expect_equal(sd1$n_obs, 3 * (20 - 1))
  expect_equal(sd2$n_obs, 3 * (20 - 2))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 Random effects matrix Z
# ═══════════════════════════════════════════════════════════════════════════════

test_that("Z is empty (0 columns) with no RE specification", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = "bernoulli")

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), "x_1", K = 1)

  expect_equal(sd$n_re, 0)
  expect_equal(ncol(sd$Z), 0)
})


test_that("re_cols creates Z with correct columns", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 2, family = "bernoulli")

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), paste0("x_", 1:2),
                     re_cols = "x_1", K = 1)

  expect_equal(sd$n_re, 1)
  expect_equal(ncol(sd$Z), 1)
  # Z values should match the corresponding x_1 values in X
  x1_idx <- which(colnames(sd$X) == "x_1")
  expect_equal(sd$Z[, 1], sd$X[, x1_idx])
})


test_that("re_temporal = TRUE adds p*K lag columns to Z", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 0, family = "bernoulli")
  K <- 1

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0),
                     re_temporal = TRUE, K = K)

  expect_equal(sd$n_re, 2 * K)  # p * K
  expect_equal(ncol(sd$Z), 2 * K)
})


test_that("re_cols + re_temporal combine correctly", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = "gaussian")
  K <- 1

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), "x_1",
                     re_cols = "x_1", re_temporal = TRUE, K = K)

  # 1 from re_cols + p*K from re_temporal = 1 + 2 = 3
  expect_equal(sd$n_re, 3)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §5 Centering
# ═══════════════════════════════════════════════════════════════════════════════

test_that("center_x = TRUE centers covariate columns but not intercept", {
  df <- make_test_df(N = 5, T_obs = 30, p = 2, q = 2, family = "gaussian")

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), paste0("x_", 1:2),
                     center_x = TRUE, K = 1)

  # Intercept should still be all 1s
  expect_true(all(sd$X[, "Intercept"] == 1))

  # Covariate columns should be approximately centered (mean ≈ 0)
  for (j in 2:ncol(sd$X)) {
    col_mean <- abs(mean(sd$X[, j]))
    expect_lt(col_mean, 0.01)
  }
})


# ═══════════════════════════════════════════════════════════════════════════════
# §6 Time gap handling (skip_lag)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("skip_lag = TRUE zeros out B rows with time gaps", {
  df <- make_test_df(N = 1, T_obs = 10, p = 2, family = "bernoulli")

  # Introduce a gap: remove time point 5
  df <- df[df$t != 5, ]

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     skip_lag = TRUE)

  # Time 6 → lag should be zeroed because t[6] - t[5] != 1 (gap)
  # Row for t=6 is the 5th modeled row (t=2,3,4,6,7,8,9,10 → modeled from t=2)
  # After removing t=5, the sub has 9 rows; first modeled row is t=2 → row 1
  # t=6 comes after removing t=5: raw rows are t=1,2,3,4,6,7,8,9,10
  # modeled rows start from t=2: t=2(1),t=3(2),t=4(3),t=6(4),...
  # Row 4 in the modeled data is t=6 whose lag is t=4 (gap of 2)
  gap_row <- 4  # t=6 in modeled data
  expect_true(all(sd$B[gap_row, ] == 0),
              info = "B row with time gap should be zeroed")
})


test_that("skip_lag = FALSE removes rows with time gaps", {
  df <- make_test_df(N = 1, T_obs = 10, p = 2, family = "bernoulli")

  # Introduce a gap
  df <- df[df$t != 5, ]

  sd_skip <- to_stan_data(df, "bernoulli", "id", "t",
                          paste0("y_", 1:2), character(0), K = 1,
                          skip_lag = TRUE)

  sd_noskip <- to_stan_data(df, "bernoulli", "id", "t",
                            paste0("y_", 1:2), character(0), K = 1,
                            skip_lag = FALSE)

  # skip_lag = FALSE should have fewer rows (gaps removed)
  expect_lt(sd_noskip$n_obs, sd_skip$n_obs)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §7 Subject ID mapping
# ═══════════════════════════════════════════════════════════════════════════════

test_that("id vector maps to sequential integers 1..J", {
  df <- make_test_df(N = 5, T_obs = 15, p = 2, family = "bernoulli")

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_true(all(sd$id %in% 1:5))
  expect_equal(length(unique(sd$id)), 5)
})


test_that("subjects with <= K rows are dropped without error", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "bernoulli")

  # Add a subject with only 1 row (< K+1 = 2)
  short_sub <- data.frame(id = 99, t = 1, y_1 = 0L, y_2 = 1L)
  df <- rbind(df, short_sub)

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  # The short subject should be silently dropped
  expect_equal(sd$J, 4)  # 3 original + 1 short (present in unique IDs)
  # But n_obs should only reflect the 3 contributing subjects
  expect_equal(sd$n_obs, 3 * (15 - 1))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §8 Ordinal-specific fields
# ═══════════════════════════════════════════════════════════════════════════════

test_that("ordinal model adds C and kappa priors to stan data", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "ordinal")

  sd <- to_stan_data(df, "ordinal", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_true("C" %in% names(sd))
  expect_true("prior_kappa_fam" %in% names(sd))
  expect_true(sd$C >= 2)
})


test_that("gaussian model adds sigma priors to stan data", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "gaussian")

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_true("prior_sigma_fam" %in% names(sd))
})


test_that("bernoulli model does NOT add C, sigma, or kappa priors", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "bernoulli")

  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_false("C" %in% names(sd))
  expect_false("prior_sigma_fam" %in% names(sd))
  expect_false("prior_kappa_fam" %in% names(sd))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §9 Prior fields always present
# ═══════════════════════════════════════════════════════════════════════════════

test_that("all families include beta, phi, sd prior fields", {
  for (fam in c("bernoulli", "gaussian", "ordinal")) {
    df <- make_test_df(N = 3, T_obs = 15, p = 2, family = fam)

    sd <- to_stan_data(df, fam, "id", "t",
                       paste0("y_", 1:2), character(0), K = 1)

    for (par in c("beta", "phi", "sd")) {
      expect_true(paste0("prior_", par, "_fam") %in% names(sd),
                  info = paste(par, "fam for", fam))
      expect_true(paste0(par, "_loc") %in% names(sd),
                  info = paste(par, "loc for", fam))
      expect_true(paste0(par, "_scale") %in% names(sd),
                  info = paste(par, "scale for", fam))
    }
  }
})


# ═══════════════════════════════════════════════════════════════════════════════
# §10 Interaction terms
# ═══════════════════════════════════════════════════════════════════════════════

test_that("fe_interactions adds columns to X", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, q = 2, family = "bernoulli")

  sd_no_int <- to_stan_data(df, "bernoulli", "id", "t",
                            paste0("y_", 1:2), paste0("x_", 1:2), K = 1)
  sd_int <- to_stan_data(df, "bernoulli", "id", "t",
                         paste0("y_", 1:2), paste0("x_", 1:2),
                         fe_interactions = list(c("x_1", "x_2")), K = 1)

  expect_gt(sd_int$n_fe, sd_no_int$n_fe)
  expect_true("x_1:x_2" %in% colnames(sd_int$X))
})


test_that("lag interaction adds p*K columns to X", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, q = 1, family = "bernoulli")
  K <- 1

  sd_int <- to_stan_data(df, "bernoulli", "id", "t",
                         paste0("y_", 1:2), "x_1",
                         fe_interactions = list(c("lag", "x_1")), K = K)

  # lag interaction should add p*K = 2 columns
  expected_cols <- paste0("lag1_y_", 1:2, ":x_1")
  expect_true(all(expected_cols %in% colnames(sd_int$X)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §11 Consistency: sim_var → to_stan_data round-trip
# ═══════════════════════════════════════════════════════════════════════════════

test_that("to_stan_data on sim_var output produces valid stan data (bernoulli)", {
  sd <- get_standata_bernoulli()

  expect_true(all(sd$Y %in% c(0L, 1L)))
  expect_equal(sd$p, 3)
  expect_equal(sd$K, 1)
  expect_true(sd$n_obs > 0)
  expect_true(sd$n_fe >= 1)  # at least intercept
})


test_that("to_stan_data on sim_var output produces valid stan data (gaussian)", {
  sd <- get_standata_gaussian()

  expect_true(all(is.finite(sd$Y)))
  expect_equal(sd$p, 3)
})


test_that("to_stan_data on sim_var output produces valid stan data (ordinal)", {
  sd <- get_standata_ordinal()

  expect_true(all(sd$Y >= 1))
  expect_true(all(sd$Y <= sd$C))
  expect_equal(sd$p, 3)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §12 Listwise deletion
# ═══════════════════════════════════════════════════════════════════════════════

test_that("listwise deletion removes rows with NA in y or x columns", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, q = 1, family = "gaussian")

  # Inject some NAs
  df$y_1[5]  <- NA
  df$x_1[10] <- NA

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), "x_1", K = 1)

  # Should have fewer obs than clean data (some rows deleted)
  sd_clean <- to_stan_data(make_test_df(N = 3, T_obs = 20, p = 2, q = 1, family = "gaussian"),
                           "gaussian", "id", "t",
                           paste0("y_", 1:2), "x_1", K = 1)

  expect_lt(sd$n_obs, sd_clean$n_obs)
  # No NA in Y or X
  expect_false(any(is.na(sd$Y)))
  expect_false(any(is.na(sd$X)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §13 Prior injection
# ═══════════════════════════════════════════════════════════════════════════════

test_that("to_stan_data runs without error using default set_priors()", {
  for (fam in c("bernoulli", "gaussian", "ordinal")) {
    df <- make_test_df(N = 3, T_obs = 15, p = 2, family = fam)
    expect_no_error(
      to_stan_data(df, fam, "id", "t",
                   paste0("y_", 1:2), character(0), K = 1,
                   priors = set_priors())
    )
  }
})

test_that("default priors produce prior_beta_fam == 1L", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2)
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors())
  expect_equal(sd$prior_beta_fam, 1L)
})

test_that("default priors produce beta_scale == 1 for bernoulli", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2)
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors())
  expect_equal(sd$beta_scale, 1)
})

test_that("custom cauchy beta prior is injected correctly", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2)
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors(beta = prior("cauchy", 0, 0.5)))
  expect_equal(sd$prior_beta_fam, 3L)
  expect_equal(sd$beta_scale, 0.5)
})

test_that("custom student_t phi prior is injected with correct df", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2)
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors(phi = prior("student_t", 0, 0.3, df = 5)))
  expect_equal(sd$prior_phi_fam, 2L)
  expect_equal(sd$phi_df, 5)
})

test_that("gaussian model output includes prior_sigma_fam and sigma_scale", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "gaussian")
  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)
  expect_true("prior_sigma_fam" %in% names(sd))
  expect_true("sigma_scale"     %in% names(sd))
})

test_that("gaussian model with default priors scales beta_scale by s_y", {
  set.seed(1)
  df <- make_test_df(N = 5, T_obs = 20, p = 2, family = "gaussian", seed = 1)
  # multiply Y values so sd >> 1
  df$y_1 <- df$y_1 * 10
  df$y_2 <- df$y_2 * 10
  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors())
  expect_gt(sd$beta_scale, 1)   # scaled up because sd(Y) >> 1
})

test_that("gaussian model with user-specified beta prior is NOT scaled", {
  set.seed(2)
  df <- make_test_df(N = 5, T_obs = 20, p = 2, family = "gaussian")
  df$y_1 <- df$y_1 * 10
  df$y_2 <- df$y_2 * 10
  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     priors = set_priors(beta = prior("normal", 0, 0.5)))
  # User set scale = 0.5; must NOT be multiplied by s_y
  expect_equal(sd$beta_scale, 0.5)
})

test_that("ordinal model includes prior_kappa_fam", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "ordinal")
  sd <- to_stan_data(df, "ordinal", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)
  expect_true("prior_kappa_fam" %in% names(sd))
})

test_that("bernoulli model does NOT include prior_kappa_fam", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, family = "bernoulli")
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)
  expect_false("prior_kappa_fam" %in% names(sd))
})

test_that("to_stan_data without explicit priors argument uses set_priors() defaults", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2)
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)
  expect_equal(sd$prior_beta_fam, 1L)
  expect_equal(sd$beta_scale, 1)
})

# Extracted from test-bayes_factor.R:921

# test -------------------------------------------------------------------------
mock <- make_mock_bvarnet("bernoulli")
expect_error(
    bf_table(mock, variable = "y_1", type = "intercepts"),
    "covariate effects"
  )

# ──────────────────────────────────────────────────────────────────────────────
# test-extract_param.R — tests for extract_param() and extract_draws()
#
# These functions require a fitted bvarnet object (Stan fit), which is
# heavy and slow to produce. This file uses mock objects to test the
# structure and logic without running MCMC. Full integration tests with
# real Stan fits should be gated behind skip_on_cran().
# make_mock_bvarnet() is defined in helper-fixtures.R and auto-sourced.
# ──────────────────────────────────────────────────────────────────────────────


# Also note: extract_draws is internal. When sourced (test_dir), it lives in the
# global env; when the package is loaded (devtools::test), use bvarnet:::.
# The helper always sources local R files so plain extract_draws() works here.

# ═══════════════════════════════════════════════════════════════════════════════
# §1 extract_draws() — class and family validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_draws rejects non-bvarnet objects", {
  expect_error(
    extract_draws(list(a = 1), "beta"),
    "inherits"
  )
})


test_that("extract_draws validates parameter argument", {
  mock_obj <- structure(list(), class = "bvarnet")

  expect_error(
    extract_draws(mock_obj, "invalid_param"),
    "arg"
  )
})


test_that("extract_draws rejects sigma for non-gaussian family", {
  mock_obj <- make_mock_bvarnet("bernoulli")

  expect_error(
    extract_draws(mock_obj, "sigma"),
    "gaussian"
  )
})


test_that("extract_draws rejects kappa for non-ordinal family", {
  mock_obj <- make_mock_bvarnet("bernoulli")

  expect_error(
    extract_draws(mock_obj, "kappa"),
    "ordinal"
  )
})


test_that("extract_draws rejects sd_u when n_re = 0", {
  mock_obj <- make_mock_bvarnet("gaussian")   # n_re = 0 by default

  expect_error(
    extract_draws(mock_obj, "sd_u"),
    "no random effects"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 extract_param() — class validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_param rejects non-bvarnet objects", {
  expect_error(
    extract_param(list(a = 1)),
    "inherits"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 compare_to_truth() — class validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("compare_to_truth rejects non-bvarnet fit object", {
  expect_error(
    compare_to_truth(list(a = 1), list()),
    "inherits"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 extract_param() — return structure
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_param returns a plain data.frame for bernoulli", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  expect_true(is.data.frame(res))
  expect_identical(class(res), "data.frame")   # not classed
})


test_that("extract_param result has all required columns", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  expected_cols <- c("type", "predictor", "outcome",
                     "mean", "median", "q5", "q95",
                     "rhat", "ess_bulk", "ess_tail")
  expect_true(all(expected_cols %in% names(res)))
})


test_that("extract_param includes beta and phi rows for bernoulli", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  expect_true(any(res$type %in% c("Intercept", "Fixed Effect")))
  expect_true(any(res$type == "Autoregressive"))
  expect_true(any(res$type == "Cross-lagged"))
})


test_that("extract_param does not include sigma/kappa for bernoulli", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  expect_false(any(res$type == "Residual SD"))
  expect_false(any(res$type == "Threshold"))
})


test_that("extract_param includes sigma rows for gaussian", {
  obj <- make_mock_bvarnet("gaussian")
  res <- extract_param(obj)

  expect_true(any(res$type == "Residual SD"))
  expect_equal(sum(res$type == "Residual SD"), obj$standata$p)
})


test_that("extract_param includes kappa rows for ordinal", {
  obj <- make_mock_bvarnet("ordinal")
  res <- extract_param(obj)

  expect_true(any(res$type == "Threshold"))
  # p=2, C-1=2 cutpoints: expect 4 threshold rows
  expect_equal(sum(res$type == "Threshold"),
               obj$standata$p * (obj$standata$C - 1L))
})


test_that("extract_param retains FE rows for pure ordinal (no sentinel filtering)", {
  obj <- make_mock_bvarnet("ordinal")
  res <- extract_param(obj)

  # Pure ordinal: beta[1,j] is x_1, not an intercept sentinel
  fe_rows <- res[res$type == "Fixed Effect", ]
  expect_equal(nrow(fe_rows), obj$standata$p)
  expect_true(all(fe_rows$predictor == "x_1"))
  expect_false(any(is.na(fe_rows$mean)))
  # No Intercept rows expected

  expect_equal(sum(res$type == "Intercept"), 0L)
})


test_that("extract_param filtering by type works", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  ar_rows <- subset(res, type == "Autoregressive")
  cl_rows <- subset(res, type == "Cross-lagged")
  expect_true(nrow(ar_rows) > 0)
  expect_true(nrow(cl_rows) > 0)
  expect_true(all(ar_rows$type == "Autoregressive"))
  expect_true(all(cl_rows$type == "Cross-lagged"))
})


test_that("extract_param rhat and ess columns are numeric", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  expect_true(is.numeric(res$rhat))
  expect_true(is.numeric(res$ess_bulk))
  expect_true(is.numeric(res$ess_tail))
})


test_that("extract_param predictor labels use variable names from standata", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj)

  intercept_rows <- subset(res, type == "Intercept")
  expect_true(all(intercept_rows$predictor == "Intercept"))

  phi_rows <- subset(res, type %in% c("Autoregressive", "Cross-lagged"))
  expect_true(all(grepl("lag1_y_[0-9]+", phi_rows$predictor)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §N — extract_param() type= argument
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_param type=NULL returns all rows (backward compat)", {
  obj  <- make_mock_bvarnet("gaussian")
  full <- extract_param(obj)
  with_null <- extract_param(obj, type = NULL)
  expect_equal(full, with_null)
})

test_that("extract_param type='Threshold' returns only threshold rows for ordinal", {
  obj <- make_mock_bvarnet("ordinal")
  res <- extract_param(obj, type = "Threshold")
  expect_true(is.data.frame(res))
  expect_true(nrow(res) > 0L)
  expect_true(all(res$type == "Threshold"))
})

test_that("extract_param type='Residual SD' returns only residual SD rows for gaussian", {
  obj <- make_mock_bvarnet("gaussian")
  res <- extract_param(obj, type = "Residual SD")
  expect_true(is.data.frame(res))
  expect_equal(nrow(res), obj$standata$p)
  expect_true(all(res$type == "Residual SD"))
})

test_that("extract_param type= returns empty data.frame when type absent in model", {
  obj <- make_mock_bvarnet("bernoulli")
  res <- extract_param(obj, type = "Threshold")
  expect_true(is.data.frame(res))
  expect_equal(nrow(res), 0L)
})

test_that("extract_param type= accepts multiple types", {
  obj <- make_mock_bvarnet("gaussian")
  res <- extract_param(obj, type = c("Autoregressive", "Cross-lagged"))
  expect_true(all(res$type %in% c("Autoregressive", "Cross-lagged")))
  expect_true(nrow(res) > 0L)
})

test_that("extract_param type= errors on unknown type value", {
  obj <- make_mock_bvarnet("bernoulli")
  expect_error(
    extract_param(obj, type = "NotAType"),
    "Unknown type value"
  )
})

#' Build a Stan data list from a long-format data frame
#'
#' Internal function called by \code{bvar()}. Constructs the list passed to
#' the Stan model: design matrices Y, X, B, Z; dimensions; and prior
#' hyperparameters derived from a \code{bvarnet_priors} object.
#'
#' @param data Data frame in long format.
#' @param family Character. \code{"bernoulli"}, \code{"ordinal"}, or
#'   \code{"gaussian"}.
#' @param id_col Character. Subject/group identifier column name.
#' @param time_col Character. Time column name.
#' @param y_cols Character vector. Outcome column names.
#' @param x_cols Character vector. Covariate column names.
#' @param center_x Logical. Grand-mean centre X. Default \code{FALSE}.
#' @param fe_interactions List or NULL. Fixed-effect interaction terms.
#' @param re_interactions List or NULL. Random-effect interaction terms.
#' @param re_cols Character vector. Random-slope columns.
#' @param re_temporal Logical. Random slopes on lag predictors. Default FALSE.
#' @param K Integer. AR order.
#' @param na_action Character. Only \code{"listwise"} currently supported.
#' @param skip_lag Logical. Zero-fill lags across irregular time gaps.
#' @param priors A \code{bvarnet_priors} object. Defaults to \code{set_priors()}.
#'
#' @return A named list ready to pass to \code{CmdStanModel$sample()}.
#' @keywords internal
#' @noRd
to_stan_data <- function(data,
                         family,
                         id_col,
                         time_col,
                         y_cols,
                         x_cols,
                         center_x = F, # grand-mean centering of covmat X
                         fe_interactions = NULL,
                         re_interactions = NULL,
                         re_cols = character(0),
                         re_temporal = FALSE,
                         K,
                         na_action = c("listwise"), # add automatic LW deletion
                         skip_lag = TRUE,           # skipping lag mechanism on/off
                         priors = set_priors(),
                         save_data = FALSE) {

  ## Use shared helper for all family-agnostic data wrangling
  family <- match.arg(family, choices = c("bernoulli", "ordinal", "gaussian"))

  shared <- .to_stan_data_shared(
    data = data, id_col = id_col, time_col = time_col,
    y_cols = y_cols, x_cols = x_cols, center_x = center_x,
    fe_interactions = fe_interactions, re_interactions = re_interactions,
    re_cols = re_cols, re_temporal = re_temporal, K = K,
    na_action = na_action, skip_lag = skip_lag, save_data = save_data
  )

  ## Family-specific Y casting
  Y <- shared$Y
  X <- shared$X
  if (family == "ordinal") {
    if (any(Y != floor(Y), na.rm = TRUE)) {
      stop("Ordinal Y must contain integer values. Found non-integer entries.")
    }
    Y <- matrix(as.integer(Y), nrow = nrow(Y), ncol = ncol(Y))
    colnames(Y) <- colnames(shared$Y)
    # Strip intercept from X for ordinal
    if ("Intercept" %in% colnames(X)) {
      X <- X[, colnames(X) != "Intercept", drop = FALSE]
    }
  } else if (family == "bernoulli") {
    Y <- matrix(as.integer(Y), nrow = nrow(Y), ncol = ncol(Y))
    colnames(Y) <- colnames(shared$Y)
  }

  n_fe <- ncol(X)

  if (family == "ordinal") {
    C <- max(Y, na.rm = TRUE)
    if (min(Y, na.rm = TRUE) < 1L) {
      stop("Ordinal Y must be coded as integers starting at 1. Found values < 1.")
    }
  }

  ## Data-dependent scaling for Gaussian
  if (family == "gaussian") {
    s_y <- mean(apply(Y, 2, sd, na.rm = TRUE))
    if (is.finite(s_y) && s_y > 0) {
      if (priors$beta$is_default)      priors$beta$scale      <- priors$beta$scale      * s_y
      if (priors$intercept$is_default) priors$intercept$scale <- priors$intercept$scale * s_y
      if (priors$sigma$is_default)     priors$sigma$scale     <- priors$sigma$scale     * s_y
    }
  }

  ## Rebuild Z from the (possibly stripped) X
  Z <- build_Z(X, shared$B, re_cols = re_cols, re_temporal = re_temporal)
  Z <- add_re_interactions_from_X(Z, X, shared$B, re_interactions)

  out <- list(
    p = shared$p,
    J = shared$J,
    K = shared$K,
    n_obs = shared$n_obs,
    n_fe = n_fe,
    n_re = ncol(Z),
    id = shared$id,
    Y = Y,
    X = X,
    B = shared$B,
    Z = Z,
    prior_beta_fam = priors$beta$family_int,
    beta_loc       = priors$beta$loc,
    beta_scale     = priors$beta$scale,
    beta_df        = priors$beta$df,
    prior_phi_fam  = priors$phi$family_int,
    phi_loc        = priors$phi$loc,
    phi_scale      = priors$phi$scale,
    phi_df         = priors$phi$df,
    prior_sd_fam   = priors$sd_u$family_int,
    sd_loc         = priors$sd_u$loc,
    sd_scale       = priors$sd_u$scale,
    sd_df          = priors$sd_u$df
  )

  ## attach FE interaction metadata (always set, even when empty)
  out$fe_interaction_terms <- shared$fe_interaction_terms
  if (length(shared$fe_interaction_terms) > 0L) {
    out$fe_interaction_colnames <- shared$fe_interaction_colnames
  }

  ## prediction metadata (not passed to Stan)
  out$id_levels <- shared$id_levels
  out$time_obs <- shared$time_obs
  out$x_center_means <- shared$x_center_means
  out$row_map <- shared$row_map
  out$n_rows_data <- shared$n_rows_data
  out$data_used <- shared$data_used
  out$design_spec <- shared$design_spec

  if (family == "ordinal") {
    out$C              <- C
    out$prior_kappa_fam <- priors$kappa$family_int
    out$kappa_loc      <- priors$kappa$loc
    out$kappa_scale    <- priors$kappa$scale
    out$kappa_df       <- priors$kappa$df
  }

  if (family == "gaussian") {
    out$prior_sigma_fam <- priors$sigma$family_int
    out$sigma_loc       <- priors$sigma$loc
    out$sigma_scale     <- priors$sigma$scale
    out$sigma_df        <- priors$sigma$df
  }

  if (family %in% c("gaussian", "bernoulli")) {
    out$prior_intercept_fam <- priors$intercept$family_int
    out$intercept_loc       <- priors$intercept$loc
    out$intercept_scale     <- priors$intercept$scale
    out$intercept_df        <- priors$intercept$df
  }

  return(out)
}


# ---- .to_stan_data_shared() — family-agnostic design matrices ----

#' Build shared design matrices (family-agnostic)
#'
#' Always includes the Intercept column in X. Does NOT pack priors or do
#' family-specific type casting. Used by both \code{to_stan_data()} (joint
#' path) and \code{.bvar_nodewise()} (mixed path).
#'
#' @return A list with p, J, K, n_obs, n_fe, n_re, id, Y, X, B, Z,
#'   id_levels, x_center_means, row_map, n_rows_data, design_spec,
#'   fe_interaction_terms, fe_interaction_colnames.
#' @keywords internal
#' @noRd
.to_stan_data_shared <- function(data, id_col, time_col, y_cols, x_cols,
                                  center_x = FALSE, fe_interactions = NULL,
                                  re_interactions = NULL, re_cols = character(0),
                                  re_temporal = FALSE, K, na_action = "listwise",
                                  skip_lag = TRUE, save_data = FALSE) {

  K <- as.integer(K)
  stopifnot(is.logical(skip_lag), length(skip_lag) == 1L, !is.na(skip_lag))

  data <- data[order(data[[id_col]], data[[time_col]]), ]

  na_action <- match.arg(na_action, "listwise")
  if (na_action == "listwise") {
    check_cols <- c(y_cols, x_cols)
    complete <- complete.cases(data[, check_cols, drop = FALSE])
    data <- data[complete, , drop = FALSE]
  }

  ids_unique <- unique(data[[id_col]])
  J <- length(ids_unique)
  p <- length(y_cols)
  q <- length(x_cols)
  PK <- p * K

  # Snapshot the cleaned data (post-sort, post-NA-deletion) if requested
  data_used <- if (isTRUE(save_data)) data else NULL

  n_rows_data <- nrow(data)
  data$.orig_row <- seq_len(n_rows_data)
  df_split <- split(data, data[[id_col]])

  n_obs_initial <- n_obs <- sum(vapply(df_split, function(sub) max(0L, nrow(sub) - K), integer(1)))

  # Y stored as real (family-agnostic)
  Y <- matrix(NA_real_, n_obs, p)
  X <- matrix(NA_real_, n_obs, q)
  B <- matrix(0, n_obs, PK)
  id_out <- integer(n_obs)
  time_obs <- rep(NA_real_, n_obs)
  row_map <- integer(n_obs)

  row <- 0L
  for (jj in seq_len(J)) {
    this_id <- ids_unique[jj]
    df_sub <- df_split[[as.character(this_id)]]
    df_sub <- df_sub[order(df_sub[[time_col]]), , drop = FALSE]

    Ti <- nrow(df_sub)
    if (Ti <= K) next

    times <- df_sub[[time_col]]
    Ymat <- as.matrix(df_sub[, y_cols, drop = FALSE])
    Xmat <- as.matrix(df_sub[, x_cols, drop = FALSE])

    for (t in (K + 1L):Ti) {
      row <- row + 1L
      id_out[row] <- jj
      time_obs[row] <- times[t]
      row_map[row] <- df_sub$.orig_row[t]
      Y[row, ] <- Ymat[t, ]
      X[row, ] <- Xmat[t, ]

      valid <- TRUE
      for (lag in 1:K) {
        if ((times[t] - times[t - lag]) != lag) {
          valid <- FALSE
          break
        }
      }

      if (valid) {
        for (lag in 1:K) {
          B[row, ((lag - 1L) * p + 1L):(lag * p)] <- Ymat[t - lag, ]
        }
      } else if (!skip_lag) {
        row <- row - 1L
        next
      }
    }
  }

  if (row < n_obs) {
    Y <- Y[seq_len(row), , drop = FALSE]
    X <- X[seq_len(row), , drop = FALSE]
    B <- B[seq_len(row), , drop = FALSE]
    id_out <- id_out[seq_len(row)]
    time_obs <- time_obs[seq_len(row)]
    row_map <- row_map[seq_len(row)]
    n_obs <- row
  }

  if (n_obs == 0L) {
    stop("All observations removed after missing-data handling. ",
         "Check your data for NAs and irregular time gaps.")
  }

  n_dropped <- n_obs_initial - n_obs
  if (n_dropped > 0) {
    message(sprintf("bvarnet: %d row(s) removed (na_action = '%s', skip_lag = %s). %d rows remain.",
                    n_dropped, na_action, skip_lag, n_obs))
  }

  x_center_means <- NULL
  if (isTRUE(center_x)) {
    means_X <- colMeans(X)
    X <- sweep(X, 2, means_X, "-")
    x_center_means <- means_X
  }

  # Always add intercept (shared convention)
  X <- cbind(Intercept = 1, X)
  colnames(X) <- c("Intercept", x_cols)

  colnames(Y) <- y_cols
  b_names <- unlist(lapply(1:K, function(lag) paste0("lag", lag, "_", y_cols)))
  colnames(B) <- b_names

  ## build FE interactions
  tmp <- add_terms_to_X(X, B, fe_interactions)
  X <- tmp$X
  fe_interaction_terms    <- normalize_terms(fe_interactions)
  fe_interaction_colnames <- tmp$new_names

  Z <- build_Z(X, B, re_cols = re_cols, re_temporal = re_temporal)
  Z <- add_re_interactions_from_X(Z, X, B, re_interactions)

  list(
    p = p, J = J, K = K, n_obs = n_obs,
    n_fe = ncol(X), n_re = ncol(Z),
    id = id_out, time_obs = time_obs, Y = Y, X = X, B = B, Z = Z,
    id_levels = as.character(ids_unique),
    x_center_means = x_center_means,
    row_map = row_map, n_rows_data = n_rows_data,
    data_used = data_used,
    design_spec = list(
      id_col = id_col, time_col = time_col,
      y_cols = y_cols, x_cols = x_cols,
      center_x = center_x,
      fe_interactions = fe_interactions,
      re_interactions = re_interactions,
      re_cols = re_cols, re_temporal = re_temporal,
      K = K, skip_lag = skip_lag, na_action = na_action
    ),
    fe_interaction_terms = fe_interaction_terms,
    fe_interaction_colnames = fe_interaction_colnames
  )
}


# ---- .to_stan_data_node() — per-node Stan data for node-wise fitting ----

#' Build Stan data for a single node from shared matrices
#'
#' @param shared List from \code{.to_stan_data_shared()}.
#' @param node Integer, which node (1..p).
#' @param family Character scalar, family for this node.
#' @param priors A \code{bvarnet_priors} object (original, unmodified).
#' @return Named list ready for \code{CmdStanModel$sample()}.
#' @keywords internal
#' @noRd
.to_stan_data_node <- function(shared, node, family, priors) {
  Y_node <- shared$Y[, node]

  if (family == "ordinal") {
    if (any(Y_node != floor(Y_node), na.rm = TRUE)) {
      stop("Ordinal Y must contain integer values. Found non-integer entries.")
    }
    Y_out <- matrix(as.integer(Y_node), ncol = 1L)
    X_out <- shared$X[, -1, drop = FALSE]  # strip intercept
    C     <- max(Y_out)
    n_fe  <- ncol(X_out)
  } else if (family == "bernoulli") {
    Y_out <- matrix(as.integer(Y_node), ncol = 1L)
    X_out <- shared$X
    n_fe  <- ncol(X_out)
  } else {
    Y_out <- matrix(Y_node, ncol = 1L)
    X_out <- shared$X
    n_fe  <- ncol(X_out)
  }

  # Rebuild Z from the (possibly stripped) X for this node
  Z_out <- build_Z(X_out, shared$B, re_cols = shared$design_spec$re_cols,
                    re_temporal = shared$design_spec$re_temporal)
  Z_out <- add_re_interactions_from_X(Z_out, X_out, shared$B,
                                       shared$design_spec$re_interactions)

  # Per-node Gaussian prior scaling (D3)
  node_priors <- priors
  if (family == "gaussian") {
    s_y <- sd(Y_node, na.rm = TRUE)
    if (is.finite(s_y) && s_y > 0) {
      if (node_priors$beta$is_default)      node_priors$beta$scale      <- node_priors$beta$scale      * s_y
      if (node_priors$intercept$is_default) node_priors$intercept$scale <- node_priors$intercept$scale * s_y
      if (node_priors$sigma$is_default)     node_priors$sigma$scale     <- node_priors$sigma$scale     * s_y
    }
  }

  # Abused K: Stan sees p=1, so K_node = p_full * K_orig to keep B correct
  K_node <- shared$p * shared$K

  out <- list(
    p      = 1L,
    J      = shared$J,
    K      = K_node,
    n_obs  = shared$n_obs,
    n_fe   = n_fe,
    n_re   = ncol(Z_out),
    id     = shared$id,
    Y      = Y_out,
    X      = X_out,
    B      = shared$B,
    Z      = Z_out,
    prior_beta_fam = node_priors$beta$family_int,
    beta_loc  = node_priors$beta$loc,
    beta_scale = node_priors$beta$scale,
    beta_df   = node_priors$beta$df,
    prior_phi_fam = node_priors$phi$family_int,
    phi_loc   = node_priors$phi$loc,
    phi_scale = node_priors$phi$scale,
    phi_df    = node_priors$phi$df,
    prior_sd_fam = node_priors$sd_u$family_int,
    sd_loc    = node_priors$sd_u$loc,
    sd_scale  = node_priors$sd_u$scale,
    sd_df     = node_priors$sd_u$df
  )

  if (family == "gaussian") {
    out$prior_sigma_fam <- node_priors$sigma$family_int
    out$sigma_loc  <- node_priors$sigma$loc
    out$sigma_scale <- node_priors$sigma$scale
    out$sigma_df   <- node_priors$sigma$df
  }

  if (family %in% c("gaussian", "bernoulli")) {
    out$prior_intercept_fam <- node_priors$intercept$family_int
    out$intercept_loc       <- node_priors$intercept$loc
    out$intercept_scale     <- node_priors$intercept$scale
    out$intercept_df        <- node_priors$intercept$df
  }

  if (family == "ordinal") {
    out$C <- C
    out$prior_kappa_fam <- node_priors$kappa$family_int
    out$kappa_loc  <- node_priors$kappa$loc
    out$kappa_scale <- node_priors$kappa$scale
    out$kappa_df   <- node_priors$kappa$df
  }

  out
}

build_Z <- function(X, B, re_cols = character(0), re_temporal = FALSE) {
  # X: n_obs x n_fe (incl intercept for bernoulli/gaussian; NO intercept for ordinal)
  # B: n_obs x (p*K)
  stopifnot(is.matrix(X), is.matrix(B))

  Z_list <- list()

  # random slopes on selected fixed-effect columns
  if (length(re_cols) > 0) {
    # Handle "Intercept" specially: for ordinal, X has no intercept column
    # (intercept is absorbed by kappa cutpoints), so create a column of 1s
    if ("Intercept" %in% re_cols && !"Intercept" %in% colnames(X)) {
      intercept_col <- matrix(1, nrow = nrow(X), ncol = 1)
      colnames(intercept_col) <- "Intercept"
      X <- cbind(X, intercept_col)
    }

    missing <- setdiff(re_cols, colnames(X))
    if (length(missing) > 0) {
      stop("re_cols not found in X: ", paste(missing, collapse = ", "))
    }
    Z_list[["X"]] <- X[, re_cols, drop = FALSE]
  }

  # random slopes on lag predictors (temporal structure)
  if (isTRUE(re_temporal)) {
    Z_list[["B"]] <- B  # all lag columns
  }

  if (length(Z_list) == 0) {
    # no random effects
    Z <- matrix(0.0, nrow(X), 0)
  } else {
    Z <- do.call(cbind, Z_list)
  }

  # ensure matrix + colnames
  Z <- as.matrix(Z)
  if (ncol(Z) > 0 && is.null(colnames(Z))) colnames(Z) <- paste0("re", seq_len(ncol(Z)))

  return(Z)
}

# Todo: Interaction effects!!!
# Todo: validate design matrix building
# Todo: validate missing data mechanism


### For debugging
# K <- 1
# data <- df
# id_col <- "id"
# time_col <- "t"
# y_cols <- c("y_1", "y_2", "y_3", "y_4")
# x_cols <- c("x_1", "x_2")
#
#
#
#
# fe_terms = list(
#   c("x_1","x_2"),                 # 2-way
#   c("x_1","x_2","lag"),          # 3-way
#   c("lag","x_1")                 # lag moderation (whole structure)
# )
#
# re_terms = list(
#   c("x_1","x_2"),                # random slope on interaction
#   c("lag","x_1")                  # random slopes on whole lag structure moderated by x_1
# )

normalize_terms <- function(terms) {
  if (is.null(terms) || length(terms) == 0) return(list())

  if (!is.list(terms)) stop("terms must be a list of character vectors, e.g. list(c('x1','x2'), c('lag','x1'))")

  #if ("lag" %in% t) t <- c("lag", t[t != "lag"]) # fix, put lags in front always

  out <- lapply(terms, function(t) {
    if (!is.character(t)) stop("Each term must be a character vector.")
    t <- trimws(t)
    if (length(t) < 2) stop("Each term must have at least 2 factors (e.g. c('x1','x2') or c('lag','x1')).")

    if (any(t == "")) stop("Empty factor name in term.")
    if (any(duplicated(t[t != "lag"]))) stop("Duplicate non-lag factor in term: ", paste(t, collapse=":"))

    t
  })

  out
}



add_terms_to_X <- function(X, B, terms) {
  terms <- normalize_terms(terms)
  if (length(terms) == 0) return(list(X = X, new_names = character(0)))

  blocks <- lapply(terms, function(f) make_term_matrix(X, B, f))
  W <- do.call(cbind, blocks)

  dup <- intersect(colnames(W), colnames(X))
  if (length(dup) > 0) stop("Fixed-effect interaction columns already exist in X: ", paste(dup, collapse=", "))

  X2 <- cbind(X, W)
  list(X = X2, new_names = colnames(W))
}

make_term_matrix <- function(Xc, B, factors) {
  has_lag <- any(factors == "lag")
  others  <- factors[factors != "lag"]

  # build multiplicative modifier from Xc columns
  if (length(others) == 0) {
    mod <- rep(1.0, nrow(Xc))
    suffix <- ""
  } else {
    missing <- setdiff(others, colnames(Xc))
    if (length(missing) > 0) stop("Unknown factor(s): ", paste(missing, collapse=", "))
    mod <- rep(1.0, nrow(Xc))
    for (v in others) mod <- mod * as.numeric(Xc[, v])
    suffix <- paste(others, collapse=":")
  }

  if (!has_lag) {
    M <- matrix(mod, nrow(Xc), 1)
    colnames(M) <- suffix
    return(M)
  }

  # lag expansion = whole temporal structure
  M <- B * mod
  colnames(M) <- if (suffix == "") colnames(B) else paste0(colnames(B), ":", suffix)
  M
}



required_fe_names_for_term <- function(B, factors) {
  has_lag <- any(factors == "lag")
  others  <- factors[factors != "lag"]
  suffix  <- if (length(others) == 0) "" else paste(others, collapse=":")

  if (!has_lag) {
    return(suffix)
  } else {
    if (is.null(colnames(B))) stop("B must have colnames for lag expansion.")
    if (suffix == "") return(colnames(B))
    return(paste0(colnames(B), ":", suffix))
  }
}

required_fe_names_for_re_terms <- function(B, re_terms) {
  re_terms <- normalize_terms(re_terms)
  unique(unlist(lapply(re_terms, function(f) required_fe_names_for_term(B, f))))
}

add_re_interactions_from_X <- function(Z, Xc, B, re_terms) {
  if (is.null(re_terms) || length(re_terms) == 0) return(Z)

  want <- required_fe_names_for_re_terms(B, re_terms)
  missing <- setdiff(want, colnames(Xc))
  if (length(missing) > 0) {
    stop(
      "Random-effect interaction(s) requested but not present in fixed effects (X). Missing: ",
      paste(missing, collapse = ", "),
      "\nAdd them to `interactions` / `lag_interactions` first."
    )
  }

  cbind(Z, Xc[, want, drop = FALSE])
}

# ──────────────────────────────────────────────────────────────────────────────
# test-input-validation.R — error condition and boundary case tests
#
# Follows the bgms pattern: every user-facing function should give
# informative errors on invalid input, tested with expect_error(regexp = ...).
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 sim_var() validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("sim_var rejects invalid family", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "poisson"),
    "arg"
  )
})


test_that("sim_var rejects N < 1", {
  expect_error(
    sim_var(N = 0, T_obs = 20, p = 2, family = "bernoulli", seed = 1),
    "N >= 1"
  )
})


test_that("sim_var rejects T_obs < K + 1", {
  expect_error(
    sim_var(N = 3, T_obs = 1, p = 2, K = 1, family = "bernoulli", seed = 1),
    "T_obs >= K"
  )
})


test_that("sim_var rejects p < 1", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 0, family = "bernoulli", seed = 1),
    "p >= 1"
  )
})


test_that("sim_var rejects wrong alpha length", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 3, family = "gaussian",
            alpha = c(1, 2), seed = 1, burnin = 0),
    "alpha"
  )
})


test_that("sim_var rejects wrong Phi dimensions", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 3, K = 1,
            family = "gaussian", Phi = matrix(0, 2, 2), seed = 1, burnin = 0),
    "Phi"
  )
})


test_that("sim_var rejects negative sigma", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "gaussian",
            sigma = c(-1, 1), seed = 1, burnin = 0),
    "sigma.*> 0"
  )
})


test_that("sim_var rejects C < 2 for ordinal", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal",
            C = 1, seed = 1, burnin = 0),
    "C >= 2"
  )
})


test_that("sim_var rejects negative burnin", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "bernoulli",
            burnin = -1, seed = 1),
    "burnin >= 0"
  )
})


test_that("sim_var rejects unordered kappa", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal", C = 3,
            kappa = list(c(1, 0), c(0, 1)),  # first one is unordered
            seed = 1, burnin = 0),
    "unsorted|kappa"
  )
})


test_that("sim_var rejects wrong kappa length", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal", C = 4,
            kappa = list(c(-1, 0), c(-1, 0, 1)),  # first has 2 instead of 3
            seed = 1, burnin = 0),
    "kappa"
  )
})


test_that("sim_var rejects wrong gamma dimensions", {
  expect_error(
    sim_var(N = 3, T_obs = 20, p = 3, q = 2,
            family = "gaussian",
            gamma = matrix(0, 1, 3),  # should be 2x3
            seed = 1, burnin = 0),
    "gamma"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 to_stan_data() validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("to_stan_data rejects invalid family", {
  df <- make_test_df(N = 3, T_obs = 10, p = 2)
  expect_error(
    to_stan_data(df, "poisson", "id", "t", paste0("y_", 1:2), character(0), K = 1),
    "arg"
  )
})


test_that("to_stan_data where ordinal Y has values < 1 gives error", {
  df <- make_test_df(N = 3, T_obs = 10, p = 2, family = "ordinal")
  df$y_1 <- df$y_1 - 1  # now contains 0s

  expect_error(
    to_stan_data(df, "ordinal", "id", "t", paste0("y_", 1:2), character(0), K = 1),
    "< 1"
  )
})


test_that("to_stan_data stops when all observations are removed", {
  # A very small dataset where NA + K causes all data to be removed
  df <- data.frame(id = c(1, 1), t = c(1, 2), y_1 = c(NA, 0L), y_2 = c(1L, NA))

  expect_error(
    to_stan_data(df, "bernoulli", "id", "t", paste0("y_", 1:2), character(0), K = 1),
    "All observations removed"
  )
})


test_that("to_stan_data rejects re_cols not found in X", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = "bernoulli")

  expect_error(
    to_stan_data(df, "bernoulli", "id", "t",
                 paste0("y_", 1:2), "x_1",
                 re_cols = "nonexistent_col", K = 1),
    "re_cols not found"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 normalize_terms() validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("normalize_terms rejects non-list input", {
  expect_error(
    bvarnet:::normalize_terms("x_1"),
    "list"
  )
})


test_that("normalize_terms rejects single-factor terms", {
  expect_error(
    bvarnet:::normalize_terms(list(c("x_1"))),
    "at least 2"
  )
})


test_that("normalize_terms rejects duplicate factors", {
  expect_error(
    bvarnet:::normalize_terms(list(c("x_1", "x_1"))),
    "Duplicate"
  )
})


test_that("normalize_terms rejects empty factor names", {
  expect_error(
    bvarnet:::normalize_terms(list(c("x_1", ""))),
    "Empty"
  )
})


test_that("normalize_terms accepts valid terms", {
  result <- bvarnet:::normalize_terms(list(c("x_1", "x_2"), c("lag", "x_1")))
  expect_length(result, 2)
  expect_equal(result[[1]], c("x_1", "x_2"))
})


test_that("normalize_terms returns empty list for NULL", {
  expect_equal(bvarnet:::normalize_terms(NULL), list())
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 Interaction term validation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("add_terms_to_X rejects unknown factor names", {
  X <- matrix(1, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, 2, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  expect_error(
    bvarnet:::add_terms_to_X(X, B, list(c("x_1", "nonexistent"))),
    "Unknown factor"
  )
})


test_that("add_terms_to_X rejects duplicate columns", {
  X <- matrix(1, 10, 3, dimnames = list(NULL, c("Intercept", "x_1", "x_1:x_2")))
  B <- matrix(0, 10, 2, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  # x_1:x_2 already exists in X, so adding the interaction should fail
  # First we need x_2 in X for the term to be valid
  X2 <- cbind(X, x_2 = 1)
  expect_error(
    bvarnet:::add_terms_to_X(X2, B, list(c("x_1", "x_2"))),
    "already exist"
  )
})


test_that("add_re_interactions_from_X rejects missing FE interactions", {
  Z <- matrix(0, 10, 0)
  X <- matrix(1, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, 2, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  # Ask for RE interaction on c("x_1","x_2") which has not been added to FE
  expect_error(
    bvarnet:::add_re_interactions_from_X(Z, X, B, list(c("x_1", "x_2"))),
    "not present in fixed effects"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §5 Edge cases
# ═══════════════════════════════════════════════════════════════════════════════

test_that("single subject (N=1) works", {
  result <- sim_var(N = 1, T_obs = 30, p = 2, family = "gaussian",
                    seed = 400, burnin = 10)
  expect_equal(nrow(result$data), 30)
  expect_equal(result$truth$N, 1L)
  expect_equal(nrow(result$truth$alpha_i), 1)
})


test_that("single node (p=1) works", {
  result <- sim_var(N = 3, T_obs = 20, p = 1, family = "bernoulli",
                    seed = 401, burnin = 10)
  expect_equal(ncol(result$truth$Phi), 1)
  expect_true("y_1" %in% names(result$data))
})


test_that("minimum ordinal C = 2 works", {
  result <- sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal",
                    C = 2, seed = 402, burnin = 10)
  expect_true(all(result$data$y_1 %in% c(1L, 2L)))
})


test_that("minimum T_obs = K + 1 works", {
  result <- sim_var(N = 3, T_obs = 2, p = 2, K = 1,
                    family = "gaussian", seed = 403, burnin = 10)
  expect_equal(nrow(result$data), 3 * 2)
})


test_that("to_stan_data works with minimal data (1 subject, K+1 rows)", {
  df <- data.frame(id = 1L, t = 1:3, y_1 = rnorm(3), y_2 = rnorm(3))

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     c("y_1", "y_2"), character(0), K = 1)

  expect_equal(sd$n_obs, 2)  # T=3, K=1 → 2 modeled rows
  expect_equal(sd$J, 1)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §6 bvar() input validation (no Stan required)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("bvar rejects invalid family string", {
  df <- make_test_df(N = 3, T_obs = 10, p = 2)

  expect_error(
    bvar(id_col = "id", time_col = "t",
         y_cols = paste0("y_", 1:2), x_cols = character(0),
         data = df, family = "poisson", K = 1),
    "arg"
  )
})

# ──────────────────────────────────────────────────────────────────────────────
# test-to_stan_data-edge.R — edge-case coverage for to_stan_data()
#
# Tests boundary dimensions, family-specific casting, design matrix mechanics,
# and the nodewise path (.to_stan_data_node).
# See dev/to_stan_data_coverage_plan.md for rationale.
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 Single subject (N=1, J=1)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("to_stan_data works with a single subject (J=1)", {
  df <- make_test_df(N = 1, T_obs = 20, p = 2, family = "gaussian")
  sd <- to_stan_data(df, "gaussian", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1)

  expect_equal(sd$J, 1L)
  expect_equal(sd$n_obs, 19L)
  expect_true(all(sd$id == 1L))
  expect_equal(ncol(sd$Z), 0L)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §2 Single node (p=1)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("to_stan_data works with a single node (p=1)", {
  df <- make_test_df(N = 3, T_obs = 15, p = 1, family = "bernoulli")
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     "y_1", character(0), K = 1)

  expect_equal(sd$p, 1L)
  expect_equal(ncol(sd$B), 1L)
  expect_equal(ncol(sd$Y), 1L)
  expect_equal(colnames(sd$B), "lag1_y_1")
})

# ═══════════════════════════════════════════════════════════════════════════════
# §3 Large K with short T (subject dropping)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("large K drops subjects with T_i <= K", {
  # 2 subjects: one with 10 rows (survives K=5), one with 4 rows (dropped)
  df_long  <- make_test_df(N = 1, T_obs = 10, p = 2, family = "gaussian", seed = 1)
  df_short <- make_test_df(N = 1, T_obs = 4,  p = 2, family = "gaussian", seed = 2)
  df_short$id <- 2L
  df <- rbind(df_long, df_short)

  sd <- suppressMessages(
    to_stan_data(df, "gaussian", "id", "t",
                 paste0("y_", 1:2), character(0), K = 5)
  )

  expect_equal(sd$J, 2L)
  # only subject 1 contributes rows: 10 - 5 = 5
  expect_equal(sd$n_obs, 5L)
  expect_true(all(sd$id == 1L))
  expect_equal(ncol(sd$B), 2 * 5)  # p * K = 10
})

# ═══════════════════════════════════════════════════════════════════════════════
# §4 K > 1 lag column ordering
# ═══════════════════════════════════════════════════════════════════════════════

test_that("K=2 lag columns are correctly ordered and filled", {
  df <- data.frame(
    id = rep(1L, 6),
    t  = 1:6,
    y_1 = c(10, 20, 30, 40, 50, 60),
    y_2 = c(11, 21, 31, 41, 51, 61)
  )
  sd <- to_stan_data(df, "gaussian", "id", "t",
                     c("y_1", "y_2"), character(0), K = 2)

  expect_equal(colnames(sd$B),
               c("lag1_y_1", "lag1_y_2", "lag2_y_1", "lag2_y_2"))

  # Row 1 of modeled data is t=3. lag1 = t=2 values, lag2 = t=1 values
  expect_equal(unname(sd$B[1, "lag1_y_1"]), 20)
  expect_equal(unname(sd$B[1, "lag1_y_2"]), 21)
  expect_equal(unname(sd$B[1, "lag2_y_1"]), 10)
  expect_equal(unname(sd$B[1, "lag2_y_2"]), 11)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §5 Ordinal Y must be integer-coded
# ═══════════════════════════════════════════════════════════════════════════════

test_that("ordinal Y with non-integer values is rejected", {
  df <- data.frame(
    id  = rep(1:2, each = 10),
    t   = rep(1:10, 2),
    y_1 = rep(c(1.5, 2.3, 3.0, 1.7, 2.0), 4)
  )
  expect_error(
    to_stan_data(df, "ordinal", "id", "t", "y_1", character(0), K = 1),
    "integer"
  )
})

test_that("ordinal Y with integer-valued doubles passes", {
  df <- data.frame(
    id  = rep(1:2, each = 10),
    t   = rep(1:10, 2),
    y_1 = rep(c(1.0, 2.0, 3.0, 1.0, 2.0), 4)
  )
  sd <- to_stan_data(df, "ordinal", "id", "t", "y_1", character(0), K = 1)
  expect_true(is.integer(sd$Y))
  expect_equal(sd$C, 3L)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §6 Ordinal with C=2 (minimum categories)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("ordinal C=2 returns correct metadata", {
  df <- data.frame(
    id  = rep(1:3, each = 10),
    t   = rep(1:10, 3),
    y_1 = rep(c(1L, 2L, 1L, 2L, 1L), 6)
  )
  sd <- to_stan_data(df, "ordinal", "id", "t", "y_1", character(0), K = 1)

  expect_equal(sd$C, 2L)
  expect_true("prior_kappa_fam" %in% names(sd))
  expect_true("kappa_scale" %in% names(sd))
})

# ═══════════════════════════════════════════════════════════════════════════════
# §7 Gaussian prior scaling with near-zero SD
# ═══════════════════════════════════════════════════════════════════════════════

test_that("gaussian constant Y keeps priors at unscaled defaults", {
  df <- data.frame(
    id  = rep(1:3, each = 10),
    t   = rep(1:10, 3),
    y_1 = rep(5.0, 30),
    y_2 = rep(3.0, 30)
  )
  priors <- set_priors()
  default_beta_scale  <- priors$beta$scale
  default_sigma_scale <- priors$sigma$scale

  sd <- to_stan_data(df, "gaussian", "id", "t",
                     c("y_1", "y_2"), character(0), K = 1,
                     priors = priors)

  # s_y = 0, so scaling should be skipped; priors stay at defaults
  expect_equal(sd$beta_scale, default_beta_scale)
  expect_equal(sd$sigma_scale, default_sigma_scale)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §8 Empty fe_interactions = list()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("fe_interactions = list() is same as NULL", {
  df <- make_test_df(N = 3, T_obs = 15, p = 2, q = 1, family = "gaussian")
  sd_null <- to_stan_data(df, "gaussian", "id", "t",
                          paste0("y_", 1:2), "x_1", K = 1,
                          fe_interactions = NULL)
  sd_empty <- to_stan_data(df, "gaussian", "id", "t",
                           paste0("y_", 1:2), "x_1", K = 1,
                           fe_interactions = list())

  expect_equal(ncol(sd_null$X), ncol(sd_empty$X))
  expect_equal(sd_null$n_fe, sd_empty$n_fe)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §9 Ordinal + re_cols includes "Intercept"
# ═══════════════════════════════════════════════════════════════════════════════

test_that("ordinal with re_cols = 'Intercept' has random intercept in Z", {
  df <- make_test_df(N = 5, T_obs = 20, p = 2, family = "ordinal")
  sd <- to_stan_data(df, "ordinal", "id", "t",
                     paste0("y_", 1:2), character(0), K = 1,
                     re_cols = "Intercept")

  # X should NOT contain Intercept (ordinal strips it)
  expect_false("Intercept" %in% colnames(sd$X))
  # Z should contain the random intercept (build_Z reintroduces it)
  expect_equal(sd$n_re, 1L)
  expect_true(all(sd$Z[, 1] == 1))
})

# ═══════════════════════════════════════════════════════════════════════════════
# §10 Multiple time gaps within one subject
# ═══════════════════════════════════════════════════════════════════════════════

test_that("multiple time gaps within one subject are handled correctly", {
  # Times: 1,2,5,6,7,10,11 — gaps at t=5 (from 2) and t=10 (from 7)
  df <- data.frame(
    id  = rep(1L, 7),
    t   = c(1, 2, 5, 6, 7, 10, 11),
    y_1 = c(0, 1, 1, 0, 1, 0, 1)
  )

  # skip_lag = TRUE: keep rows, zero B at gaps

  sd_skip <- to_stan_data(df, "bernoulli", "id", "t",
                          "y_1", character(0), K = 1, skip_lag = TRUE)

  # Modeled rows: t=2,5,6,7,10,11 (6 rows, indices start at K+1=2)
  expect_equal(sd_skip$n_obs, 6L)

  # B for t=5 (gap from t=2, diff=3) should be zero
  # t=5 is the 2nd modeled row
  expect_equal(unname(sd_skip$B[2, 1]), 0)

  # B for t=10 (gap from t=7, diff=3) should be zero
  # t=10 is the 5th modeled row
  expect_equal(unname(sd_skip$B[5, 1]), 0)

  # B for t=6 (lag from t=5, diff=1) should be non-zero
  # t=6 is the 3rd modeled row
  expect_equal(unname(sd_skip$B[3, 1]), 1)  # y_1 at t=5

  # skip_lag = FALSE: drop gap rows
  sd_drop <- suppressMessages(
    to_stan_data(df, "bernoulli", "id", "t",
                 "y_1", character(0), K = 1, skip_lag = FALSE)
  )
  # Rows with valid lags: t=2, t=6, t=7, t=11 → 4 rows
  expect_equal(sd_drop$n_obs, 4L)
})

# ═══════════════════════════════════════════════════════════════════════════════
# §11 Node-level extraction per family (.to_stan_data_node)
# ═══════════════════════════════════════════════════════════════════════════════

test_that(".to_stan_data_node extracts correct column and applies family rules", {
  # Build shared data with 2 nodes (gaussian-like Y, will cast per family)
  set.seed(99)
  df <- data.frame(
    id  = rep(1:3, each = 15),
    t   = rep(1:15, 3),
    y_1 = rnorm(45, mean = 0, sd = 5),
    y_2 = rnorm(45, mean = 0, sd = 0.1)
  )
  # Make y_1 ordinal-safe (integer 1..3) for ordinal tests below
  df$y_1_ord <- sample(1:3, 45, replace = TRUE)
  # Make y_1 binary for bernoulli tests
  df$y_1_bin <- rbinom(45, 1, 0.5)

  priors <- set_priors()

  # --- Gaussian: scales priors by per-node sd ---
  shared_g <- .to_stan_data_shared(df, "id", "t",
                                    c("y_1", "y_2"), character(0), K = 1)
  node1 <- .to_stan_data_node(shared_g, 1L, "gaussian", set_priors())
  node2 <- .to_stan_data_node(shared_g, 2L, "gaussian", set_priors())

  # Y should be the correct column
  expect_equal(as.numeric(node1$Y), shared_g$Y[, 1])
  expect_equal(as.numeric(node2$Y), shared_g$Y[, 2])

  # Priors should be scaled by per-node sd, not mean sd
  sd_y1 <- sd(shared_g$Y[, 1])
  sd_y2 <- sd(shared_g$Y[, 2])
  expect_equal(node1$beta_scale, 1.0 * sd_y1)  # default scale=1
  expect_equal(node2$beta_scale, 1.0 * sd_y2)
  # They should differ because y_1 has sd≈5, y_2 has sd≈0.1
  expect_true(abs(node1$beta_scale - node2$beta_scale) > 1)

  # X should include Intercept for gaussian
  expect_true("Intercept" %in% colnames(node1$X))

  # --- Bernoulli: keeps intercept, integer Y ---
  shared_b <- .to_stan_data_shared(df, "id", "t",
                                    "y_1_bin", character(0), K = 1)
  node_b <- .to_stan_data_node(shared_b, 1L, "bernoulli", set_priors())
  expect_true(is.integer(node_b$Y))
  expect_true("Intercept" %in% colnames(node_b$X))

  # --- Ordinal: strips intercept from X ---
  shared_o <- .to_stan_data_shared(df, "id", "t",
                                    "y_1_ord", character(0), K = 1)
  node_o <- .to_stan_data_node(shared_o, 1L, "ordinal", set_priors())
  expect_true(is.integer(node_o$Y))
  expect_false("Intercept" %in% colnames(node_o$X))
  expect_true("C" %in% names(node_o))
})

#' StudentLife Data
#'
#' Assessing mental health, academic performance and behavioral trends of college students using smartphones.
#' Wang, R., Chen, F., Chen, Z., Li, T., Harari, G., Tignor, S., Zhou, X., Ben-Zeev, D., & Campbell, A. T. (2014).
#' StudentLife: assessing mental health, academic performance and behavioral trends of college students using smartphones.
#' Proceedings of the 2014 ACM International Joint Conference on Pervasive and Ubiquitous Computing, 3–14. https://doi.org/10.1145/2632048.2632054
#'
#' @format ## `studentlife`
#' A data frame with 1912 rows and 72 columns:
#'
#' @source https://openesmdata.org/datasets/0004_wang/
#' @source https://doi.org/10.1145/2632048.2632054
"studentlife"

#' Extract labelled parameter summaries from a fitted bvarnet model
#'
#' Returns a single flat data frame with posterior summaries (mean, median,
#' 5th/95th percentiles) and convergence diagnostics (Rhat, ESS) for all
#' model parameters.
#'
#' @param object A \code{bvarnet} object returned by \code{bvar()}.
#' @param bayes_factor Logical; if \code{TRUE}, append \code{BF01} and
#'   \code{BF10} columns computed via the Savage-Dickey density ratio for
#'   beta and phi parameters.  Default \code{FALSE}.
#' @param null_value Numeric scalar; the null hypothesis value for Bayes
#'   factor computation (default 0).  Only used when \code{bayes_factor = TRUE}.
#' @param type Character vector or \code{NULL} (default). If supplied, only
#'   rows matching the given type(s) are returned.  Valid values are:
#'   \code{"Intercept"}, \code{"Fixed Effect"}, \code{"Autoregressive"},
#'   \code{"Cross-lagged"}, \code{"Random Effect SD"}, \code{"Residual SD"},
#'   \code{"Threshold"}.
#'
#' @return A data frame with columns: \code{type}, \code{predictor},
#'   \code{outcome}, \code{mean}, \code{median}, \code{q5}, \code{q95},
#'   \code{rhat}, \code{ess_bulk}, \code{ess_tail}, and optionally
#'   \code{BF01}, \code{BF10}.
#'
#' @export
extract_param <- function(object, bayes_factor = FALSE, null_value = 0,
                          type = NULL) {
  stopifnot(inherits(object, "bvarnet"))

  .valid_types <- c("Intercept", "Fixed Effect", "Autoregressive",
                    "Cross-lagged", "Random Effect SD", "Residual SD",
                    "Threshold")
  if (!is.null(type)) {
    bad <- setdiff(type, .valid_types)
    if (length(bad) > 0L)
      stop(sprintf(
        "Unknown type value(s): %s. Valid types are: %s.",
        paste(sQuote(bad), collapse = ", "),
        paste(sQuote(.valid_types), collapse = ", ")
      ), call. = FALSE)
  }

  sd   <- object$standata
  nm   <- get_param_names(sd)
  smry <- object$convergence   # data.frame: variable, rhat, ess_bulk, ess_tail

  # Join Rhat + ESS from object$convergence by Stan parameter name.
  # stan_colnames: character vector aligned with rows of tab.
  join_convergence <- function(tab, stan_colnames) {
    idx          <- match(stan_colnames, smry$variable)
    tab$rhat     <- smry$rhat[idx]
    tab$ess_bulk <- smry$ess_bulk[idx]
    tab$ess_tail <- smry$ess_tail[idx]
    tab
  }

  # ---------- Intercepts & fixed effects (beta) ----------
  draws_beta <- extract_draws(object, "beta")

  # Remove ordinal-intercept NA sentinel columns (D4) BEFORE summary.
  # Sentinels only exist in mixed-family models where X retains the Intercept
  # row; pure ordinal strips the Intercept so beta[1,j] is a real covariate.
  beta_cols <- colnames(draws_beta)
  has_intercept <- "Intercept" %in% colnames(sd$X)
  if (has_intercept) {
    ord_indices <- which(object$family == "ordinal")
    sentinel_cols <- paste0("beta[1,", ord_indices, "]")
    keep_beta_cols <- !beta_cols %in% sentinel_cols
  } else {
    keep_beta_cols <- rep(TRUE, length(beta_cols))
  }
  draws_beta_clean <- draws_beta[, keep_beta_cols, drop = FALSE]

  # Build filtered name lists for row/col mapping
  # Original order: (fe1,y1), (fe2,y1), ..., (fe_nfe,y1), (fe1,y2), ...
  all_beta_indices <- expand.grid(fe = seq_along(nm$fe),
                                   node = seq_along(nm$y))
  keep_idx <- keep_beta_cols
  kept_fe <- all_beta_indices$fe[keep_idx]
  kept_node <- all_beta_indices$node[keep_idx]

  # Build summary table manually for the filtered columns
  d_mean   <- colMeans(draws_beta_clean)
  d_median <- apply(draws_beta_clean, 2L, stats::median)
  d_q5     <- apply(draws_beta_clean, 2L, stats::quantile, probs = 0.05)
  d_q95    <- apply(draws_beta_clean, 2L, stats::quantile, probs = 0.95)
  beta_tab <- data.frame(
    type      = ifelse(nm$fe[kept_fe] == "Intercept", "Intercept", "Fixed Effect"),
    predictor = nm$fe[kept_fe],
    outcome   = nm$y[kept_node],
    mean      = as.numeric(d_mean),
    median    = as.numeric(d_median),
    q5        = as.numeric(d_q5),
    q95       = as.numeric(d_q95),
    stringsAsFactors = FALSE
  )
  beta_stan_names <- colnames(draws_beta_clean)
  beta_tab <- join_convergence(beta_tab, beta_stan_names)

  # ---------- Autoregressive & Cross-lagged effects (phi) ----------
  draws_phi <- extract_draws(object, "phi")
  phi_tab   <- build_summary_table(draws_phi, nm$b, nm$y, "placeholder")

  # Classify each phi row as Autoregressive or Cross-lagged
  p_  <- sd$p
  K_  <- sd$K
  nr_ <- p_ * K_
  nc_ <- p_
  col_indices <- rep(seq_len(nc_), each = nr_)
  row_indices <- rep(seq_len(nr_), times = nc_)
  row_within  <- ((row_indices - 1L) %% p_) + 1L
  phi_tab$type <- ifelse(row_within == col_indices,
                         "Autoregressive", "Cross-lagged")
  phi_tab <- join_convergence(phi_tab, colnames(draws_phi))

  # ---------- Random-effect SDs (sd_u) ----------
  re_sd_tab <- if (sd$n_re > 0) {
    draws_sd <- extract_draws(object, "sd_u")
    tab      <- build_summary_table(draws_sd, nm$y, nm$re, "Random Effect SD")
    join_convergence(tab, colnames(draws_sd))
  } else NULL

  # ---------- Residual SD (sigma, gaussian only) ----------
  gauss_idx <- .family_which(object, "gaussian")
  sigma_tab <- if (length(gauss_idx) > 0) {
    draws_sigma <- extract_draws(object, "sigma")
    gauss_names <- nm$y[gauss_idx]
    tab <- build_summary_table(draws_sigma, gauss_names, "sigma", "Residual SD")
    join_convergence(tab, colnames(draws_sigma))
  } else NULL

  # ---------- Thresholds (kappa, ordinal only) ----------
  ord_idx <- .family_which(object, "ordinal")
  kappa_tab <- if (length(ord_idx) > 0) {
    draws_kappa <- extract_draws(object, "kappa")
    cn    <- colnames(draws_kappa)
    parts <- strsplit(gsub("kappa\\[|\\]", "", cn), ",")
    j_idx <- as.integer(vapply(parts, `[[`, character(1L), 1L))
    c_idx <- as.integer(vapply(parts, `[[`, character(1L), 2L))
    tab <- data.frame(
      type      = "Threshold",
      predictor = paste0("kappa(", nm$y[j_idx], ", c", c_idx, ")"),
      outcome   = "\u2014",
      mean      = colMeans(draws_kappa),
      median    = apply(draws_kappa, 2L, stats::median),
      q5        = apply(draws_kappa, 2L, stats::quantile, probs = 0.05),
      q95       = apply(draws_kappa, 2L, stats::quantile, probs = 0.95),
      stringsAsFactors = FALSE
    )
    join_convergence(tab, cn)
  } else NULL

  # ---------- Combine into a single flat data.frame ----------
  out <- do.call(rbind, Filter(Negate(is.null),
                               list(beta_tab, phi_tab, re_sd_tab, sigma_tab, kappa_tab)))
  rownames(out) <- NULL

  # ---------- Append BFs for beta and phi params ----------
  if (isTRUE(bayes_factor)) {
    out$BF01 <- NA_real_
    out$BF10 <- NA_real_

    # Identify rows with Stan param names we can compute BFs for
    # beta and phi rows have matching Stan colnames from their draws
    bf_types <- c("Intercept", "Fixed Effect", "Autoregressive", "Cross-lagged")
    bf_stan_names <- c(beta_stan_names, colnames(draws_phi))

    bf_idx <- which(out$type %in% bf_types)
    for (i in seq_along(bf_idx)) {
      row_i    <- bf_idx[i]
      stan_nm  <- bf_stan_names[i]
      res <- tryCatch(
        savage_dickey(object, params = stan_nm, null_value = null_value,
                      method = "logspline"),
        error = function(e) NULL
      )
      if (!is.null(res)) {
        out$BF01[row_i] <- res$BF01
        out$BF10[row_i] <- res$BF10
      }
    }
  }

  if (!is.null(type))
    out <- out[out$type %in% type, , drop = FALSE]

  out
}

## ---- family vector helpers (D2) ----

#' Does any node have a given family?
#' @keywords internal
#' @noRd
.family_has <- function(object, fam) any(object$family == fam)

#' Which nodes have a given family?
#' @keywords internal
#' @noRd
.family_which <- function(object, fam) which(object$family == fam)

#' Is the model mixed-family?
#' @keywords internal
#' @noRd
.is_mixed <- function(object) length(unique(object$family)) > 1L

#' Format a family vector for display
#' @keywords internal
#' @noRd
.format_family <- function(family_vec) {
  if (length(unique(family_vec)) == 1L) return(unname(family_vec[1]))
  paste0("mixed (", paste(names(family_vec), family_vec,
                          sep = "=", collapse = ", "), ")")
}


## ---- resolve variable names from standata ----
#' @keywords internal
get_param_names <- function(sd) {
  p    <- sd$p
  K    <- sd$K
  n_fe <- sd$n_fe
  n_re <- sd$n_re

  y_names  <- colnames(sd$Y)  %||% paste0("y", seq_len(p))
  fe_names <- colnames(sd$X)  %||% paste0("fe", seq_len(n_fe))
  b_names  <- colnames(sd$B)  %||% paste0("lag", rep(seq_len(K), each = p), "_", rep(y_names, K))
  re_names <- if (n_re > 0) (colnames(sd$Z) %||% paste0("re", seq_len(n_re))) else character(0)

  list(y = y_names, fe = fe_names, b = b_names, re = re_names)
}


## ---- build a labelled data.frame from a draws matrix ----
## draws: iterations x parameters matrix (from extract_draws)
## row_names / col_names map to the [row, col] Stan indices
## column order in draws must follow row-major: (1,1), (2,1), ..., (nr,1), (1,2), ...
#' @keywords internal
build_summary_table <- function(draws, row_names, col_names, type) {
  nr   <- length(row_names)
  nc   <- length(col_names)
  ncol_draws <- ncol(draws)
  stopifnot(ncol_draws == nr * nc)

  d_mean   <- colMeans(draws)
  d_median <- apply(draws, 2, stats::median)
  d_q5     <- apply(draws, 2, stats::quantile, probs = 0.05)
  d_q95    <- apply(draws, 2, stats::quantile, probs = 0.95)

  data.frame(
    type      = rep(type, nr * nc),
    predictor = rep(row_names, times = nc),
    outcome   = rep(col_names, each  = nr),
    mean      = as.numeric(d_mean),
    median    = as.numeric(d_median),
    q5        = as.numeric(d_q5),
    q95       = as.numeric(d_q95),
    stringsAsFactors = FALSE
  )
}


## ---- extract posterior draws as a matrix (Stan column names preserved) ----
#' Extract raw posterior draws for a single parameter block
#'
#' Returns an \code{(iterations * chains)} by \code{params} matrix with
#' Stan-indexed column names (e.g. \code{"beta[1,1]"}, \code{"phi[2,3]"}).
#'
#' @param object A \code{bvarnet} object returned by \code{\link{bvar}}.
#' @param parameter Character. One of \code{"beta"}, \code{"phi"},
#'   \code{"sd_u"}, \code{"sigma"}, or \code{"kappa"}.
#'
#' @return A numeric matrix with one row per posterior draw and one column
#'   per Stan parameter element.
#'
#' @export
extract_draws <- function(object, parameter = c("beta", "phi", "sd_u", "sigma", "kappa")) {
  stopifnot(inherits(object, "bvarnet"))
  parameter <- match.arg(parameter, c("beta", "phi", "sd_u", "sigma", "kappa"))

  if (parameter == "sigma" && !.family_has(object, "gaussian"))
    stop("Parameter 'sigma' only exists for gaussian models.")
  if (parameter == "kappa" && !.family_has(object, "ordinal"))
    stop("Parameter 'kappa' only exists for ordinal models.")
  if (parameter == "sd_u" && object$standata$n_re == 0)
    stop("Parameter 'sd_u' not available \u2014 model has no random effects (n_re = 0).")

  draws <- object$draws                        # 3D array: iter x chains x params
  idx   <- grep(paste0("^", parameter, "\\["), dimnames(draws)[[3]])
  chunk <- draws[, , idx, drop = FALSE]
  # flatten chains into rows
  dim(chunk) <- c(prod(dim(chunk)[1:2]), dim(chunk)[3])
  colnames(chunk) <- dimnames(draws)[[3]][idx]
  chunk
}


# ---- Random-effect draw extraction (u) ----

#' Extract all posterior draws of subject-level random effects \code{u}
#'
#' Returns a 4D array with dimensions
#' \code{[draw, node, subject, re]} matching the Stan declaration
#' \code{array[p] matrix[J, n_re] u}.
#'
#' CmdStan flattens \code{array[p] matrix[J, n_re] u} as
#' \code{u[node, subject, re]} in column-major order within each array
#' element, i.e. \code{u[1,1,1], u[1,1,2], ..., u[1,J,n_re], u[2,1,1], ...}.
#'
#' @param object A \code{bvarnet} object.
#' @return A 4D array with dimensions
#'   \code{[S, p, J, n_re]} where \code{S = n_iter * n_chains}.
#' @keywords internal
#' @noRd
.extract_u_draws <- function(object) {
  stopifnot(inherits(object, "bvarnet"))
  sd <- object$standata
  if (sd$n_re == 0L)
    stop("No random effects in this model (n_re = 0).", call. = FALSE)

  p    <- sd$p
  J    <- sd$J
  n_re <- sd$n_re

  draws <- object$draws   # iter x chains x params
  idx   <- grep("^u\\[", dimnames(draws)[[3]])

  if (length(idx) == 0L)
    stop("No 'u[...]' parameters found in posterior draws.", call. = FALSE)

  # Expected number of u parameters: p * J * n_re
  expected <- p * J * n_re
  if (length(idx) != expected)
    stop(sprintf(
      "Expected %d u parameters (p=%d, J=%d, n_re=%d) but found %d in draws.",
      expected, p, J, n_re, length(idx)
    ), call. = FALSE)

  # Flatten iter x chains
  chunk <- draws[, , idx, drop = FALSE]
  S <- prod(dim(chunk)[1:2])
  dim(chunk) <- c(S, length(idx))

  # CmdStan order for array[p] matrix[J, n_re]:
  #   fastest-varying = re (col of matrix), then subject (row of matrix),
  #   then node (array index).
  # So the flat order is: u[1,1,1], u[1,1,2], ..., u[1,1,n_re],
  #                        u[1,2,1], ..., u[1,J,n_re],
  #                        u[2,1,1], ...
  # Reshape: chunk is S x (p * J * n_re).
  # We want out[draw, node, subject, re].
  out <- array(NA_real_, dim = c(S, p, J, n_re))
  col <- 0L
  for (node in seq_len(p)) {
    for (subj in seq_len(J)) {
      for (re in seq_len(n_re)) {
        col <- col + 1L
        out[, node, subj, re] <- chunk[, col]
      }
    }
  }

  dimnames(out) <- list(
    draw    = NULL,
    node    = colnames(sd$Y),
    subject = seq_len(J),
    re      = if (n_re > 0L && !is.null(colnames(sd$Z))) colnames(sd$Z) else paste0("re", seq_len(n_re))
  )

  out
}


#' Posterior mean of subject-level random effects \code{u}
#'
#' Returns a 3D array \code{[node, subject, re]} containing the
#' posterior mean of each \code{u[node, subject, re]} element.
#'
#' @param object A \code{bvarnet} object.
#' @return A 3D array with dimensions \code{[p, J, n_re]}.
#' @keywords internal
#' @noRd
.posterior_mean_u <- function(object) {
  u_draws <- .extract_u_draws(object)   # [S, p, J, n_re]
  # Average over the draw dimension (dim 1)
  apply(u_draws, c(2, 3, 4), mean)
}


## ---- text formatter for data frames (knitr-safe) ----
# Format a data frame as aligned text lines, bypassing print.data.frame
# which may be intercepted by knitr/rmarkdown in Rmd rendering.
.fmt_df <- function(df, right = FALSE) {
  m    <- format.data.frame(df, na.encode = FALSE)
  flag <- if (right) "" else "-"
  widths <- vapply(seq_along(m), function(j)
    max(nchar(names(m)[j]), max(nchar(m[[j]]))), integer(1))
  hdr <- vapply(seq_along(m), function(j)
    formatC(names(m)[j], width = widths[j], flag = flag), character(1))
  out <- character(nrow(m) + 1L)
  out[1L] <- paste0(" ", paste(hdr, collapse = " "))
  for (i in seq_len(nrow(m))) {
    vals <- vapply(seq_along(m), function(j)
      formatC(m[[j]][i], width = widths[j], flag = flag), character(1))
    out[i + 1L] <- paste0(" ", paste(vals, collapse = " "))
  }
  out
}

## ---- print method for bvarnet objects ----
#' Print a bvarnet model object
#'
#' Displays a brief summary of the fitted model: family, dimensions,
#' Rhat, divergences, chain return codes, priors, and total sampling time.
#'
#' @param x A \code{bvarnet} object.
#' @param ... Ignored.
#'
#' @return \code{x} invisibly.
#' @export
print.bvarnet <- function(x, ...) {
  sd <- x$standata

  cat("BVAR Network fit\n")
  cat(strrep("=", 40), "\n")

  # Family
  cat("Family:      ", .format_family(x$family), "\n", sep = "")

  # Dimensions
  cat("Outcomes (p):", sd$p,    "\n")
  cat("Lags (K):    ", sd$K,    "\n")
  if (!is.null(sd$n_fe)) cat("Fixed eff.:  ", sd$n_fe, "\n")
  if (!is.null(sd$n_re) && sd$n_re > 0)
    cat("Random eff.: ", sd$n_re, "\n")
  cat("Observations:", sd$n_obs, "\n")

  # Convergence
  smry <- x$convergence
  if (!is.null(smry) && "rhat" %in% names(smry)) {
    rhat_max <- max(smry$rhat, na.rm = TRUE)
    cat(sprintf("Rhat max:    %.3f\n", rhat_max))
    if (rhat_max > 1.01)
      cat("  WARNING: Rhat > 1.01 detected \u2014 chains may not have converged.\n")
  }

  diag <- x$diagnostics
  n_div <- if (!is.null(diag) && "num_divergent" %in% names(diag))
    sum(diag$num_divergent) else 0L
  if (n_div > 0)
    cat(sprintf("Divergences: %d  WARNING: check model/priors.\n", n_div))
  else
    cat("Divergences: 0\n")

  # Return codes
  rc <- x$return_codes
  if (!is.null(rc) && any(rc != 0))
    cat(sprintf("Chain status: %d chain(s) returned non-zero exit codes: %s\n",
                sum(rc != 0), paste(which(rc != 0), collapse = ", ")))

  # Priors
  if (!is.null(x$priors) && inherits(x$priors, "bvarnet_priors")) {
    half_pars <- c("sd_u", "sigma")

    # Determine which priors to show
    priors_to_show <- x$priors_needed
    if (is.null(priors_to_show)) {
      # Backward compat: old objects without priors_needed
      has_intercept <- .family_has(x, "gaussian") || .family_has(x, "bernoulli")
      priors_to_show <- if (has_intercept) c("intercept", "beta", "phi") else c("beta", "phi")
      has_re <- !is.null(sd$n_re) && sd$n_re > 0
      if (has_re) priors_to_show <- c(priors_to_show, "sd_u")
      if (.family_has(x, "gaussian")) priors_to_show <- c(priors_to_show, "sigma")
      if (.family_has(x, "ordinal"))  priors_to_show <- c(priors_to_show, "kappa")
    }
    # Filter to slots that actually exist (robustness for serialized objects)
    priors_to_show <- Filter(function(nm) !is.null(x$priors[[nm]]), priors_to_show)

    any_user <- any(!vapply(
      x$priors[priors_to_show],
      function(p) isTRUE(p$is_default), logical(1)
    ))

    if (any_user) {
      # Show each prior on its own line with (default) tags
      cat("Priors:\n")
      for (nm in priors_to_show) {
        half <- nm %in% half_pars
        tag <- if (isTRUE(x$priors[[nm]]$is_default)) "  (default)" else ""
        cat(sprintf("  %-6s ~ %s%s\n", nm, format(x$priors[[nm]], half = half), tag))
      }
    } else {
      # All defaults — compact single line
      prior_str <- paste(
        vapply(priors_to_show, function(nm) {
          half <- nm %in% half_pars
          paste0(nm, " ~ ", format(x$priors[[nm]], half = half))
        }, character(1L)),
        collapse = ", "
      )
      cat("Priors:       ", prior_str, " (all defaults)\n", sep = "")
    }
  }

  # Timing
  t <- x$timing
  if (!is.null(t$total))
    cat(sprintf("Total time:  %.1f sec\n", t$total))

  cat(strrep("=", 40), "\n")
  invisible(x)
}

# ──────────────────────────────────────────────────────────────────────────────
# test-re-extractors.R — tests for .extract_u_draws() and .posterior_mean_u()
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 .extract_u_draws()
# ═══════════════════════════════════════════════════════════════════════════════

test_that(".extract_u_draws returns correct 4D array dimensions", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 2L, J = 5L,
                            n_iter = 20L, n_chains = 2L)
  u <- .extract_u_draws(obj)

  expect_true(is.array(u))
  expect_equal(length(dim(u)), 4L)
  # [draw, node, subject, re]
  S <- 20L * 2L
  expect_equal(dim(u), c(S, 2L, 5L, 2L))
})

test_that(".extract_u_draws has informative dimnames", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 1L, J = 3L)
  u <- .extract_u_draws(obj)

  dn <- dimnames(u)
  expect_named(dn, c("draw", "node", "subject", "re"))
  expect_equal(dn$node, c("y_1", "y_2"))
  expect_equal(dn$subject, as.character(1:3))
  # RE names come from Z colnames
  expect_equal(dn$re, "z_1")
})

test_that(".extract_u_draws errors when n_re = 0", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 0L)
  expect_error(.extract_u_draws(obj), "n_re = 0")
})

test_that(".extract_u_draws recovers deterministic values", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 1L, J = 2L,
                            n_iter = 4L, n_chains = 1L)
  u <- .extract_u_draws(obj)

  # Manually extract u[1,1,1] from draws to verify
  raw <- obj$draws[, , "u[1,1,1]", drop = TRUE]
  flat <- as.vector(raw)
  expect_equal(u[, 1, 1, 1], flat)

  # u[2,2,1]
  raw2 <- obj$draws[, , "u[2,2,1]", drop = TRUE]
  flat2 <- as.vector(raw2)
  expect_equal(u[, 2, 2, 1], flat2)
})

test_that(".extract_u_draws works for all families", {
  for (fam in c("bernoulli", "gaussian", "ordinal")) {
    obj <- make_mock_bvarnet(family = fam, n_re = 1L, J = 3L)
    u <- .extract_u_draws(obj)
    S <- 20L * 2L
    expect_equal(dim(u), c(S, 2L, 3L, 1L),
                 info = paste("family:", fam))
  }
})

test_that(".extract_u_draws works with multiple REs", {
  obj <- make_mock_bvarnet(family = "gaussian", n_re = 3L, J = 4L)
  u <- .extract_u_draws(obj)
  S <- 20L * 2L
  expect_equal(dim(u), c(S, 2L, 4L, 3L))
  expect_equal(dimnames(u)$re, c("z_1", "z_2", "z_3"))
})

# ═══════════════════════════════════════════════════════════════════════════════
# §2 .posterior_mean_u()
# ═══════════════════════════════════════════════════════════════════════════════

test_that(".posterior_mean_u returns correct 3D array", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 2L, J = 5L,
                            n_iter = 20L, n_chains = 2L)
  u_mean <- .posterior_mean_u(obj)

  expect_true(is.array(u_mean))
  expect_equal(length(dim(u_mean)), 3L)
  # [node, subject, re]
  expect_equal(dim(u_mean), c(2L, 5L, 2L))
})

test_that(".posterior_mean_u equals manual column mean", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 1L, J = 2L,
                            n_iter = 10L, n_chains = 1L)
  u_mean <- .posterior_mean_u(obj)

  # Manually compute for u[1,1,1]
  raw <- as.vector(obj$draws[, , "u[1,1,1]", drop = TRUE])
  expect_equal(u_mean[1, 1, 1], mean(raw))

  # u[2,2,1]
  raw2 <- as.vector(obj$draws[, , "u[2,2,1]", drop = TRUE])
  expect_equal(u_mean[2, 2, 1], mean(raw2))
})

test_that(".posterior_mean_u errors when n_re = 0", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 0L)
  expect_error(.posterior_mean_u(obj), "n_re = 0")
})

test_that(".posterior_mean_u has informative dimnames", {
  obj <- make_mock_bvarnet(family = "gaussian", n_re = 2L, J = 3L)
  u_mean <- .posterior_mean_u(obj)

  dn <- dimnames(u_mean)
  expect_equal(dn[[1]], c("y_1", "y_2"))
  expect_equal(dn[[2]], as.character(1:3))
  expect_equal(dn[[3]], c("z_1", "z_2"))
})

# ═══════════════════════════════════════════════════════════════════════════════
# §3 Backward compatibility: make_mock_bvarnet with n_re = 0
# ═══════════════════════════════════════════════════════════════════════════════

test_that("make_mock_bvarnet with n_re=0 still works for existing tests", {
  obj <- make_mock_bvarnet(family = "bernoulli")
  expect_s3_class(obj, "bvarnet")
  expect_equal(obj$standata$n_re, 0L)
  # No u draws
  u_idx <- grep("^u\\[", dimnames(obj$draws)[[3]])
  expect_length(u_idx, 0L)
})

test_that("make_mock_bvarnet with n_re>0 has sd_u draws", {
  obj <- make_mock_bvarnet(family = "bernoulli", n_re = 2L, J = 3L)
  sd_u_idx <- grep("^sd_u\\[", dimnames(obj$draws)[[3]])
  # p=2 nodes, n_re=2 → 4 sd_u parameters

  expect_length(sd_u_idx, 4L)
  # All sd_u draws should be positive
  expect_true(all(obj$draws[, , sd_u_idx] > 0))
})

# ──────────────────────────────────────────────────────────────────────────────
# test-helpers.R — tests for helper functions (no Stan fit required)
#
# Tests get_param_names(), build_summary_table(), and the internal
# data manipulation utilities. extract_draws() requires a real Stan fit
# so is tested separately in test-extract_param.R (skip_on_cran).
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 get_param_names()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("get_param_names returns correct structure with named columns", {
  # Create a mock standata-like list
  sd <- list(
    p = 3, K = 1, n_fe = 2, n_re = 1,
    Y = matrix(0, 10, 3, dimnames = list(NULL, c("y1", "y2", "y3"))),
    X = matrix(0, 10, 2, dimnames = list(NULL, c("Intercept", "x1"))),
    B = matrix(0, 10, 3, dimnames = list(NULL, c("lag1_y1", "lag1_y2", "lag1_y3"))),
    Z = matrix(0, 10, 1, dimnames = list(NULL, c("x1")))
  )

  nm <- bvarnet:::get_param_names(sd)

  expect_type(nm, "list")
  expect_named(nm, c("y", "fe", "b", "re"))
  expect_equal(nm$y, c("y1", "y2", "y3"))
  expect_equal(nm$fe, c("Intercept", "x1"))
  expect_equal(nm$b, c("lag1_y1", "lag1_y2", "lag1_y3"))
  expect_equal(nm$re, "x1")
})


test_that("get_param_names uses fallback names when colnames are NULL", {
  sd <- list(
    p = 2, K = 1, n_fe = 1, n_re = 0,
    Y = matrix(0, 10, 2),  # no colnames
    X = matrix(0, 10, 1),
    B = matrix(0, 10, 2),
    Z = matrix(0, 10, 0)
  )

  nm <- bvarnet:::get_param_names(sd)

  expect_equal(nm$y, c("y1", "y2"))
  expect_equal(nm$fe, "fe1")
  expect_equal(length(nm$re), 0)
})


test_that("get_param_names with n_re = 0 returns empty re vector", {
  sd <- list(p = 2, K = 1, n_fe = 1, n_re = 0,
             Y = matrix(0, 5, 2, dimnames = list(NULL, c("a", "b"))),
             X = matrix(0, 5, 1, dimnames = list(NULL, "Int")),
             B = matrix(0, 5, 2, dimnames = list(NULL, c("l1_a", "l1_b"))),
             Z = matrix(0, 5, 0))

  nm <- bvarnet:::get_param_names(sd)
  expect_equal(nm$re, character(0))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 build_summary_table()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("build_summary_table returns correct data frame structure", {
  set.seed(500)
  draws <- matrix(rnorm(200 * 6), nrow = 200, ncol = 6)
  row_names <- c("pred1", "pred2")
  col_names <- c("out1", "out2", "out3")

  tab <- bvarnet:::build_summary_table(draws, row_names, col_names, "TestType")

  expect_s3_class(tab, "data.frame")
  expect_equal(nrow(tab), 2 * 3)  # nr * nc
  expect_true(all(c("type", "predictor", "outcome",
                     "mean", "median", "q5", "q95") %in% names(tab)))
})


test_that("build_summary_table type column is filled correctly", {
  draws <- matrix(rnorm(100 * 4), nrow = 100, ncol = 4)
  tab <- bvarnet:::build_summary_table(draws, c("a", "b"), c("c", "d"), "MyType")

  expect_true(all(tab$type == "MyType"))
})


test_that("build_summary_table predictor × outcome layout is correct", {
  draws <- matrix(rnorm(100 * 6), nrow = 100, ncol = 6)
  row_names <- c("r1", "r2", "r3")
  col_names <- c("c1", "c2")

  tab <- bvarnet:::build_summary_table(draws, row_names, col_names, "T")

  # Column order: rows cycle within columns
  # (r1,c1), (r2,c1), (r3,c1), (r1,c2), (r2,c2), (r3,c2)
  expect_equal(tab$predictor, rep(row_names, times = 2))
  expect_equal(tab$outcome, rep(col_names, each = 3))
})


test_that("build_summary_table q5 <= median <= q95", {
  set.seed(501)
  draws <- matrix(rnorm(500 * 4), nrow = 500, ncol = 4)
  tab <- bvarnet:::build_summary_table(draws, c("a", "b"), c("c", "d"), "T")

  expect_true(all(tab$q5 <= tab$median))
  expect_true(all(tab$median <= tab$q95))
})


test_that("build_summary_table rejects wrong number of columns", {
  draws <- matrix(rnorm(100 * 5), nrow = 100, ncol = 5)  # 5 columns

  expect_error(
    bvarnet:::build_summary_table(draws, c("a", "b"), c("c", "d"), "T"),
    # should fail: nr*nc = 2*2 = 4 != 5
    "ncol_draws == nr"
  )
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 build_Z()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("build_Z returns empty matrix when no RE specified", {
  X <- matrix(1, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, 3, dimnames = list(NULL, paste0("lag1_y_", 1:3)))

  Z <- bvarnet:::build_Z(X, B)

  expect_equal(ncol(Z), 0)
  expect_equal(nrow(Z), 10)
})


test_that("build_Z with re_cols selects correct X columns", {
  X <- matrix(rnorm(30), 10, 3,
              dimnames = list(NULL, c("Intercept", "x_1", "x_2")))
  B <- matrix(0, 10, 2)

  Z <- bvarnet:::build_Z(X, B, re_cols = c("x_1", "x_2"))

  expect_equal(ncol(Z), 2)
  expect_equal(Z[, 1], X[, "x_1"])
  expect_equal(Z[, 2], X[, "x_2"])
})


test_that("build_Z with re_temporal includes all B columns", {
  X <- matrix(1, 10, 1, dimnames = list(NULL, "Intercept"))
  B <- matrix(rnorm(20), 10, 2,
              dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  Z <- bvarnet:::build_Z(X, B, re_temporal = TRUE)

  expect_equal(ncol(Z), 2)
  expect_equal(Z[, 1], B[, 1])
  expect_equal(Z[, 2], B[, 2])
})


test_that("build_Z rejects invalid re_cols", {
  X <- matrix(1, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, 2)

  expect_error(
    bvarnet:::build_Z(X, B, re_cols = "missing_col"),
    "re_cols not found"
  )
})


test_that("build_Z creates Intercept column when absent from X (ordinal case)", {
  # Ordinal X has no Intercept column (absorbed by kappa cutpoints)
  X <- matrix(rnorm(20), 10, 2, dimnames = list(NULL, c("x_1", "x_2")))
  B <- matrix(0, 10, 3)

  Z <- bvarnet:::build_Z(X, B, re_cols = "Intercept")

  expect_equal(ncol(Z), 1)
  expect_equal(nrow(Z), 10)
  expect_true(all(Z[, 1] == 1))
  expect_equal(colnames(Z), "Intercept")
})


test_that("build_Z creates Intercept + selects other re_cols when Intercept absent", {
  X <- matrix(rnorm(20), 10, 2, dimnames = list(NULL, c("x_1", "x_2")))
  B <- matrix(0, 10, 3)

  Z <- bvarnet:::build_Z(X, B, re_cols = c("Intercept", "x_1"))

  expect_equal(ncol(Z), 2)
  expect_true(all(Z[, "Intercept"] == 1))
  expect_equal(Z[, "x_1"], X[, "x_1"])
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 make_term_matrix()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("make_term_matrix creates simple interaction from X columns", {
  X <- matrix(c(rep(1, 5), 1:5, 6:10), 5, 3,
              dimnames = list(NULL, c("Intercept", "x_1", "x_2")))
  B <- matrix(0, 5, 2)

  M <- bvarnet:::make_term_matrix(X, B, c("x_1", "x_2"))

  expect_equal(ncol(M), 1)
  expect_equal(colnames(M), "x_1:x_2")
  expect_equal(M[, 1], X[, "x_1"] * X[, "x_2"])
})


test_that("make_term_matrix with 'lag' expands across B columns", {
  X <- matrix(c(rep(1, 5), 1:5), 5, 2,
              dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(rnorm(10), 5, 2,
              dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))

  M <- bvarnet:::make_term_matrix(X, B, c("lag", "x_1"))

  expect_equal(ncol(M), 2)
  expect_equal(colnames(M), c("lag1_y_1:x_1", "lag1_y_2:x_1"))
  expect_equal(M[, 1], B[, 1] * X[, "x_1"])
})


# ═══════════════════════════════════════════════════════════════════════════════
# §N — print.bvarnet() priors display
# ═══════════════════════════════════════════════════════════════════════════════

test_that("print.bvarnet() shows 'all defaults' when no priors are user-set", {
  obj <- make_mock_bvarnet()
  out <- capture.output(print(obj))
  expect_true(any(grepl("all defaults", out)))
})

test_that("print.bvarnet() includes 'phi' in priors summary", {
  obj <- make_mock_bvarnet()
  expect_output(print(obj), "phi")
})

test_that("print.bvarnet() includes 'Half-' for sd_u in priors when model has REs", {
  obj <- make_mock_bvarnet(n_re = 1L)
  expect_output(print(obj), "Half-")
})

test_that("print.bvarnet() omits sd_u from priors when model has no REs", {
  obj <- make_mock_bvarnet()
  out <- capture.output(print(obj))
  priors_line <- out[grepl("Priors", out)]
  expect_false(any(grepl("sd_u", priors_line)))
})

test_that("print.bvarnet() shows 'sigma' in priors for gaussian family", {
  obj <- make_mock_bvarnet(family = "gaussian")
  expect_output(print(obj), "sigma")
})

test_that("print.bvarnet() shows 'kappa' in priors for ordinal family", {
  obj <- make_mock_bvarnet(family = "ordinal")
  expect_output(print(obj), "kappa")
})

test_that("print.bvarnet() omits sigma for bernoulli family", {
  obj <- make_mock_bvarnet(family = "bernoulli")
  out <- capture.output(print(obj))
  expect_false(any(grepl("sigma", out)))
})

test_that("print.bvarnet() shows (default) tag for user-set priors", {
  obj <- make_mock_bvarnet(family = "gaussian")
  obj$priors$beta <- prior("cauchy", 0, 2)
  out <- capture.output(print(obj))
  # beta should NOT have (default) tag
  beta_line <- out[grepl("beta", out)]
  expect_false(any(grepl("\\(default\\)", beta_line)))
  # phi and sigma should have (default) tag
  phi_line <- out[grepl("phi", out)]
  expect_true(any(grepl("\\(default\\)", phi_line)))
  sigma_line <- out[grepl("sigma", out)]
  expect_true(any(grepl("\\(default\\)", sigma_line)))
})

test_that("print.bvarnet() backward compat: old objects without priors_needed still work", {
  obj <- make_mock_bvarnet(family = "gaussian")
  obj$priors_needed <- NULL
  out <- capture.output(print(obj))
  expect_true(any(grepl("beta", out)))
  expect_true(any(grepl("sigma", out)))
})

# test-parse_family.R — unit tests for .parse_family() and family helpers
# ──────────────────────────────────────────────────────────────────────────────

test_that(".parse_family() recycles scalar to named vector", {
  y_cols <- c("y_1", "y_2", "y_3")
  result <- .parse_family("gaussian", y_cols)
  expect_length(result, 3L)
  expect_equal(names(result), y_cols)
  expect_true(all(result == "gaussian"))
})

test_that(".parse_family() accepts named vector (reordered to y_cols)", {
  y_cols <- c("y_1", "y_2")
  fam <- c(y_2 = "ordinal", y_1 = "bernoulli")
  result <- .parse_family(fam, y_cols)
  expect_equal(as.character(result), c("bernoulli", "ordinal"))
  expect_equal(names(result), y_cols)
})

test_that(".parse_family() accepts unnamed vector of length p", {
  y_cols <- c("y_1", "y_2")
  result <- .parse_family(c("gaussian", "bernoulli"), y_cols)
  expect_equal(as.character(result), c("gaussian", "bernoulli"))
  expect_equal(names(result), y_cols)
})

test_that(".parse_family() errors on invalid family string", {
  expect_error(.parse_family("poisson", c("y_1", "y_2")))
})

test_that(".parse_family() errors on wrong length", {
  expect_error(.parse_family(c("gaussian", "bernoulli", "ordinal"), c("y_1", "y_2")))
})

test_that(".parse_family() errors on mismatched names", {
  fam <- c(y_1 = "gaussian", y_3 = "bernoulli")
  expect_error(.parse_family(fam, c("y_1", "y_2")))
})

# ── family helpers ──────────────────────────────────────────────────────────

test_that(".family_has() detects families in vector", {
  obj <- list(family = c(y_1 = "gaussian", y_2 = "ordinal"))
  expect_true(.family_has(obj, "gaussian"))
  expect_true(.family_has(obj, "ordinal"))
  expect_false(.family_has(obj, "bernoulli"))
})

test_that(".family_which() returns correct indices", {
  obj <- list(family = c(y_1 = "gaussian", y_2 = "ordinal", y_3 = "gaussian"))
  expect_equal(unname(.family_which(obj, "gaussian")), c(1L, 3L))
  expect_equal(unname(.family_which(obj, "ordinal")), 2L)
})

test_that(".is_mixed() distinguishes mixed from homogeneous", {
  mixed <- list(family = c(y_1 = "gaussian", y_2 = "bernoulli"))
  homo  <- list(family = c(y_1 = "gaussian", y_2 = "gaussian"))
  expect_true(.is_mixed(mixed))
  expect_false(.is_mixed(homo))
})

test_that(".format_family() formats correctly", {
  fam_scalar <- c(y_1 = "gaussian", y_2 = "gaussian")
  expect_equal(.format_family(fam_scalar), "gaussian")

  fam_mixed <- c(y_1 = "gaussian", y_2 = "ordinal")
  result <- .format_family(fam_mixed)
  expect_true(grepl("gaussian", result))
  expect_true(grepl("ordinal", result))
})

# ════──────────────────────────────────────────────────────────────────────────
# test-bvarnet-object.R — tests for the bvarnet object structure
#
# Covers: object slots, draws array properties, extract_draws() subsetting,
# and print.bvarnet().
# make_mock_bvarnet() is defined in helper-fixtures.R and auto-sourced.
# extract_draws() is internal; the helper always sources local R files so
# it is available in the global env as plain extract_draws().
# ════──────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 — Object structure
# ═══════════════════════════════════════════════════════════════════════════════

test_that("bvarnet object has class 'bvarnet'", {
  obj <- make_mock_bvarnet()
  expect_s3_class(obj, "bvarnet")
})


test_that("bvarnet object contains expected top-level names", {
  obj <- make_mock_bvarnet()
  expected <- c("draws", "convergence", "diagnostics", "timing",
                "metadata", "return_codes", "family", "standata", "priors")
  expect_true(all(expected %in% names(obj)))
})


test_that("bvarnet object does not contain a 'fit' field", {
  obj <- make_mock_bvarnet()
  expect_null(obj$fit)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 — draws slot
# ═══════════════════════════════════════════════════════════════════════════════

test_that("draws slot is a plain base-R array", {
  obj <- make_mock_bvarnet()
  expect_true(is.array(obj$draws))
  expect_false(inherits(obj$draws, "draws_array"))
  expect_identical(class(obj$draws), "array")
})


test_that("draws slot has 3 dimensions", {
  obj <- make_mock_bvarnet()
  expect_equal(length(dim(obj$draws)), 3L)
})


test_that("draws dimnames[[3]] is a non-null character vector", {
  obj <- make_mock_bvarnet()
  dn3 <- dimnames(obj$draws)[[3]]
  expect_false(is.null(dn3))
  expect_true(is.character(dn3))
  expect_true(length(dn3) > 0)
})


test_that("draws parameter names are in Stan bracket format", {
  obj <- make_mock_bvarnet()
  dn3 <- dimnames(obj$draws)[[3]]
  expect_true(all(grepl("^[a-z_]+\\[[0-9,]+\\]$", dn3)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 — convergence slot
# ═══════════════════════════════════════════════════════════════════════════════

test_that("convergence slot is a plain data.frame (not tibble)", {
  obj <- make_mock_bvarnet()
  expect_identical(class(obj$convergence), "data.frame")
})


test_that("convergence slot has required convergence columns", {
  obj  <- make_mock_bvarnet()
  cols <- names(obj$convergence)
  expect_true("variable"  %in% cols)
  expect_true("rhat"      %in% cols)
  expect_true("ess_bulk"  %in% cols)
  expect_true("ess_tail"  %in% cols)
})


test_that("convergence slot has one row per parameter", {
  obj <- make_mock_bvarnet("bernoulli")
  expect_equal(nrow(obj$convergence), dim(obj$draws)[3])
})


test_that("convergence rhat values are numeric", {
  obj <- make_mock_bvarnet()
  expect_true(is.numeric(obj$convergence$rhat))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 — diagnostics slot
# ═══════════════════════════════════════════════════════════════════════════════

test_that("diagnostics slot is a data.frame", {
  obj <- make_mock_bvarnet()
  expect_true(is.data.frame(obj$diagnostics))
})


test_that("diagnostics slot has required columns", {
  obj  <- make_mock_bvarnet()
  cols <- names(obj$diagnostics)
  expect_true("num_divergent"     %in% cols)
  expect_true("num_max_treedepth" %in% cols)
  expect_true("ebfmi"             %in% cols)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §5 — extract_draws() base-R subsetting
# ═══════════════════════════════════════════════════════════════════════════════

test_that("extract_draws returns a matrix with Stan column names", {
  obj    <- make_mock_bvarnet("bernoulli")
  result <- extract_draws(obj, "beta")

  expect_true(is.matrix(result))
  expect_true(all(grepl("^beta\\[", colnames(result))))
})


test_that("extract_draws returns correct row count (iter * chains)", {
  n_iter <- 20L; n_chains <- 2L
  obj    <- make_mock_bvarnet("bernoulli", n_iter = n_iter, n_chains = n_chains)
  result <- extract_draws(obj, "beta")

  expect_equal(nrow(result), n_iter * n_chains)
})


test_that("extract_draws returns correct column count for beta", {
  obj    <- make_mock_bvarnet("bernoulli")  # p=2, n_fe=2 → 2*2=4 beta params
  result <- extract_draws(obj, "beta")

  expect_equal(ncol(result), 4L)
})


test_that("extract_draws for 'beta' does not include phi, sigma, or kappa columns", {
  obj    <- make_mock_bvarnet("gaussian")
  result <- extract_draws(obj, "beta")

  expect_false(any(grepl("^phi\\[",   colnames(result))))
  expect_false(any(grepl("^sigma\\[", colnames(result))))
})


test_that("extract_draws for sigma returns correct column count", {
  obj    <- make_mock_bvarnet("gaussian")   # p=2 → 2 sigma params
  result <- extract_draws(obj, "sigma")

  expect_equal(ncol(result), 2L)
  expect_true(all(grepl("^sigma\\[", colnames(result))))
})


test_that("extract_draws for kappa returns correct column count", {
  obj    <- make_mock_bvarnet("ordinal")    # p=2, C-1=2 → 4 kappa params
  result <- extract_draws(obj, "kappa")

  expect_equal(ncol(result), 4L)
  expect_true(all(grepl("^kappa\\[", colnames(result))))
})


test_that("extract_draws column values match expected draws slice", {
  obj  <- make_mock_bvarnet("bernoulli")
  res  <- extract_draws(obj, "beta")
  drws <- obj$draws

  # Manually replicate what extract_draws should do
  idx   <- grep("^beta\\[", dimnames(drws)[[3]])
  chunk <- drws[, , idx, drop = FALSE]
  dim(chunk) <- c(prod(dim(chunk)[1:2]), dim(chunk)[3])
  colnames(chunk) <- dimnames(drws)[[3]][idx]

  expect_equal(res, chunk)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §6 — print.bvarnet()
# ═══════════════════════════════════════════════════════════════════════════════

test_that("print.bvarnet outputs model family", {
  obj <- make_mock_bvarnet("gaussian")
  expect_output(print(obj), "gaussian")
})


test_that("print.bvarnet outputs dimension information", {
  obj <- make_mock_bvarnet("bernoulli")
  expect_output(print(obj), "2")    # p = 2 appears
})


test_that("print.bvarnet outputs convergence information", {
  obj <- make_mock_bvarnet()
  expect_output(print(obj), "Rhat")
})


test_that("print.bvarnet outputs divergence information", {
  obj <- make_mock_bvarnet()
  expect_output(print(obj), "Divergences")
})


test_that("print.bvarnet outputs timing", {
  obj <- make_mock_bvarnet()
  expect_output(print(obj), "5")   # timing$total = 5.0
})


test_that("print.bvarnet warns when divergences > 0", {
  obj <- make_mock_bvarnet()
  obj$diagnostics$num_divergent <- c(3L, 0L)
  expect_output(print(obj), "WARNING")
})


test_that("print.bvarnet warns when Rhat > 1.01", {
  obj <- make_mock_bvarnet()
  obj$convergence$rhat <- rep(1.05, nrow(obj$convergence))
  expect_output(print(obj), "WARNING")
})


test_that("print.bvarnet returns object invisibly", {
  obj <- make_mock_bvarnet()
  ret <- withVisible(print(obj))
  expect_false(ret$visible)
  expect_identical(ret$value, obj)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §7 — priors slot
# ═══════════════════════════════════════════════════════════════════════════════

test_that("priors slot is present on mock bvarnet object", {
  obj <- make_mock_bvarnet()
  expect_true("priors" %in% names(obj))
})

test_that("priors slot inherits bvarnet_priors", {
  obj <- make_mock_bvarnet()
  expect_s3_class(obj$priors, "bvarnet_priors")
})

test_that("priors slot has correct default beta", {
  obj <- make_mock_bvarnet()
  expect_equal(obj$priors$beta$family_int, 1L)
  expect_equal(obj$priors$beta$loc,   0)
  expect_equal(obj$priors$beta$scale, 1)
})

test_that("priors slot has all expected named entries", {
  obj <- make_mock_bvarnet()
  expect_named(obj$priors, c("intercept", "beta", "phi", "sd_u", "kappa", "sigma"))
})

# Extracted from test-simulate-predict.R:386

# setup ------------------------------------------------------------------------
library(testthat)
test_env <- simulate_test_env(package = "bvarnet", path = "..")
attach(test_env, warn.conflicts = FALSE)

# prequel ----------------------------------------------------------------------
make_predictable_mock <- function(family = "gaussian", n_re = 0L, J = 5L,
                                   T_obs = 20L, p = 2L, q = 1L, K = 1L) {
  set.seed(99L)
  N <- J
  # Make long-format data
  df <- expand.grid(t = seq_len(T_obs), id = seq_len(N))
  df <- df[order(df$id, df$t), ]
  for (j in seq_len(p)) {
    col <- paste0("y_", j)
    if (family == "bernoulli")     df[[col]] <- rbinom(nrow(df), 1, 0.5)
    else if (family == "gaussian") df[[col]] <- rnorm(nrow(df))
    else                           df[[col]] <- sample(1:3, nrow(df), replace = TRUE)
  }
  for (j in seq_len(q)) df[[paste0("x_", j)]] <- rnorm(nrow(df))

  y_cols <- paste0("y_", seq_len(p))
  x_cols <- if (q > 0) paste0("x_", seq_len(q)) else character(0)

  # Use to_stan_data to get proper matrices
  sd <- to_stan_data(
    data     = df,
    family   = family,
    id_col   = "id",
    time_col = "t",
    y_cols   = y_cols,
    x_cols   = x_cols,
    K        = K,
    re_cols  = if (n_re > 0L) "Intercept" else character(0),
    re_temporal = (n_re > 1L)
  )

  n_fe   <- sd$n_fe
  actual_n_re <- sd$n_re
  PK     <- p * K
  n_iter <- 20L; n_chains <- 2L

  # Build draws array
  beta_nm <- character(0)
  for (node in seq_len(p)) for (fe in seq_len(n_fe))
    beta_nm <- c(beta_nm, sprintf("beta[%d,%d]", fe, node))
  phi_nm <- character(0)
  for (node in seq_len(p)) for (lag_idx in seq_len(PK))
    phi_nm <- c(phi_nm, sprintf("phi[%d,%d]", lag_idx, node))
  par_nms <- c(beta_nm, phi_nm)

  if (family == "gaussian")
    par_nms <- c(par_nms, paste0("sigma[", seq_len(p), "]"))
  if (family == "ordinal") {
    C <- sd$C
    for (node in seq_len(p)) for (k in seq_len(C - 1L))
      par_nms <- c(par_nms, sprintf("kappa[%d,%d]", node, k))
  }
  if (actual_n_re > 0L) {
    for (node in seq_len(p)) for (re in seq_len(actual_n_re))
      par_nms <- c(par_nms, sprintf("sd_u[%d,%d]", node, re))
    for (node in seq_len(p)) for (subj in seq_len(J)) for (re in seq_len(actual_n_re))
      par_nms <- c(par_nms, sprintf("u[%d,%d,%d]", node, subj, re))
  }

  n_par <- length(par_nms)
  draws <- array(rnorm(n_iter * n_chains * n_par, mean = 0.1, sd = 0.3),
                 dim = c(n_iter, n_chains, n_par),
                 dimnames = list(NULL, NULL, par_nms))

  # Make sigma positive, sd_u positive
  sigma_idx <- grep("^sigma\\[", par_nms)
  if (length(sigma_idx) > 0) draws[, , sigma_idx] <- abs(draws[, , sigma_idx]) + 0.1
  sd_u_idx <- grep("^sd_u\\[", par_nms)
  if (length(sd_u_idx) > 0) draws[, , sd_u_idx] <- abs(draws[, , sd_u_idx]) + 0.1

  # Make kappa ordered
  if (family == "ordinal") {
    for (node in seq_len(p)) {
      k_idx <- grep(sprintf("^kappa\\[%d,", node), par_nms)
      for (ch in 1:n_chains) {
        for (it in 1:n_iter) {
          vals <- draws[it, ch, k_idx]
          draws[it, ch, k_idx] <- sort(vals)
        }
      }
    }
  }

  smry <- data.frame(
    variable = par_nms, rhat = 1.001,
    ess_bulk = 3000, ess_tail = 2800, stringsAsFactors = FALSE
  )

  structure(
    list(
      draws        = draws,
      convergence  = smry,
      diagnostics  = data.frame(num_divergent = integer(n_chains),
                                num_max_treedepth = integer(n_chains),
                                ebfmi = rep(1.0, n_chains)),
      timing       = list(total = 5.0),
      metadata     = list(),
      return_codes = rep(0L, n_chains),
      family       = family,
      standata     = sd,
      priors       = set_priors()
    ),
    class = "bvarnet"
  )
}

# test -------------------------------------------------------------------------
skip_if_not(instantiate::stan_cmdstan_exists())
sim <- sim_var(N = 5, T_obs = 30, p = 2, K = 1,
                 family = "gaussian", q = 0, seed = 1)
fit <- bvar(id_col = "id", time_col = "t",
              y_cols = c("y_1", "y_2"), x_cols = character(0),
              data = sim$data, family = "gaussian",
              iter = 200, warmup = 100, chains = 2, seed = 1)

#' Fit a Bayesian multilevel VAR network model
#'
#' The `bvar` function estimates the posterior distribution of the specified Bayesian (Multilevel) Vector Autoregression.
#'
#' @param id_col Character. Name of the subject/group identifier column.
#' @param time_col Character. Name of the time column.
#' @param y_cols Character vector. Names of the outcome columns.
#' @param x_cols Character vector or NULL. Names of the covariate columns.
#' @param center_x Logical. Grand-mean centre covariates before fitting?
#'   Default \code{FALSE}.
#' @param fe_interactions List or NULL. Fixed-effect interaction terms to add
#'   to the design matrix. Each element is a character vector of column names
#'   to interact, or \code{c("lag", "x")} to interact all lag columns with
#'   a covariate.
#' @param re_interactions List or NULL. Random-effect interaction terms.
#' @param re_cols Character vector. Columns from X and/or "Intercept" to include as random slopes.
#' @param re_temporal Logical. Include random slopes on lag predictors?
#'   Default \code{FALSE}.
#' @param K Integer. AR order. Default 1.
#' @param na_action Character. Missing-data strategy; currently only
#'   \code{"listwise"}.
#' @param skip_lag Logical. If \code{TRUE} (default), rows with irregular time
#'   gaps have their lag set to zero rather than being dropped.
#' @param data Data frame in long format.
#' @param family Character scalar or vector. Observation model per node.
#'   A scalar is recycled to all \code{y_cols}. A vector of length
#'   \code{length(y_cols)} (named or positional) specifies per-node families.
#'   Valid values: \code{"bernoulli"}, \code{"ordinal"}, \code{"gaussian"}.
#' @param priors A \code{bvarnet_priors} object from \code{set_priors()}.
#'   Defaults to \code{set_priors()} (package defaults).
#' @param iter Integer. Number of post-warmup iterations per chain. Default 4000.
#' @param warmup Integer. Number of warmup iterations per chain. Default 1000.
#' @param chains Integer. Number of MCMC chains. Default 4.
#' @param cores Integer. Number of chains to run in parallel. Default 1.
#' @param seed Integer or NULL. RNG seed.
#' @param adapt_delta Numeric in (0, 1). Target average proposal acceptance
#'   probability during warmup adaptation. Higher values (e.g., 0.95–0.99)
#'   reduce divergences at the cost of slower sampling. Default \code{NULL}
#'   (CmdStan default of 0.8).
#' @param max_treedepth Integer. Maximum depth of the NUTS binary tree.
#'   Increasing this allows the sampler to take more leapfrog steps per
#'   iteration, which can help with difficult posteriors (e.g., funnels in
#'   hierarchical logistic models) but increases computation. Default
#'   \code{NULL} (CmdStan default of 10).
#' @param save_data Logical. If \code{TRUE}, store the preprocessed (sorted,
#'   listwise-deleted) estimation data in the \code{data_used} slot of the
#'   returned object for reproducibility and downstream analyses.
#'   Default \code{FALSE}.
#'
#' @return A \code{bvarnet} object (a named list) with slots:
#'   \code{draws}, \code{convergence}, \code{diagnostics}, \code{timing},
#'   \code{metadata}, \code{return_codes}, \code{family}, \code{standata},
#'   \code{priors}. If \code{save_data = TRUE}, also includes
#'   \code{data_used} (the cleaned estimation data frame).
#'
#' @examples 
#' \dontrun{
#' # Run bvar on studentlife data
#' data(studentlife, package = "bvarnet")
#' fit <- bvar(
#'   id_col = "id",
#'   time_col = "time",
#'   y_cols = c("anxious", "calm", "conventional", "critical", "dependable"),
#'   re_temporal = TRUE,
#'   K = 1,
#'   data = studentlife,
#'   family = "ordinal",
#'   priors = set_priors(),
#'   seed = 1337)
#'
#' summary(fit)
#' }

#' @export
bvar <- function(id_col,
                 time_col,
                 y_cols,
                 x_cols = NULL,
                 center_x = FALSE,
                 fe_interactions = NULL,
                 re_interactions = NULL,
                 re_cols = NULL,
                 re_temporal = FALSE,
                 K = 1,
                 na_action = c("listwise"),
                 skip_lag = TRUE,
                 data,
                 family = c("bernoulli", "ordinal", "gaussian"),
                 priors = set_priors(),
                 iter = 4000,
                 warmup = 1000,
                 chains = 4,
                 cores = 1,
                 seed = NULL,
                 adapt_delta = NULL,
                 max_treedepth = NULL,
                 save_data = FALSE

  ) {

  family_vec <- .parse_family(family, y_cols)
  is_mixed   <- length(unique(family_vec)) > 1L

  # Validate and ensure all required prior slots exist
  if (!inherits(priors, "bvarnet_priors"))
    stop("'priors' must be a bvarnet_priors object from set_priors().",
         call. = FALSE)
  priors <- .ensure_prior_slots(priors, family_vec)

  if (is_mixed) {
    return(.bvar_nodewise(
      id_col = id_col, time_col = time_col, y_cols = y_cols,
      x_cols = x_cols, center_x = center_x,
      fe_interactions = fe_interactions,
      re_interactions = re_interactions,
      re_cols = re_cols, re_temporal = re_temporal,
      K = K, na_action = na_action, skip_lag = skip_lag,
      data = data, family_vec = family_vec, priors = priors,
      iter = iter, warmup = warmup, chains = chains, cores = cores,
      seed = seed, adapt_delta = adapt_delta, max_treedepth = max_treedepth,
      save_data = save_data
    ))
  }

  # --- existing joint path (homogeneous family) ---
  family <- unname(family_vec[1])  # scalar for backward compat
  model_name <- switch(family,
                       bernoulli = "model_binary",
                       ordinal   = "model_ordinal",
                       gaussian  = "model_gaussian",
                       stop("Unknown family: ", family)
  )
  stanmodel <- instantiate::stan_package_model(name = model_name, package = "bvarnet")

  standata <- to_stan_data(data = data,
                            family = family,
                            id_col = id_col,
                            time_col = time_col,
                            y_cols = y_cols,
                            x_cols = x_cols,
                            center_x = center_x,
                            fe_interactions = fe_interactions,
                            re_interactions = re_interactions,
                            re_cols = re_cols,
                            re_temporal = re_temporal,
                            K = K,
                            na_action = na_action,
                            skip_lag = skip_lag,
                            priors = priors,
                            save_data = save_data
                           )

  # Prior warnings (uses actual n_re from built design)
  priors_needed <- .prior_warnings(priors, family_vec, standata$n_re)

  stanfit <- stanmodel$sample(data = standata[!names(standata) %in%
                                      c("fe_interaction_terms",
                                        "fe_interaction_colnames",
                                        "id_levels",
                                        "x_center_means",
                                        "time_obs",
                                        "data_used",
                                        "design_spec")],
                              seed = seed,
                              iter_warmup = warmup,
                              iter_sampling = iter,
                              chains = chains,
                              parallel_chains = cores,
                              adapt_delta = adapt_delta,
                              max_treedepth = max_treedepth)

  # Extract everything from CmdStanMCMC into plain base-R objects, then discard
  # the fit object (CSV refs, compiled binary, lazy draws) to keep memory lean.
  raw_draws   <- stanfit$draws(format = "array")
  draws       <- unclass(raw_draws)         # strip draws_array class; dimnames preserved
  attr(draws, "class") <- NULL              # ensure it is a plain array

  # Compute convergence diagnostics from posterior draws (lightweight).
  conv_tbl    <- posterior::summarise_draws(raw_draws,
                   posterior::rhat, posterior::ess_bulk, posterior::ess_tail)
  convergence <- as.data.frame(conv_tbl)
  names(convergence) <- gsub("^posterior::", "", names(convergence))

  diagnostics  <- as.data.frame(stanfit$diagnostic_summary())
  timing       <- stanfit$time()
  metadata     <- stanfit$metadata()
  return_codes <- stanfit$return_codes()

  out <- structure(
    list(
      draws         = draws,
      convergence   = convergence,
      diagnostics   = diagnostics,
      timing        = timing,
      metadata      = metadata,
      return_codes  = return_codes,
      family        = family_vec,
      standata      = standata,
      priors        = priors,
      priors_needed = priors_needed,
      data_used     = standata$data_used
    ),
    class = "bvarnet"
  )
  out
}


# ---- .parse_family() — validate & normalise family to named character vector ----

#' Parse and validate the family argument
#'
#' Returns a named character vector of length \code{p} with names from
#' \code{y_cols}.
#'
#' @param family Character scalar or vector.
#' @param y_cols Character vector of outcome names.
#' @return Named character vector.
#' @keywords internal
#' @noRd
.parse_family <- function(family, y_cols) {
  valid <- c("bernoulli", "ordinal", "gaussian")
  p <- length(y_cols)

  if (length(family) == 1L && is.null(names(family))) {
    family <- match.arg(family, valid)
    out <- stats::setNames(rep(family, p), y_cols)
  } else if (length(family) == p && is.null(names(family))) {
    bad <- setdiff(family, valid)
    if (length(bad) > 0L)
      stop("Invalid family value(s): ", paste(bad, collapse = ", "),
           ". Must be one of: ", paste(valid, collapse = ", "), call. = FALSE)
    out <- stats::setNames(family, y_cols)
  } else if (!is.null(names(family))) {
    if (!setequal(names(family), y_cols))
      stop("Names of 'family' must match y_cols: ",
           paste(y_cols, collapse = ", "), call. = FALSE)
    bad <- setdiff(family, valid)
    if (length(bad) > 0L)
      stop("Invalid family value(s): ", paste(bad, collapse = ", "),
           ". Must be one of: ", paste(valid, collapse = ", "), call. = FALSE)
    out <- family[y_cols]  # reorder to match y_cols
  } else {
    stop("'family' must be a scalar, a length-p vector, or a named vector ",
         "with names matching y_cols.", call. = FALSE)
  }
  out
}


# ---- .bvar_nodewise() — fit each node independently ----

#' @keywords internal
.bvar_nodewise <- function(id_col, time_col, y_cols, x_cols,
                           center_x, fe_interactions, re_interactions,
                           re_cols, re_temporal, K, na_action, skip_lag,
                           data, family_vec, priors,
                           iter, warmup, chains, cores,
                           seed, adapt_delta, max_treedepth,
                           save_data = FALSE) {
  p <- length(y_cols)

  # --- Shared matrices (D3) ---
  shared <- .to_stan_data_shared(
    data = data, id_col = id_col, time_col = time_col,
    y_cols = y_cols, x_cols = x_cols, center_x = center_x,
    fe_interactions = fe_interactions, re_interactions = re_interactions,
    re_cols = re_cols, re_temporal = re_temporal, K = K,
    na_action = na_action, skip_lag = skip_lag, save_data = save_data
  )

  # Prior warnings (uses actual n_re from built design)
  priors_needed <- .prior_warnings(priors, family_vec, shared$n_re)

  # --- Fit each node ---
  fits <- vector("list", p)
  for (j in seq_len(p)) {
    fam_j      <- family_vec[j]
    model_name <- switch(fam_j,
      bernoulli = "model_binary",
      ordinal   = "model_ordinal",
      gaussian  = "model_gaussian"
    )
    stanmodel <- instantiate::stan_package_model(
      name = model_name, package = "bvarnet"
    )
    sd_node <- .to_stan_data_node(shared, j, fam_j, priors)

    fits[[j]] <- stanmodel$sample(
      data            = sd_node,
      seed            = seed,
      iter_warmup     = warmup,
      iter_sampling   = iter,
      chains          = chains,
      parallel_chains = min(cores, chains),
      adapt_delta     = adapt_delta,
      max_treedepth   = max_treedepth
    )
  }

  # --- Combine into bvarnet object ---
  .combine_nodewise_fits(fits, family_vec, shared, priors, priors_needed,
                         iter, chains)
}


# ---- .combine_nodewise_fits() — merge per-node fits into one bvarnet object ----

#' @keywords internal
.combine_nodewise_fits <- function(fits, family_vec, shared, priors,
                                   priors_needed, iter, chains) {
  p <- length(family_vec)
  n_fe_full <- shared$n_fe   # includes Intercept

  # --- a) Combined draws array ---
  draw_chunks <- vector("list", p)
  lp_chunks   <- vector("list", p)
  conv_chunks <- vector("list", p)

  for (j in seq_len(p)) {
    raw_draws <- fits[[j]]$draws(format = "array")
    arr <- unclass(raw_draws)
    attr(arr, "class") <- NULL
    par_names <- dimnames(arr)[[3]]

    # Extract and remove lp__
    lp_idx <- which(par_names == "lp__")
    if (length(lp_idx) > 0L) {
      lp_chunks[[j]] <- arr[, , lp_idx, drop = TRUE]  # [iter, chains]
      arr <- arr[, , -lp_idx, drop = FALSE]
      par_names <- par_names[-lp_idx]
    }

    # Rename parameters from p=1 to full-model indexing
    fam_j <- family_vec[j]
    new_names <- character(length(par_names))
    for (k in seq_along(par_names)) {
      new_names[k] <- .rename_node_param(par_names[k], j, fam_j, n_fe_full)
    }
    dimnames(arr)[[3]] <- new_names

    # For ordinal: insert NA sentinel for beta[1, j]
    if (fam_j == "ordinal") {
      sentinel_name <- sprintf("beta[1,%d]", j)
      sentinel_arr <- array(NA_real_,
                            dim = c(dim(arr)[1], dim(arr)[2], 1L),
                            dimnames = list(NULL, NULL, sentinel_name))
      arr <- abind_simple(sentinel_arr, arr)
    }

    draw_chunks[[j]] <- arr

    # Convergence: rename variables
    conv_tbl <- posterior::summarise_draws(raw_draws,
                  posterior::rhat, posterior::ess_bulk, posterior::ess_tail)
    conv_df <- as.data.frame(conv_tbl)
    names(conv_df) <- gsub("^posterior::", "", names(conv_df))
    conv_df <- conv_df[conv_df$variable != "lp__", , drop = FALSE]
    conv_df$variable <- vapply(conv_df$variable, function(nm) {
      .rename_node_param(nm, j, fam_j, n_fe_full)
    }, character(1L), USE.NAMES = FALSE)
    if (fam_j == "ordinal") {
      sentinel_row <- data.frame(
        variable = sprintf("beta[1,%d]", j),
        rhat = NA_real_, ess_bulk = NA_real_, ess_tail = NA_real_,
        stringsAsFactors = FALSE
      )
      conv_df <- rbind(sentinel_row, conv_df)
    }
    conv_chunks[[j]] <- conv_df
  }

  # Bind draws across 3rd dimension
  combined_draws <- do.call(function(...) abind_simple(...), draw_chunks)

  # Add summed lp__
  if (length(lp_chunks) > 0L && !is.null(lp_chunks[[1]])) {
    lp_sum <- Reduce(`+`, lp_chunks)
    lp_arr <- array(lp_sum, dim = c(dim(lp_sum), 1L),
                    dimnames = list(NULL, NULL, "lp__"))
    combined_draws <- abind_simple(combined_draws, lp_arr)
  }

  # --- b) Convergence ---
  combined_convergence <- do.call(rbind, conv_chunks)
  rownames(combined_convergence) <- NULL

  # --- c) Diagnostics (D8) ---
  diag_list <- lapply(fits, function(f) as.data.frame(f$diagnostic_summary()))
  diagnostics <- data.frame(
    num_divergent     = Reduce(`+`, lapply(diag_list, `[[`, "num_divergent")),
    num_max_treedepth = Reduce(`+`, lapply(diag_list, `[[`, "num_max_treedepth")),
    ebfmi             = Reduce(pmin, lapply(diag_list, `[[`, "ebfmi"))
  )

  # --- d) Timing, metadata, return codes ---
  timing   <- fits[[1]]$time()
  metadata <- fits[[1]]$metadata()
  rc_list  <- lapply(fits, function(f) f$return_codes())
  return_codes <- Reduce(pmax, lapply(rc_list, abs))

  # --- e) Augment shared standata (D6) ---
  standata_full <- shared
  stopifnot(standata_full$p == p, standata_full$K == shared$K)

  ord_idx <- which(family_vec == "ordinal")
  if (length(ord_idx) > 0L) {
    standata_full$C_per_node <- vapply(ord_idx, function(j)
      as.integer(max(shared$Y[, j])), integer(1L))
    names(standata_full$C_per_node) <- as.character(ord_idx)
  }

  # --- f) Return object ---
  structure(
    list(
      draws            = combined_draws,
      convergence      = combined_convergence,
      diagnostics      = diagnostics,
      node_diagnostics = diag_list,
      timing           = timing,
      metadata         = metadata,
      return_codes     = return_codes,
      family           = family_vec,
      standata         = standata_full,
      priors           = priors,
      priors_needed    = priors_needed,
      data_used        = shared$data_used
    ),
    class = "bvarnet"
  )
}


# ---- helpers for .combine_nodewise_fits() ----

#' Rename a per-node (p=1) Stan parameter name to full-model naming
#' @keywords internal
#' @noRd
.rename_node_param <- function(name, j, family, n_fe_full) {
  if (grepl("^beta\\[", name)) {
    parts <- regmatches(name, regexec("^beta\\[(\\d+),(\\d+)\\]$", name))[[1]]
    k <- as.integer(parts[2])
    if (family == "ordinal") {
      return(sprintf("beta[%d,%d]", k + 1L, j))
    }
    return(sprintf("beta[%d,%d]", k, j))
  }
  if (grepl("^phi\\[", name)) {
    parts <- regmatches(name, regexec("^phi\\[(\\d+),(\\d+)\\]$", name))[[1]]
    m <- as.integer(parts[2])
    return(sprintf("phi[%d,%d]", m, j))
  }
  if (grepl("^sigma\\[", name)) {
    return(sprintf("sigma[%d]", j))
  }
  if (grepl("^kappa\\[", name)) {
    parts <- regmatches(name, regexec("^kappa\\[(\\d+),(\\d+)\\]$", name))[[1]]
    c_val <- as.integer(parts[3])
    return(sprintf("kappa[%d,%d]", j, c_val))
  }
  if (grepl("^sd_u\\[", name)) {
    parts <- regmatches(name, regexec("^sd_u\\[(\\d+),(\\d+)\\]$", name))[[1]]
    r <- as.integer(parts[3])
    return(sprintf("sd_u[%d,%d]", j, r))
  }
  if (grepl("^u\\[", name)) {
    parts <- regmatches(name, regexec("^u\\[(\\d+),(\\d+),(\\d+)\\]$", name))[[1]]
    i_val <- as.integer(parts[3])
    r_val <- as.integer(parts[4])
    return(sprintf("u[%d,%d,%d]", j, i_val, r_val))
  }
  if (grepl("^z_u\\[", name)) {
    parts <- regmatches(name, regexec("^z_u\\[(\\d+),(\\d+),(\\d+)\\]$", name))[[1]]
    i_val <- as.integer(parts[3])
    r_val <- as.integer(parts[4])
    return(sprintf("z_u[%d,%d,%d]", j, i_val, r_val))
  }
  name
}


#' Bind arrays along the third dimension (simple abind replacement)
#' @keywords internal
#' @noRd
abind_simple <- function(...) {
  arrays <- list(...)
  if (length(arrays) == 1L && is.list(arrays[[1]]) && !is.array(arrays[[1]]))
    arrays <- arrays[[1]]
  arrays <- Filter(Negate(is.null), arrays)
  if (length(arrays) == 0L) return(NULL)
  if (length(arrays) == 1L) return(arrays[[1]])

  d1 <- dim(arrays[[1]])[1]
  d2 <- dim(arrays[[1]])[2]
  all_names <- unlist(lapply(arrays, function(a) dimnames(a)[[3]]))
  total_d3 <- sum(vapply(arrays, function(a) dim(a)[3], integer(1L)))

  out <- array(NA_real_, dim = c(d1, d2, total_d3))
  offset <- 0L
  for (a in arrays) {
    d3 <- dim(a)[3]
    out[, , (offset + 1L):(offset + d3)] <- a
    offset <- offset + d3
  }
  dimnames(out) <- list(dimnames(arrays[[1]])[[1]],
                        dimnames(arrays[[1]])[[2]],
                        all_names)
  out
}

# -----------------------------------------------------------------------------
# Prior system for bvarnet
#
# Phase 1 supports three prior families:
#   1 = Normal, 2 = Student-t, 3 = Cauchy
# Families 4 (Laplace), 7 (Exponential), 8 (Flat) are reserved and will be
# enabled once the Stan function consolidation (#include) is available.
# -----------------------------------------------------------------------------

.supported_families <- list(
  normal    = list(int = 1L, needs_scale = TRUE,  needs_df = FALSE),
  student_t = list(int = 2L, needs_scale = TRUE,  needs_df = TRUE),
  cauchy    = list(int = 3L, needs_scale = TRUE,  needs_df = FALSE)
)

#' Construct a single prior distribution
#'
#' Builds a \code{bvarnet_prior} object specifying the prior family and its
#' parameters.  Supported families in Phase 1 are \code{"normal"},
#' \code{"student_t"}, and \code{"cauchy"}.
#'
#' @param family Character. One of \code{"normal"}, \code{"student_t"},
#'   \code{"cauchy"}.
#' @param loc Location parameter (default 0).
#' @param scale Scale parameter (default 1). Must be > 0.
#' @param df Degrees of freedom for \code{"student_t"} (default 7). Must be
#'   > 0 when \code{family = "student_t"}.
#'
#' @return A \code{bvarnet_prior} S3 object.
#' @export
prior <- function(family, loc = 0, scale = 1, df = 7) {
  family <- tryCatch(
    match.arg(family, choices = names(.supported_families)),
    error = function(e) {
      stop(sprintf(
        "Unrecognised prior family '%s'. Supported: %s",
        family, paste(names(.supported_families), collapse = ", ")
      ), call. = FALSE)
    }
  )

  fam_info <- .supported_families[[family]]

  if (fam_info$needs_scale) {
    if (!is.numeric(scale) || length(scale) != 1L || is.na(scale) || scale <= 0)
      stop("'scale' must be a single positive number.", call. = FALSE)
    if (scale > 50)
      warning("Prior scale is very large. If you want an uninformative prior, consider increasing scale further or switching to a different family.", call. = FALSE)
    if (scale < 0.01)
      warning("Prior scale is very small -- this is a strongly informative prior.", call. = FALSE)
  }

  if (fam_info$needs_df) {
    if (!is.numeric(df) || length(df) != 1L || is.na(df) || df <= 0)
      stop("'df' must be a single positive number for Student-t.", call. = FALSE)
  } else {
    df <- 0
  }

  structure(
    list(
      family     = family,
      family_int = fam_info$int,
      loc        = loc,
      scale      = scale,
      df         = df,
      is_default = FALSE
    ),
    class = "bvarnet_prior"
  )
}

#' Format a bvarnet_prior for printing
#'
#' @param x A \code{bvarnet_prior} object.
#' @param half Logical; if \code{TRUE} prepends "Half-" to indicate a
#'   half-prior (used for positive-constrained parameters like sd_u and sigma).
#' @param ... Ignored.
#' @return A character string.
#' @export
format.bvarnet_prior <- function(x, half = FALSE, ...) {
  prefix <- if (isTRUE(half)) "Half-" else ""
  switch(x$family,
    normal    = sprintf("%sNormal(%g, %g)",               prefix, x$loc, x$scale),
    student_t = sprintf("%sStudent-t(%g, %g, df = %g)",  prefix, x$loc, x$scale, x$df),
    cauchy    = sprintf("%sCauchy(%g, %g)",               prefix, x$loc, x$scale)
  )
}

#' Print a bvarnet_prior
#'
#' @param x A \code{bvarnet_prior} object.
#' @param ... Passed to \code{format.bvarnet_prior()}.
#' @return \code{x} invisibly.
#' @export
print.bvarnet_prior <- function(x, ...) {
  cat(format(x, ...), "\n")
  invisible(x)
}

# Internal helper: construct a default prior (is_default = TRUE)
.default_prior <- function(family, loc, scale, df = 7) {
  p <- prior(family, loc = loc, scale = scale, df = df)
  p$is_default <- TRUE
  p
}

#' Build a prior specification object for `bvar()`
#'
#' Returns a \code{bvarnet_priors} object containing a \code{bvarnet_prior}
#' for every model parameter type.  Any argument left as \code{NULL} uses the
#' package default. 
#' Available prior distributions are:
#' - normal(loc, scale)
#' - student_t(loc, scale, df)
#' - cauchy(loc, scale)
#' For standart deviations and random effects, the prior is automatically converted to a half-prior (truncated at `loc`) in the Stan code, so the printed format reflects this.
#'
#' @param intercept Prior for the intercept. Only applies to gaussian and bernoulli models;
#' for ordinal models the intercept is absorbed into the kappa (threshold parameter).
#' @param beta      Prior for fixed-effect regression coefficients (slopes).
#' @param phi    Prior for lag coefficients.
#' @param sd_u   Prior for random-effect standard deviations (half-prior).
#' @param kappa  Prior for ordinal cut-points (ordinal models only).
#' @param sigma  Prior for residual standard deviation (gaussian models only;
#'   half-prior).
#'
#' @return A \code{bvarnet_priors} S3 object.
#' @export
set_priors <- function(intercept = NULL,
                       beta      = NULL,
                       phi       = NULL,
                       sd_u      = NULL,
                       kappa     = NULL,
                       sigma     = NULL) {

  defaults <- list(
    intercept = .default_prior("normal", 0, 1),
    beta      = .default_prior("normal", 0, 1),
    phi       = .default_prior("normal", 0, 0.5),
    sd_u      = .default_prior("normal", 0, 1),
    kappa     = .default_prior("normal", 0, 2),
    sigma     = .default_prior("normal", 0, 2.5)
  )

  resolve <- function(user, default) {
    if (is.null(user)) return(default)
    if (!inherits(user, "bvarnet_prior"))
      stop("Each prior must be a bvarnet_prior object created with prior().",
           call. = FALSE)
    user
  }

  structure(
    list(
      intercept = resolve(intercept, defaults$intercept),
      beta      = resolve(beta,      defaults$beta),
      phi       = resolve(phi,       defaults$phi),
      sd_u      = resolve(sd_u,      defaults$sd_u),
      kappa     = resolve(kappa,     defaults$kappa),
      sigma     = resolve(sigma,     defaults$sigma)
    ),
    class = "bvarnet_priors"
  )
}

#' Print a bvarnet_priors specification
#'
#' Shows only the priors explicitly set by the user.  When no priors have
#' been overridden (all defaults), a compact note is printed instead.
#'
#' @param x A \code{bvarnet_priors} object.
#' @param ... Ignored.
#' @return \code{x} invisibly.
#' @export
print.bvarnet_priors <- function(x, ...) {
  half_pars <- c("sd_u", "sigma")
  nms <- names(x)
  # Only consider actual bvarnet_prior entries (skip NULLs from filtered objects)
  nms <- nms[vapply(x[nms], function(p) inherits(p, "bvarnet_prior"), logical(1))]

  user_set <- nms[!vapply(x[nms], function(p) isTRUE(p$is_default), logical(1))]

  cat("bvarnet prior specification:\n")

  if (length(user_set) == 0L) {
    cat("  (all defaults \u2014 see ?get_default_priors)\n")
  } else {
    for (nm in user_set) {
      cat(sprintf("  %-6s ~ %s\n", nm, format(x[[nm]], half = nm %in% half_pars)))
    }
  }

  invisible(x)
}

#' Get the default prior specification for a given model family
#'
#' Returns a \code{bvarnet_priors} object showing the default priors that
#' apply to a particular model configuration. Parameters irrelevant to
#' the chosen family or model structure are omitted, so the returned object
#' reflects what the sampler will actually use.
#'
#' @param family Character (optional). One of \code{"bernoulli"},
#'   \code{"ordinal"}, \code{"gaussian"}. When \code{NULL} (the default),
#'   all parameter priors are shown.
#' @param has_re Logical. Does the model include random effects?
#'   Default \code{TRUE}. When \code{FALSE}, the \code{sd_u} prior is
#'   omitted.
#' @return A \code{bvarnet_priors} object.
#' @export
get_default_priors <- function(family = NULL, has_re = TRUE) {
  p <- set_priors()

  if (!is.null(family)) {
    family <- match.arg(family, c("bernoulli", "ordinal", "gaussian"))
    if (family != "gaussian") p$sigma <- NULL
    if (family != "ordinal")  p$kappa <- NULL
  }

  if (!isTRUE(has_re)) p$sd_u <- NULL

  p
}


# ---- Internal helpers for prior resolution in bvar() ----

#' Ensure all prior slots required by the model family exist
#'
#' If slots are missing (e.g. from a filtered \code{get_default_priors()}
#' object), they are filled with package defaults and a warning is issued.
#' Called by \code{bvar()} before passing priors to \code{to_stan_data()}.
#'
#' @param priors A \code{bvarnet_priors} object (possibly incomplete).
#' @param family_vec Named character vector of families per node.
#' @return A complete \code{bvarnet_priors} object.
#' @keywords internal
#' @noRd
.ensure_prior_slots <- function(priors, family_vec) {
  # sd_u is always passed to Stan (even when n_re == 0)
  required <- c("beta", "phi", "sd_u")
  if (any(family_vec %in% c("gaussian", "bernoulli"))) required <- c(required, "intercept")
  if (any(family_vec == "gaussian")) required <- c(required, "sigma")
  if (any(family_vec == "ordinal"))  required <- c(required, "kappa")

  present <- names(Filter(Negate(is.null), priors))
  missing <- setdiff(required, present)

  if (length(missing) > 0L) {
    defaults <- list(
      beta      = .default_prior("normal", 0, 1),
      intercept = .default_prior("normal", 0, 1),
      phi       = .default_prior("normal", 0, 0.5),
      sd_u      = .default_prior("normal", 0, 1),
      kappa     = .default_prior("normal", 0, 2),
      sigma     = .default_prior("normal", 0, 2.5)
    )
    for (nm in missing) priors[[nm]] <- defaults[[nm]]
    warning("Prior(s) missing from input but required by this model: ",
            paste(missing, collapse = ", "), ". Using package defaults.",
            call. = FALSE)
  }
  priors
}


#' Emit user-facing warnings about prior usage and return needed-prior names
#'
#' Warns when user-set priors are not needed by the model, and messages when
#' the model uses default priors that the user did not explicitly set (only
#' if the user set at least one prior).
#'
#' @param priors A \code{bvarnet_priors} object.
#' @param family_vec Named character vector of families per node.
#' @param n_re Integer. Number of random-effect columns from the built design.
#' @return Character vector of prior names the model actually uses.
#' @keywords internal
#' @noRd
.prior_warnings <- function(priors, family_vec, n_re) {
  has_intercept <- any(family_vec %in% c("gaussian", "bernoulli"))
  needed <- if (has_intercept) c("intercept", "beta", "phi") else c("beta", "phi")
  if (n_re > 0L) needed <- c(needed, "sd_u")
  if (any(family_vec == "gaussian")) needed <- c(needed, "sigma")
  if (any(family_vec == "ordinal"))  needed <- c(needed, "kappa")

  # Identify which slots the user explicitly set
  nms <- names(Filter(function(p) inherits(p, "bvarnet_prior"), priors))
  user_set <- nms[!vapply(priors[nms], function(p) isTRUE(p$is_default), logical(1))]

  # User-set priors the model doesn't need
  unused <- setdiff(user_set, needed)
  if (length(unused) > 0L)
    warning("Prior(s) set but not used by this model: ",
            paste(unused, collapse = ", "), ". These will be ignored.",
            call. = FALSE)

  # Defaults auto-filled for model (only warn if user set at least one)
  auto_filled <- setdiff(needed, user_set)
  if (length(user_set) > 0L && length(auto_filled) > 0L)
    message("Using default priors for: ", paste(auto_filled, collapse = ", "))

  needed
}

# ─────────────────────────────────────────────────────────────────────────────
# test-set-priors.R — tests for prior(), set_priors(), format(), print(),
#                     and get_default_priors()
# ─────────────────────────────────────────────────────────────────────────────

# ═════════════════════════════════════════════════════════════════════════════
# §1 — prior() construction
# ═════════════════════════════════════════════════════════════════════════════

test_that("prior('normal') returns a bvarnet_prior object", {
  p <- prior("normal", 0, 1)
  expect_s3_class(p, "bvarnet_prior")
})

test_that("prior('normal') has correct family_int", {
  expect_equal(prior("normal", 0, 1)$family_int, 1L)
})

test_that("prior('student_t') has correct family_int and preserves df", {
  p <- prior("student_t", 0, 1, df = 7)
  expect_equal(p$family_int, 2L)
  expect_equal(p$df, 7)
})

test_that("prior('cauchy') has correct family_int", {
  expect_equal(prior("cauchy", 0, 1)$family_int, 3L)
})

test_that("all Phase-1 families produce objects with expected family_int values", {
  supported <- list(
    list("normal",    1L),
    list("student_t", 2L),
    list("cauchy",    3L)
  )
  for (s in supported) {
    p <- prior(s[[1]], 0, 1)
    expect_equal(p$family_int, s[[2]], info = s[[1]])
  }
})

test_that("prior() built directly by user has is_default == FALSE", {
  expect_false(prior("normal", 0, 1)$is_default)
})

test_that("prior() stores loc and scale correctly", {
  p <- prior("normal", loc = 0.5, scale = 2)
  expect_equal(p$loc,   0.5)
  expect_equal(p$scale, 2)
})

test_that("unrecognised family stops with informative error", {
  expect_error(prior("horseshoe"), "Unrecognised prior family")
  expect_error(prior("flat"),      "Unrecognised prior family")
  expect_error(prior("laplace"),   "Unrecognised prior family")
})

test_that("scale <= 0 stops", {
  expect_error(prior("normal", 0, 0),  "scale")
  expect_error(prior("normal", 0, -1), "scale")
})

test_that("df <= 0 for student_t stops", {
  expect_error(prior("student_t", 0, 1, df = 0),  "df")
  expect_error(prior("student_t", 0, 1, df = -1), "df")
})

test_that("very large scale produces a warning", {
  expect_warning(prior("normal", 0, 100), "very large")
})

test_that("very small scale produces a warning", {
  expect_warning(prior("normal", 0, 0.001), "strongly informative")
})

test_that("non-student_t family ignores df (sets it to 0)", {
  expect_equal(prior("normal", 0, 1)$df, 0)
  expect_equal(prior("cauchy", 0, 1)$df, 0)
})


# ═════════════════════════════════════════════════════════════════════════════
# §2 — set_priors() defaults
# ═════════════════════════════════════════════════════════════════════════════

test_that("set_priors() returns a bvarnet_priors object", {
  expect_s3_class(set_priors(), "bvarnet_priors")
})

test_that("set_priors() has the correct names", {
  expect_named(set_priors(), c("intercept", "beta", "phi", "sd_u", "kappa", "sigma"))
})

test_that("default beta is Normal(0, 1)", {
  sp <- set_priors()
  expect_equal(sp$beta$family_int, 1L)
  expect_equal(sp$beta$loc,   0)
  expect_equal(sp$beta$scale, 1)
})

test_that("default phi scale is 0.5", {
  expect_equal(set_priors()$phi$scale, 0.5)
})

test_that("default sd_u scale is 1", {
  expect_equal(set_priors()$sd_u$scale, 1)
})

test_that("default sigma scale is 2.5", {
  expect_equal(set_priors()$sigma$scale, 2.5)
})

test_that("default kappa scale is 2", {
  expect_equal(set_priors()$kappa$scale, 2)
})

test_that("all defaults have is_default == TRUE", {
  sp <- set_priors()
  for (nm in names(sp)) {
    expect_true(sp[[nm]]$is_default, info = nm)
  }
})

test_that("overriding beta sets is_default to FALSE for beta only", {
  sp <- set_priors(beta = prior("cauchy", 0, 0.5))
  expect_equal(sp$beta$family_int, 3L)
  expect_false(sp$beta$is_default)
  # Other slots unchanged
  expect_true(sp$phi$is_default)
  expect_true(sp$sd_u$is_default)
})

test_that("overriding phi does not affect other slots", {
  sp <- set_priors(phi = prior("student_t", 0, 0.3, df = 5))
  expect_equal(sp$phi$family_int, 2L)
  expect_equal(sp$phi$df, 5)
  expect_true(sp$beta$is_default)
})

test_that("passing non-bvarnet_prior to set_priors() stops", {
  expect_error(set_priors(beta = "normal"), "bvarnet_prior")
  expect_error(set_priors(phi  = list(family = "normal")), "bvarnet_prior")
})


# ═════════════════════════════════════════════════════════════════════════════
# §3 — format.bvarnet_prior() and print.bvarnet_priors()
# ═════════════════════════════════════════════════════════════════════════════

test_that("format(prior('normal', 0, 1)) equals 'Normal(0, 1)'", {
  expect_equal(format(prior("normal", 0, 1)), "Normal(0, 1)")
})

test_that("format with half = TRUE prepends 'Half-'", {
  expect_equal(format(prior("normal", 0, 1), half = TRUE), "Half-Normal(0, 1)")
})

test_that("format(prior('student_t', ...)) contains 'Student-t'", {
  expect_match(format(prior("student_t", 0, 1, df = 7)), "Student-t")
})

test_that("format(prior('cauchy', ...)) contains 'Cauchy'", {
  expect_match(format(prior("cauchy", 0, 1)), "Cauchy")
})

test_that("print(set_priors()) shows 'all defaults' message", {
  out <- capture.output(print(set_priors()))
  expect_true(any(grepl("all defaults", out)))
})

test_that("print(set_priors()) does not show individual prior lines when all defaults", {
  out <- capture.output(print(set_priors()))
  # Should NOT have individual "beta ~" lines
  expect_false(any(grepl("^\\s*beta\\s+~", out)))
})

test_that("print with one user prior shows only that prior", {
  sp <- set_priors(beta = prior("cauchy", 0, 2))
  out <- capture.output(print(sp))
  # beta should appear on its own line
  expect_true(any(grepl("^\\s*beta\\s+~", out)))
  # defaults should NOT appear
  expect_false(any(grepl("phi", out)))
  expect_false(any(grepl("sd_u", out)))
  expect_false(any(grepl("Defaults:", out)))
})

test_that("print with multiple user priors shows only those", {
  sp <- set_priors(
    beta = prior("cauchy", 0, 2),
    phi  = prior("student_t", 0, 0.3, df = 5)
  )
  out <- capture.output(print(sp))
  expect_true(any(grepl("beta", out)))
  expect_true(any(grepl("phi", out)))
  expect_false(any(grepl("sd_u", out)))
  expect_false(any(grepl("sigma", out)))
})

test_that("print with all user-set priors shows all of them", {
  sp <- set_priors(
    beta  = prior("normal", 0, 2),
    phi   = prior("normal", 0, 1),
    sd_u  = prior("cauchy", 0, 1),
    kappa = prior("normal", 0, 3),
    sigma = prior("cauchy", 0, 2)
  )
  out <- capture.output(print(sp))
  expect_false(any(grepl("all defaults", out)))
  expect_true(any(grepl("beta",  out)))
  expect_true(any(grepl("phi",   out)))
  expect_true(any(grepl("sd_u",  out)))
  expect_true(any(grepl("sigma", out)))
  expect_true(any(grepl("kappa", out)))
})


# ═════════════════════════════════════════════════════════════════════════════
# §4 — get_default_priors()
# ═════════════════════════════════════════════════════════════════════════════

test_that("get_default_priors('bernoulli') returns a bvarnet_priors object", {
  expect_s3_class(get_default_priors("bernoulli"), "bvarnet_priors")
})

test_that("get_default_priors('ordinal') has a kappa slot", {
  sp <- get_default_priors("ordinal")
  expect_true("kappa" %in% names(sp))
  expect_s3_class(sp$kappa, "bvarnet_prior")
})

test_that("get_default_priors('gaussian') has a sigma slot", {
  sp <- get_default_priors("gaussian")
  expect_true("sigma" %in% names(sp))
  expect_s3_class(sp$sigma, "bvarnet_prior")
})

test_that("get_default_priors() with invalid family stops", {
  expect_error(get_default_priors("poisson"), "arg")
})

test_that("get_default_priors('bernoulli') omits sigma and kappa", {
  sp <- get_default_priors("bernoulli")
  expect_null(sp$sigma)
  expect_null(sp$kappa)
  expect_true("beta" %in% names(sp))
  expect_true("phi"  %in% names(sp))
  expect_true("sd_u" %in% names(sp))
})

test_that("get_default_priors('gaussian') omits kappa but keeps sigma", {
  sp <- get_default_priors("gaussian")
  expect_null(sp$kappa)
  expect_true("sigma" %in% names(sp))
})

test_that("get_default_priors('ordinal') omits sigma but keeps kappa", {
  sp <- get_default_priors("ordinal")
  expect_null(sp$sigma)
  expect_true("kappa" %in% names(sp))
})

test_that("get_default_priors(has_re = FALSE) omits sd_u", {
  sp <- get_default_priors("bernoulli", has_re = FALSE)
  expect_null(sp$sd_u)
  expect_true("beta" %in% names(sp))
  expect_true("phi"  %in% names(sp))
})

test_that("get_default_priors() with no args returns all six", {
  sp <- get_default_priors()
  expect_named(sp, c("intercept", "beta", "phi", "sd_u", "kappa", "sigma"))
})


# ═════════════════════════════════════════════════════════════════════════════
# §5 — .ensure_prior_slots()
# ═════════════════════════════════════════════════════════════════════════════

test_that(".ensure_prior_slots fills missing sigma for gaussian family", {
  p <- get_default_priors("bernoulli")  # no sigma
  family_vec <- c(y_1 = "gaussian")
  expect_warning(
    filled <- bvarnet:::.ensure_prior_slots(p, family_vec),
    "sigma"
  )
  expect_s3_class(filled$sigma, "bvarnet_prior")
  expect_true(filled$sigma$is_default)
})

test_that(".ensure_prior_slots fills missing kappa for ordinal family", {
  p <- get_default_priors("bernoulli")  # no kappa
  family_vec <- c(y_1 = "ordinal")
  expect_warning(
    filled <- bvarnet:::.ensure_prior_slots(p, family_vec),
    "kappa"
  )
  expect_s3_class(filled$kappa, "bvarnet_prior")
})

test_that(".ensure_prior_slots fills missing sd_u from filtered object", {
  p <- get_default_priors("bernoulli", has_re = FALSE)  # no sd_u
  family_vec <- c(y_1 = "bernoulli")
  expect_warning(
    filled <- bvarnet:::.ensure_prior_slots(p, family_vec),
    "sd_u"
  )
  expect_s3_class(filled$sd_u, "bvarnet_prior")
})

test_that(".ensure_prior_slots does nothing for complete priors", {
  p <- set_priors()
  family_vec <- c(y_1 = "gaussian", y_2 = "ordinal")
  # No warning expected
  expect_silent(bvarnet:::.ensure_prior_slots(p, family_vec))
})


# ═════════════════════════════════════════════════════════════════════════════
# §6 — .prior_warnings()
# ═════════════════════════════════════════════════════════════════════════════

test_that(".prior_warnings returns correct needed set for bernoulli without REs", {
  p <- set_priors()
  needed <- bvarnet:::.prior_warnings(p, c(y_1 = "bernoulli"), n_re = 0L)
  expect_equal(needed, c("intercept", "beta", "phi"))
})

test_that(".prior_warnings returns correct needed set for gaussian with REs", {
  p <- set_priors()
  needed <- bvarnet:::.prior_warnings(p, c(y_1 = "gaussian"), n_re = 2L)
  expect_equal(needed, c("intercept", "beta", "phi", "sd_u", "sigma"))
})

test_that(".prior_warnings returns correct needed set for ordinal", {
  p <- set_priors()
  needed <- bvarnet:::.prior_warnings(p, c(y_1 = "ordinal"), n_re = 0L)
  expect_equal(needed, c("beta", "phi", "kappa"))
})

test_that(".prior_warnings warns about unused user-set priors", {
  p <- set_priors(sigma = prior("cauchy", 0, 2))
  expect_warning(
    bvarnet:::.prior_warnings(p, c(y_1 = "bernoulli"), n_re = 0L),
    "not used.*sigma"
  )
})

test_that(".prior_warnings warns about unused sd_u when no REs", {
  p <- set_priors(sd_u = prior("cauchy", 0, 1))
  expect_warning(
    bvarnet:::.prior_warnings(p, c(y_1 = "bernoulli"), n_re = 0L),
    "not used.*sd_u"
  )
})

test_that(".prior_warnings messages about auto-filled defaults", {
  p <- set_priors(beta = prior("cauchy", 0, 2))
  expect_message(
    bvarnet:::.prior_warnings(p, c(y_1 = "gaussian"), n_re = 0L),
    "Using default priors for: intercept, phi, sigma"
  )
})

test_that(".prior_warnings does NOT message when all priors are defaults", {
  p <- set_priors()
  expect_silent(
    bvarnet:::.prior_warnings(p, c(y_1 = "gaussian"), n_re = 0L)
  )
})

test_that(".prior_warnings does NOT message when all needed priors are user-set", {
  p <- set_priors(
    beta      = prior("cauchy", 0, 2),
    intercept = prior("normal", 0, 1),
    phi       = prior("normal", 0, 0.3)
  )
  expect_silent(
    bvarnet:::.prior_warnings(p, c(y_1 = "bernoulli"), n_re = 0L)
  )
})

# ──────────────────────────────────────────────────────────────────────────────
# sim_var() — simulate data from a multilevel VAR model
#
# Generates data matching the exact generative model estimated by
# bvar() for all three families: bernoulli, ordinal, gaussian.
# ──────────────────────────────────────────────────────────────────────────────

#' Simulate data from a multilevel VAR model
#'
#' Generates data from the generative model implied by each Stan model family.
#' Useful for testing parameter recovery and model validation.
#'
#' @param N Integer. Number of subjects (groups).
#' @param T_obs Integer. Number of time points per subject.
#' @param p Integer. Number of outcome nodes.
#' @param K Integer. AR order (default 1).
#' @param family Character. One of \code{"bernoulli"}, \code{"ordinal"},
#'   \code{"gaussian"}.
#' @param alpha Numeric vector of length \code{p}. Population intercepts
#'   (on logit scale for bernoulli, identity for gaussian). For ordinal,
#'   this is absorbed into kappa and should be left NULL. Generated if NULL.
#' @param gamma Matrix \code{q x p}. Population covariate effects. Generated
#'   if NULL and \code{q > 0}.
#' @param Phi Matrix \code{(p*K) x p}. Population lag coefficients. Generated
#'   if NULL.
#' @param sigma Numeric vector of length \code{p}. Residual SD per node
#'   (gaussian only). Generated if NULL.
#' @param kappa List of \code{p} ordered vectors, each of length \code{C-1}.
#'   Cutpoints per node (ordinal only). Generated if NULL.
#' @param q Integer. Number of covariates (default 0).
#' @param x_gen Function \code{f(N, T_obs)} returning an \code{N x T_obs x q}
#'   array of covariates. If NULL, default generation is used.
#' @param sd_alpha Numeric. SD of random intercepts (scalar or p-vector).
#'   Default 0.5. Set to 0 to simulate a fixed-effects-only model with no
#'   between-person variation in intercepts.
#' @param sd_phi Numeric. SD of random lag coefficients (scalar or matrix).
#'   Default 0.2.
#' @param sd_gamma Numeric or NULL. SD of random covariate slopes. NULL means
#'   no random slopes on covariates.
#' @param re_temporal Logical. Include random slopes on lag predictors?
#'   Default FALSE.
#' @param C Integer. Number of ordinal categories (ordinal only, default 5).
#' @param burnin Integer. Number of time points to discard as warmup before
#'   recording data (default 500). The VAR process is simulated for
#'   \code{burnin + T_obs} time points per subject, and the first
#'   \code{burnin} are discarded. This allows the process to reach its
#'   stationary distribution before data collection begins.
#' @param seed Integer or NULL. RNG seed.
#'
#' @details
#' To simulate a VAR without any random effects (i.e. all subjects share
#' identical parameters), set \code{sd_alpha = 0}, \code{re_temporal = FALSE}
#' (the default), and \code{sd_gamma = NULL} (the default).
#'
#' @return A list with two components:
#' \describe{
#'   \item{data}{A long-format data frame with columns \code{id}, \code{t},
#'     \code{y_1}, ..., \code{y_p}, and optionally \code{x_1}, ..., \code{x_q}.}
#'   \item{truth}{A list of true generating parameters.}
#' }
#'
#' @export
sim_var <- function(
    N,
    T_obs,
    p,
    K        = 1L,
    family   = c("bernoulli", "ordinal", "gaussian"),
    alpha    = NULL,
    gamma    = NULL,
    Phi      = NULL,
    sigma    = NULL,
    kappa    = NULL,
    q        = 0L,
    x_gen    = NULL,
    sd_alpha = 0.5,
    sd_phi   = 0.2,
    sd_gamma = NULL,
    re_temporal = FALSE,
    C        = 5L,
    burnin   = 500L,
    seed     = NULL
) {
  ## ── 0. Validation & setup ──────────────────────────────────────────────────
  N      <- as.integer(N)
  T_obs  <- as.integer(T_obs)
  p      <- as.integer(p)
  K      <- as.integer(K)
  q      <- as.integer(q)
  C      <- as.integer(C)

  burnin <- as.integer(burnin)

  stopifnot(N >= 1L, T_obs >= K + 1L, p >= 1L, K >= 1L, q >= 0L, burnin >= 0L)

  # Parse family: scalar (or default match.arg triple) vs per-node vector
  y_cols <- paste0("y_", seq_len(p))
  valid_families <- c("bernoulli", "ordinal", "gaussian")
  if (length(family) == 1L || identical(family, valid_families)) {
    # Scalar string or untouched default arg → match.arg, then recycle
    family <- match.arg(family, valid_families)
    family_vec <- setNames(rep(family, p), y_cols)
  } else {
    family_vec <- .parse_family(family, y_cols)
  }

  if (any(family_vec == "ordinal")) stopifnot(C >= 2L)
  if (!is.null(seed)) set.seed(seed)

  PK <- p * K
  T_total <- T_obs + burnin  # total simulated length (burnin then recorded)

  # sd_alpha: broadcast scalar to p-vector
  if (length(sd_alpha) == 1L) sd_alpha <- rep(sd_alpha, p)
  stopifnot(length(sd_alpha) == p)

  ## ── 1. Covariates ─────────────────────────────────────────────────────────
  # Generate for the full T_total length (burnin + recorded)
  if (q > 0L) {
    if (!is.null(x_gen)) {
      X_cov <- x_gen(N, T_total)
      stopifnot(is.array(X_cov), identical(dim(X_cov), c(N, T_total, q)))
    } else {
      X_cov <- generate_default_covariates(N, T_total, q)
    }
  } else {
    X_cov <- array(0, dim = c(N, T_total, 0L))
  }

  ## ── 2. Population parameters ──────────────────────────────────────────────
  # Intercepts (ordinal nodes get 0 — kappa absorbs intercept)
  if (is.null(alpha)) {
    alpha <- runif(p, -1, 1)
    alpha[family_vec == "ordinal"] <- 0
  }
  stopifnot(length(alpha) == p)

  # Covariate effects
  if (q > 0L) {
    if (is.null(gamma)) {
      gamma <- matrix(runif(q * p, -0.5, 0.5), nrow = q, ncol = p)
    }
    stopifnot(is.matrix(gamma), nrow(gamma) == q, ncol(gamma) == p)
  } else {
    gamma <- matrix(0, nrow = 0L, ncol = p)
  }

  # Lag coefficients
  if (is.null(Phi)) {
    Phi <- generate_stable_phi(p, K)
  }
  stopifnot(is.matrix(Phi), nrow(Phi) == PK, ncol(Phi) == p)
  if (!check_var_stability(Phi, p, K)) {
    warning("Provided Phi is not VAR-stable (eigenvalues >= 1). ",
            "Rescaling to ensure stability.")
    Phi <- rescale_to_stable(Phi, p, K)
  }

  # Gaussian: residual SD (NA for non-gaussian nodes)
  gauss_idx <- which(family_vec == "gaussian")
  if (length(gauss_idx) > 0L) {
    if (is.null(sigma)) {
      sigma <- rep(NA_real_, p)
      sigma[gauss_idx] <- runif(length(gauss_idx), 0.5, 1.5)
    }
    stopifnot(length(sigma) == p, all(sigma[gauss_idx] > 0, na.rm = TRUE))
  } else {
    sigma <- rep(NA_real_, p)
  }

  # Ordinal: cutpoints (NULL for non-ordinal nodes)
  ord_idx <- which(family_vec == "ordinal")
  if (length(ord_idx) > 0L) {
    if (is.null(kappa)) {
      kappa <- vector("list", p)
      for (node in ord_idx) {
        kappa[[node]] <- generate_default_kappa(C)
      }
    }
    stopifnot(is.list(kappa), length(kappa) == p)
    for (node in ord_idx) {
      stopifnot(length(kappa[[node]]) == C - 1L)
      stopifnot(!is.unsorted(kappa[[node]], strictly = TRUE))
    }
  } else {
    kappa <- vector("list", p)
  }

  ## ── 3. Person-level random effects ────────────────────────────────────────
  # Random intercepts (always present for bernoulli/gaussian, not for ordinal)
  alpha_i <- matrix(NA_real_, N, p)
  for (i in seq_len(N)) {
    alpha_i[i, ] <- rnorm(p, mean = alpha, sd = sd_alpha)
  }

  # Random temporal effects
  Phi_i <- array(NA_real_, dim = c(N, PK, p))
  if (isTRUE(re_temporal)) {
    sd_phi_mat <- if (is.matrix(sd_phi)) sd_phi else matrix(sd_phi, PK, p)
    for (i in seq_len(N)) {
      Phi_i[i, , ] <- Phi + matrix(rnorm(PK * p, 0, sd_phi_mat), PK, p)
    }
  } else {
    for (i in seq_len(N)) {
      Phi_i[i, , ] <- Phi
    }
  }

  # Random covariate slopes
  gamma_i <- array(NA_real_, dim = c(N, max(q, 0L), p))
  if (!is.null(sd_gamma) && q > 0L) {
    sd_gamma_mat <- if (is.matrix(sd_gamma)) sd_gamma else matrix(sd_gamma, q, p)
    for (i in seq_len(N)) {
      gamma_i[i, , ] <- gamma + matrix(rnorm(q * p, 0, sd_gamma_mat), q, p)
    }
  } else if (q > 0L) {
    for (i in seq_len(N)) {
      gamma_i[i, , ] <- gamma
    }
  }

  ## ── 4. Forward simulation ─────────────────────────────────────────────────
  # Simulate T_total = burnin + T_obs time points; discard first `burnin`
  Y_full <- array(NA_real_, dim = c(N, T_total, p))

  for (i in seq_len(N)) {
    # --- Initialize first K time points (intercept + covariates only) ---
    for (t in seq_len(K)) {
      eta <- alpha_i[i, ]
      if (q > 0L) {
        g_mat <- matrix(gamma_i[i, , ], nrow = q, ncol = p)
        x_vec <- matrix(X_cov[i, t, ], nrow = 1L)
        eta <- eta + as.numeric(x_vec %*% g_mat)
      }
      for (j in seq_len(p)) {
        Y_full[i, t, j] <- generate_response_node(
          eta[j], family_vec[j], sigma[j], kappa[[j]]
        )
      }
    }

    # --- Forward simulate t = K+1 .. T_total ---
    for (t in (K + 1L):T_total) {
      # Build lag vector: [y_{t-1}, y_{t-2}, ..., y_{t-K}]
      lag_y <- numeric(PK)
      for (lag in seq_len(K)) {
        idx <- ((lag - 1L) * p + 1L):(lag * p)
        lag_y[idx] <- Y_full[i, t - lag, ]
      }

      # Linear predictor
      eta <- alpha_i[i, ] +
        as.numeric(t(Phi_i[i, , ]) %*% lag_y)

      if (q > 0L) {
        g_mat <- matrix(gamma_i[i, , ], nrow = q, ncol = p)
        x_vec <- matrix(X_cov[i, t, ], nrow = 1L)
        eta <- eta + as.numeric(x_vec %*% g_mat)
      }

      for (j in seq_len(p)) {
        Y_full[i, t, j] <- generate_response_node(
          eta[j], family_vec[j], sigma[j], kappa[[j]]
        )
      }
    }
  }

  # Discard burnin period
  keep_idx <- (burnin + 1L):T_total
  Y     <- Y_full[, keep_idx, , drop = FALSE]
  X_cov <- X_cov[, keep_idx, , drop = FALSE]

  ## ── 5. Assemble long-format data frame ────────────────────────────────────
  df <- assemble_long_df(Y, X_cov, N, T_obs, p, q)

  ## ── 6. Build truth object ─────────────────────────────────────────────────
  # Build sd_u to match Stan layout: matrix[p, n_re]
  #   n_re columns depend on re_temporal and sd_gamma
  sd_u_list <- list()
  if (!is.null(sd_gamma) && q > 0L) {
    sd_gamma_mat <- if (is.matrix(sd_gamma)) sd_gamma else matrix(sd_gamma, q, p)
    sd_u_list[["gamma"]] <- t(sd_gamma_mat)  # p x q
  }
  if (isTRUE(re_temporal)) {
    sd_phi_mat <- if (is.matrix(sd_phi)) sd_phi else matrix(sd_phi, PK, p)
    sd_u_list[["phi"]] <- t(sd_phi_mat)  # p x PK
  }
  if (length(sd_u_list) > 0L) {
    sd_u <- do.call(cbind, sd_u_list)
  } else {
    sd_u <- matrix(0, p, 0L)
  }

  truth <- list(
    alpha      = alpha,
    gamma      = if (q > 0L) gamma else NULL,
    Phi        = Phi,
    sigma      = sigma,
    kappa      = kappa,
    sd_alpha   = sd_alpha,
    sd_u       = sd_u,
    alpha_i    = alpha_i,
    Phi_i      = Phi_i,
    gamma_i    = if (q > 0L) gamma_i else NULL,
    family     = family_vec,
    N          = N,
    T_obs      = T_obs,
    p          = p,
    K          = K,
    q          = q,
    C          = if (any(family_vec == "ordinal")) C else NULL,
    burnin     = burnin
  )

  list(data = df, truth = truth)
}


# ──────────────────────────────────────────────────────────────────────────────
# Internal helper functions
# ──────────────────────────────────────────────────────────────────────────────

#' Generate default covariates (odd = continuous, even = binary)
#' @noRd
generate_default_covariates <- function(N, T_obs, q) {
  X <- array(NA_real_, dim = c(N, T_obs, q))
  for (j in seq_len(q)) {
    if (j %% 2L == 1L) {
      X[, , j] <- rnorm(N * T_obs)
    } else {
      X[, , j] <- rbinom(N * T_obs, 1L, 0.5)
    }
  }
  X
}


#' Generate a stable VAR(K) lag coefficient matrix
#'
#' Creates a Phi matrix with moderate auto-regressive effects on the diagonal
#' and sparse cross-lag effects, rescaled to ensure VAR stability.
#' @noRd
generate_stable_phi <- function(p, K) {
  PK <- p * K
  Phi <- matrix(0, PK, p)

  for (lag in seq_len(K)) {
    block_rows <- ((lag - 1L) * p + 1L):(lag * p)
    block <- matrix(0, p, p)

    # Autoregressive effects (decay with lag)
    diag(block) <- runif(p, 0.2, 0.6) / lag

    # Sparse cross-lag effects (about 30% density)
    for (row in seq_len(p)) {
      for (col in seq_len(p)) {
        if (row != col && runif(1) < 0.3) {
          block[row, col] <- runif(1, -0.2, 0.2) / lag
        }
      }
    }

    Phi[block_rows, ] <- block
  }

  # Ensure stability
  if (!check_var_stability(Phi, p, K)) {
    Phi <- rescale_to_stable(Phi, p, K)
  }

  Phi
}


#' Check VAR stability (all companion matrix eigenvalues inside unit circle)
#' @noRd
check_var_stability <- function(Phi, p, K) {
  companion <- build_companion(Phi, p, K)
  max_ev <- max(abs(eigen(companion, only.values = TRUE)$values))
  max_ev < 1.0
}


#' Build companion matrix for VAR(K)
#' @noRd
build_companion <- function(Phi, p, K) {
  PK <- p * K
  if (K == 1L) {
    return(t(Phi))  # p x p
  }

  companion <- matrix(0, PK, PK)
  for (lag in seq_len(K)) {
    rows <- ((lag - 1L) * p + 1L):(lag * p)
    companion[1:p, rows] <- t(Phi[rows, ])
  }
  if (K > 1L) {
    companion[(p + 1L):PK, 1:(p * (K - 1L))] <- diag(p * (K - 1L))
  }
  companion
}


#' Rescale Phi to ensure VAR stability (spectral radius < 0.95)
#' @noRd
rescale_to_stable <- function(Phi, p, K, target = 0.95) {
  companion <- build_companion(Phi, p, K)
  max_ev <- max(abs(eigen(companion, only.values = TRUE)$values))
  if (max_ev >= 1.0) {
    scale_factor <- target / max_ev
    Phi <- Phi * scale_factor
  }
  Phi
}


#' Generate default ordered cutpoints for ordinal model
#' @noRd
generate_default_kappa <- function(C) {
  # C-1 cutpoints evenly spaced from approximately -2 to 2
  seq(-2, 2, length.out = C - 1L)
}


#' Generate response for a single time point (all p nodes)
#'
#' @param eta Numeric vector of length p. Linear predictor per node.
#' @param family Character. Model family.
#' @param sigma Numeric vector (gaussian only).
#' @param kappa List of p vectors (ordinal only).
#' @param C Integer (ordinal only).
#' @param p Integer. Number of nodes.
#' @return Vector of length p with generated responses.
#' @noRd
generate_response <- function(eta, family, sigma = NULL, kappa = NULL,
                              C = NULL, p) {
  switch(family,
    bernoulli = generate_response_binary(eta),
    gaussian  = generate_response_gaussian(eta, sigma),
    ordinal   = generate_response_ordinal(eta, kappa, C, p),
    stop("Unknown family: ", family)
  )
}


#' Generate response for a single node (scalar eta)
#'
#' @param eta Numeric scalar. Linear predictor for this node.
#' @param family Character. Family for this node.
#' @param sigma Numeric scalar (gaussian) or NA.
#' @param kappa Numeric vector of cutpoints (ordinal) or NULL.
#' @return Scalar response value.
#' @noRd
generate_response_node <- function(eta, family, sigma = NA_real_,
                                   kappa = NULL) {
  switch(family,
    bernoulli = rbinom(1L, 1L, 1 / (1 + exp(-eta))),
    gaussian  = rnorm(1L, eta, sigma),
    ordinal   = {
      kappa_cumsum <- c(0, cumsum(kappa))
      C_j <- length(kappa) + 1L
      lambda <- (seq_len(C_j) - 1L) * eta - kappa_cumsum
      lambda <- lambda - max(lambda)
      probs  <- exp(lambda) / sum(exp(lambda))
      sample.int(C_j, 1L, prob = probs)
    },
    stop("Unknown family: ", family)
  )
}


#' Binary response: logistic link + Bernoulli draw
#' @noRd
generate_response_binary <- function(eta) {
  prob <- 1 / (1 + exp(-eta))
  rbinom(length(eta), 1L, prob)
}


#' Gaussian response: identity link + normal noise
#' @noRd
generate_response_gaussian <- function(eta, sigma) {
  rnorm(length(eta), mean = eta, sd = sigma)
}


#' Ordinal response: adjacent-category logit
#'
#' Matches the Stan parameterisation:
#'   lambda[c] = (c-1) * eta - kappa_cumsum[c]
#'   P(Y = c) = exp(lambda[c]) / sum(exp(lambda))
#' @noRd
generate_response_ordinal <- function(eta, kappa, C, p) {
  y <- integer(p)
  for (node in seq_len(p)) {
    # Cumulative sum of kappa for this node
    kappa_cumsum <- c(0, cumsum(kappa[[node]]))

    # Adjacent-category log-odds
    lambda <- (seq_len(C) - 1L) * eta[node] - kappa_cumsum

    # Softmax with log-sum-exp stability
    lambda <- lambda - max(lambda)
    probs <- exp(lambda) / sum(exp(lambda))

    y[node] <- sample.int(C, size = 1L, prob = probs)
  }
  y
}


#' Assemble Y and X arrays into a long-format data frame
#' @noRd
assemble_long_df <- function(Y, X_cov, N, T_obs, p, q) {
  rows <- vector("list", N)
  for (i in seq_len(N)) {
    row_data <- data.frame(
      id = rep(i, T_obs),
      t  = seq_len(T_obs)
    )
    # y columns
    for (j in seq_len(p)) {
      row_data[[paste0("y_", j)]] <- Y[i, , j]
    }
    # x columns
    if (q > 0L) {
      for (j in seq_len(q)) {
        row_data[[paste0("x_", j)]] <- X_cov[i, , j]
      }
    }
    rows[[i]] <- row_data
  }
  do.call(rbind, rows)
}


# ──────────────────────────────────────────────────────────────────────────────
# compare_to_truth() — compare fitted parameters to generating truth
# ──────────────────────────────────────────────────────────────────────────────

#' Compare fitted model parameters to simulation truth
#'
#' Extracts posterior summaries from a fitted \code{bvarnet} object and
#' compares them to the true parameter values used for data generation.
#'
#' @param fit A fitted \code{bvarnet} object (output from \code{bvar()}).
#' @param truth The \code{truth} component from \code{sim_var()} output.
#' @param ci_width Numeric. Width of the credible interval (default 0.90).
#' @param bayes_factor Logical; if \code{TRUE}, compute Savage-Dickey BFs for
#'   beta and phi parameters and append \code{BF01}, \code{BF10}, and
#'   \code{bf_correct} columns.  \code{bf_correct} is \code{TRUE} when BF01 > 1
#'   for true null parameters (true value == \code{null_value}) and BF10 > 1
#'   for true non-null parameters.  Default \code{FALSE}.
#' @param null_value Numeric scalar; the null hypothesis value for Bayes
#'   factor computation (default 0).  Only used when \code{bayes_factor = TRUE}.
#'
#' @return A data frame with columns: parameter, node, true_value,
#'   post_mean, post_sd, ci_lower, ci_upper, covered (logical), and optionally
#'   BF01, BF10, bf_correct.
#'
#' @export
compare_to_truth <- function(fit, truth, ci_width = 0.90,
                             bayes_factor = FALSE, null_value = 0) {
  stopifnot(inherits(fit, "bvarnet"))

  alpha_lo <- (1 - ci_width) / 2
  alpha_hi <- 1 - alpha_lo
  p <- truth$p
  family_vec <- truth$family
  if (length(family_vec) == 1L)
    family_vec <- rep(family_vec, p)

  results <- list()

  # ── beta (intercept + covariates) ────────────────────────────────────────
  draws_beta <- extract_draws(fit, "beta")
  # Stan layout: beta[fe_idx, node]
  n_fe <- fit$standata$n_fe
  for (node in seq_len(p)) {
    family_node <- family_vec[node]
    for (fe in seq_len(n_fe)) {
      par_name <- paste0("beta[", fe, ",", node, "]")
      d <- draws_beta[, par_name]

      # Map to truth:
      #   fe=1 for bernoulli/gaussian is intercept → alpha[node]
      #   fe=1 for ordinal is NA sentinel (skip)
      #   fe>1 is gamma[fe-1, node] (bernoulli/gaussian) or gamma[fe, node] (ordinal)
      if (family_node %in% c("bernoulli", "gaussian") && fe == 1L) {
        true_val <- truth$alpha[node]
        par_label <- "intercept"
      } else if (family_node == "ordinal" && fe == 1L) {
        # NA sentinel — skip ordinal intercept row
        next
      } else {
        gamma_idx <- if (family_node == "ordinal") fe else fe - 1L
        true_val <- if (!is.null(truth$gamma) && gamma_idx <= nrow(truth$gamma)) {
          truth$gamma[gamma_idx, node]
        } else {
          NA_real_
        }
        par_label <- paste0("gamma_", gamma_idx)
      }

      results[[length(results) + 1L]] <- data.frame(
        parameter  = par_label,
        node       = node,
        true_value = true_val,
        post_mean  = mean(d),
        post_sd    = sd(d),
        ci_lower   = unname(quantile(d, alpha_lo)),
        ci_upper   = unname(quantile(d, alpha_hi)),
        .stan_name = par_name,
        stringsAsFactors = FALSE
      )
    }
  }

  # ── phi (lag coefficients) ──────────────────────────────────────────────
  draws_phi <- extract_draws(fit, "phi")
  PK <- p * truth$K
  for (node in seq_len(p)) {
    for (lag_idx in seq_len(PK)) {
      par_name <- paste0("phi[", lag_idx, ",", node, "]")
      d <- draws_phi[, par_name]
      true_val <- truth$Phi[lag_idx, node]

      results[[length(results) + 1L]] <- data.frame(
        parameter  = "phi",
        node       = node,
        true_value = true_val,
        post_mean  = mean(d),
        post_sd    = sd(d),
        ci_lower   = unname(quantile(d, alpha_lo)),
        ci_upper   = unname(quantile(d, alpha_hi)),
        .stan_name = par_name,
        stringsAsFactors = FALSE
      )
    }
  }

  # ── sigma (gaussian nodes) ───────────────────────────────────────────
  gauss_nodes <- which(family_vec == "gaussian")
  if (length(gauss_nodes) > 0L) {
    draws_sigma <- extract_draws(fit, "sigma")
    for (node in gauss_nodes) {
      par_name <- paste0("sigma[", node, "]")
      d <- draws_sigma[, par_name]

      results[[length(results) + 1L]] <- data.frame(
        parameter  = "sigma",
        node       = node,
        true_value = truth$sigma[node],
        post_mean  = mean(d),
        post_sd    = sd(d),
        ci_lower   = unname(quantile(d, alpha_lo)),
        ci_upper   = unname(quantile(d, alpha_hi)),
        .stan_name = par_name,
        stringsAsFactors = FALSE
      )
    }
  }

  # ── kappa (ordinal nodes) ───────────────────────────────────────────
  ord_nodes <- which(family_vec == "ordinal")
  if (length(ord_nodes) > 0L) {
    draws_kappa <- extract_draws(fit, "kappa")
    C <- truth$C
    for (node in ord_nodes) {
      for (k in seq_len(C - 1L)) {
        par_name <- paste0("kappa[", node, ",", k, "]")
        d <- draws_kappa[, par_name]

        results[[length(results) + 1L]] <- data.frame(
          parameter  = "kappa",
          node       = node,
          true_value = truth$kappa[[node]][k],
          post_mean  = mean(d),
          post_sd    = sd(d),
          ci_lower   = unname(quantile(d, alpha_lo)),
          ci_upper   = unname(quantile(d, alpha_hi)),
          .stan_name = par_name,
          stringsAsFactors = FALSE
        )
      }
    }
  }

  # ── Combine and add coverage indicator ─────────────────────────────────
  out <- do.call(rbind, results)
  out$covered <- out$true_value >= out$ci_lower & out$true_value <= out$ci_upper
  rownames(out) <- NULL

  # ── Bayes factors for beta and phi ─────────────────────────────────────
  if (isTRUE(bayes_factor)) {
    out$BF01 <- NA_real_
    out$BF10 <- NA_real_
    out$bf_correct <- NA

    bf_params <- c("intercept", "phi")
    bf_idx <- which(out$parameter %in% bf_params |
                    grepl("^gamma_", out$parameter))

    for (i in bf_idx) {
      res <- tryCatch(
        savage_dickey(fit, params = out$.stan_name[i],
                      null_value = null_value, method = "logspline"),
        error = function(e) NULL
      )
      if (!is.null(res)) {
        out$BF01[i] <- res$BF01
        out$BF10[i] <- res$BF10
        is_null <- isTRUE(all.equal(out$true_value[i], null_value))
        out$bf_correct[i] <- if (is_null) res$BF01 > 1 else res$BF10 > 1
      }
    }
  }

  # Drop internal helper column
  out$.stan_name <- NULL
  out
}

# test-mixed-integration.R — mock-based tests for mixed-family support
# ──────────────────────────────────────────────────────────────────────────────

# ── Mock construction ────────────────────────────────────────────────────────

test_that("make_mock_bvarnet() creates valid mixed GB object", {
  obj <- make_mock_bvarnet(family = c("gaussian", "bernoulli"))
  expect_s3_class(obj, "bvarnet")
  expect_equal(as.character(obj$family), c("gaussian", "bernoulli"))
  expect_equal(names(obj$family), c("y_1", "y_2"))

  par_nms <- dimnames(obj$draws)[[3]]
  # sigma only for node 1 (gaussian)
  expect_true("sigma[1]" %in% par_nms)
  expect_false("sigma[2]" %in% par_nms)
  # no kappa
  expect_false(any(grepl("^kappa\\[", par_nms)))
})

test_that("make_mock_bvarnet() creates valid mixed GO object", {
  obj <- make_mock_bvarnet(family = c("gaussian", "ordinal"))
  par_nms <- dimnames(obj$draws)[[3]]
  # sigma only for node 1

  expect_true("sigma[1]" %in% par_nms)
  expect_false("sigma[2]" %in% par_nms)
  # kappa only for node 2
  expect_true("kappa[2,1]" %in% par_nms)
  expect_true("kappa[2,2]" %in% par_nms)
  expect_false("kappa[1,1]" %in% par_nms)
  # ordinal beta[1,2] is NA sentinel
  expect_true(all(is.na(obj$draws[, , "beta[1,2]"])))
  # gaussian beta[1,1] is real
  expect_false(any(is.na(obj$draws[, , "beta[1,1]"])))
})

test_that("make_mock_bvarnet() creates valid mixed BO object", {
  obj <- make_mock_bvarnet(family = c("bernoulli", "ordinal"))
  par_nms <- dimnames(obj$draws)[[3]]
  expect_false(any(grepl("^sigma\\[", par_nms)))
  expect_true("kappa[2,1]" %in% par_nms)
  expect_true(all(is.na(obj$draws[, , "beta[1,2]"])))
})

# ── extract_param on mixed objects ───────────────────────────────────────────

test_that("extract_param() works on mixed GB mock", {
  obj <- make_mock_bvarnet(family = c("gaussian", "bernoulli"))
  result <- extract_param(obj)
  expect_s3_class(result, "data.frame")

  # sigma rows only for gaussian nodes
  sigma_rows <- result[result$type == "Residual SD", ]
  expect_equal(nrow(sigma_rows), 1L)
  expect_equal(sigma_rows$predictor, "y_1")

  # no kappa rows
  kappa_rows <- result[result$type == "Threshold", ]
  expect_equal(nrow(kappa_rows), 0L)
})

test_that("extract_param() filters ordinal intercept rows in mixed GO mock", {
  obj <- make_mock_bvarnet(family = c("gaussian", "ordinal"))
  result <- extract_param(obj)

  # beta section: ordinal node should NOT have an Intercept row
  intercept_rows <- result[result$type == "Intercept", ]
  ord_intercept <- intercept_rows[intercept_rows$outcome == "y_2", ]
  expect_equal(nrow(ord_intercept), 0L)

  # gaussian node SHOULD have an Intercept row
  gauss_intercept <- intercept_rows[intercept_rows$outcome == "y_1", ]
  expect_equal(nrow(gauss_intercept), 1L)
})

# ── print method on mixed objects ────────────────────────────────────────────

test_that("print.bvarnet() works on mixed objects", {
  obj <- make_mock_bvarnet(family = c("gaussian", "bernoulli"))
  out <- capture.output(print(obj))
  out_str <- paste(out, collapse = "\n")
  expect_true(grepl("gaussian", out_str))
  expect_true(grepl("bernoulli", out_str))
})

# ── sim_var with mixed family ───────────────────────────────────────────────

test_that("sim_var() generates data with mixed GB family", {
  sim <- sim_var(N = 5, T_obs = 20, p = 2, family = c("gaussian", "bernoulli"),
                 seed = 100)
  expect_true(is.data.frame(sim$data))
  expect_equal(as.character(sim$truth$family), c("gaussian", "bernoulli"))
  # sigma: non-NA for gaussian node
  expect_false(is.na(sim$truth$sigma[1]))
  expect_true(is.na(sim$truth$sigma[2]))
})

test_that("sim_var() generates data with mixed GO family", {
  sim <- sim_var(N = 5, T_obs = 20, p = 2, family = c("gaussian", "ordinal"),
                 C = 4, seed = 101)
  expect_equal(sim$truth$C, 4L)
  expect_true(is.numeric(sim$truth$kappa[[2]]))
  expect_null(sim$truth$kappa[[1]])
})

test_that("sim_var() generates data with mixed GBO family (p=3)", {
  sim <- sim_var(N = 5, T_obs = 20, p = 3,
                 family = c("gaussian", "bernoulli", "ordinal"),
                 C = 3, seed = 102)
  expect_equal(length(sim$truth$family), 3L)
  # y columns present
  expect_true(all(c("y_1", "y_2", "y_3") %in% names(sim$data)))
})

test_that("generate_response_node() dispatches correctly", {
  set.seed(99)
  # bernoulli
  y_b <- generate_response_node(0, "bernoulli")
  expect_true(y_b %in% c(0L, 1L))
  # gaussian
  y_g <- generate_response_node(0, "gaussian", sigma = 1)
  expect_true(is.numeric(y_g))
  # ordinal
  y_o <- generate_response_node(0, "ordinal", kappa = c(-1, 0, 1))
  expect_true(y_o %in% 1:4)
})

# ──────────────────────────────────────────────────────────────────────────────
# test-sim_bvarnet.R — simulation function tests
#
# Follows the bgms stochastic-robust testing philosophy:
#   • Range invariants (binary ∈ {0,1}, ordinal ∈ {1,...,C}, gaussian finite)
#   • Dimension consistency (N × T_obs × p layout)
#   • Seed reproducibility
#   • Parameter effect direction (coarse, wide tolerance)
#   • Burnin behaviour
#   • VAR stability enforcement
# ──────────────────────────────────────────────────────────────────────────────

# ═══════════════════════════════════════════════════════════════════════════════
# §1 Return structure
# ═══════════════════════════════════════════════════════════════════════════════

test_that("sim_var returns correct structure for all families", {
  for (fam in c("bernoulli", "gaussian", "ordinal")) {
    result <- sim_var(N = 3, T_obs = 20, p = 2, K = 1,
                      family = fam, q = 1, seed = 1, burnin = 10)

    expect_type(result, "list")
    expect_named(result, c("data", "truth"))

    # data is a data.frame
    expect_s3_class(result$data, "data.frame")

    # truth is a list
    expect_type(result$truth, "list")

    # truth always contains these core elements
    expect_true(all(c("alpha", "Phi", "family", "N", "T_obs", "p", "K", "q",
                       "sd_alpha", "alpha_i", "Phi_i", "burnin") %in%
                      names(result$truth)))
  }
})


test_that("truth structure varies correctly by family", {
  gauss <- sim_var(N = 3, T_obs = 20, p = 2, family = "gaussian", seed = 1, burnin = 10)
  expect_true("sigma" %in% names(gauss$truth))
  expect_true(all(!is.na(gauss$truth$sigma)))
  # kappa entries are NULL for non-ordinal

  expect_true(all(vapply(gauss$truth$kappa, is.null, logical(1L))))
  expect_null(gauss$truth$C)

  ordinal <- sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal", seed = 1, burnin = 10)
  expect_true("kappa" %in% names(ordinal$truth))
  expect_true("C" %in% names(ordinal$truth))
  # sigma is NA for ordinal nodes
  expect_true(all(is.na(ordinal$truth$sigma)))

  bern <- sim_var(N = 3, T_obs = 20, p = 2, family = "bernoulli", seed = 1, burnin = 10)
  expect_true(all(is.na(bern$truth$sigma)))
  expect_true(all(vapply(bern$truth$kappa, is.null, logical(1L))))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §2 Dimension consistency
# ═══════════════════════════════════════════════════════════════════════════════

test_that("output data frame has correct dimensions and columns", {
  N <- 5; T_obs <- 30; p <- 3; q <- 2
  result <- sim_var(N = N, T_obs = T_obs, p = p, q = q,
                    family = "bernoulli", seed = 10, burnin = 50)
  df <- result$data

  expect_equal(nrow(df), N * T_obs)
  expect_true("id" %in% names(df))
  expect_true("t"  %in% names(df))

  y_cols <- paste0("y_", 1:p)
  x_cols <- paste0("x_", 1:q)
  expect_true(all(y_cols %in% names(df)))
  expect_true(all(x_cols %in% names(df)))
})


test_that("truth matrices have correct dimensions", {
  N <- 4; p <- 3; K <- 2; q <- 2
  result <- sim_var(N = N, T_obs = 20, p = p, K = K, q = q,
                    family = "gaussian", seed = 20, burnin = 10)
  truth <- result$truth

  expect_equal(length(truth$alpha), p)
  expect_equal(dim(truth$Phi), c(p * K, p))
  expect_equal(dim(truth$gamma), c(q, p))
  expect_equal(length(truth$sigma), p)
  expect_equal(dim(truth$alpha_i), c(N, p))
  expect_equal(dim(truth$Phi_i), c(N, p * K, p))
})


test_that("ordinal kappa has correct structure", {
  p <- 3; C <- 5
  result <- sim_var(N = 3, T_obs = 20, p = p, family = "ordinal",
                    C = C, seed = 30, burnin = 10)
  kappa <- result$truth$kappa

  expect_type(kappa, "list")
  expect_length(kappa, p)

  for (node in seq_len(p)) {
    expect_length(kappa[[node]], C - 1)
    # cutpoints must be strictly ordered
    expect_true(!is.unsorted(kappa[[node]], strictly = TRUE))
  }
})


# ═══════════════════════════════════════════════════════════════════════════════
# §3 Range invariants
# ═══════════════════════════════════════════════════════════════════════════════

test_that("bernoulli responses are 0 or 1", {
  result <- sim_var(N = 10, T_obs = 40, p = 4, family = "bernoulli",
                    seed = 100, burnin = 50)
  y_cols <- paste0("y_", 1:4)
  for (col in y_cols) {
    vals <- result$data[[col]]
    expect_true(all(vals %in% c(0L, 1L)))
  }
})


test_that("ordinal responses are in {1, ..., C}", {
  C <- 5
  result <- sim_var(N = 10, T_obs = 40, p = 4, family = "ordinal",
                    C = C, seed = 101, burnin = 50)
  y_cols <- paste0("y_", 1:4)
  for (col in y_cols) {
    vals <- result$data[[col]]
    expect_true(all(vals %in% 1:C))
  }
})


test_that("gaussian responses are finite", {
  result <- sim_var(N = 10, T_obs = 40, p = 4, family = "gaussian",
                    seed = 102, burnin = 50)
  y_cols <- paste0("y_", 1:4)
  for (col in y_cols) {
    vals <- result$data[[col]]
    expect_true(all(is.finite(vals)))
  }
})


# ═══════════════════════════════════════════════════════════════════════════════
# §4 Seed reproducibility
# ═══════════════════════════════════════════════════════════════════════════════

test_that("same seed produces identical output", {
  r1 <- sim_var(N = 5, T_obs = 20, p = 2, family = "bernoulli",
                q = 1, seed = 777, burnin = 10)
  r2 <- sim_var(N = 5, T_obs = 20, p = 2, family = "bernoulli",
                q = 1, seed = 777, burnin = 10)

  expect_identical(r1$data, r2$data)
  expect_identical(r1$truth$Phi, r2$truth$Phi)
  expect_identical(r1$truth$alpha, r2$truth$alpha)
  expect_identical(r1$truth$alpha_i, r2$truth$alpha_i)
})


test_that("different seeds produce different output", {
  r1 <- sim_var(N = 5, T_obs = 20, p = 2, family = "bernoulli",
                q = 1, seed = 777, burnin = 10)
  r2 <- sim_var(N = 5, T_obs = 20, p = 2, family = "bernoulli",
                q = 1, seed = 888, burnin = 10)

  expect_false(identical(r1$data, r2$data))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §5 Burnin behaviour
# ═══════════════════════════════════════════════════════════════════════════════

test_that("burnin = 0 is allowed and produces T_obs rows per subject", {
  result <- sim_var(N = 3, T_obs = 20, p = 2, family = "bernoulli",
                    seed = 50, burnin = 0)
  expect_equal(nrow(result$data), 3 * 20)
  expect_equal(result$truth$burnin, 0L)
})


test_that("increasing burnin does not change output dimensions", {
  r1 <- sim_var(N = 3, T_obs = 20, p = 2, family = "gaussian",
                seed = 51, burnin = 10)
  r2 <- sim_var(N = 3, T_obs = 20, p = 2, family = "gaussian",
                seed = 51, burnin = 500)

  expect_equal(nrow(r1$data), nrow(r2$data))
  expect_equal(r1$truth$T_obs, r2$truth$T_obs)
})


test_that("burnin and non-burnin produce different data (same seed)", {
  r1 <- sim_var(N = 5, T_obs = 30, p = 2, family = "gaussian",
                seed = 52, burnin = 0)
  r2 <- sim_var(N = 5, T_obs = 30, p = 2, family = "gaussian",
                seed = 52, burnin = 500)

  # Same seed but different burnin → different recorded data
  expect_false(identical(r1$data, r2$data))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §6 VAR stability
# ═══════════════════════════════════════════════════════════════════════════════

test_that("auto-generated Phi is always stable", {
  for (trial in 1:5) {
    result <- sim_var(N = 3, T_obs = 20, p = 4, K = 2,
                      family = "gaussian", seed = trial, burnin = 10)
    Phi <- result$truth$Phi
    p <- result$truth$p; K <- result$truth$K

    companion <- bvarnet:::build_companion(Phi, p, K)
    max_ev <- max(abs(eigen(companion, only.values = TRUE)$values))
    expect_lt(max_ev, 1.0)
  }
})


test_that("unstable user-supplied Phi triggers warning and rescaling", {
  # Create an obviously unstable Phi (diagonal >> 1)
  unstable_Phi <- matrix(0, 3, 3)
  diag(unstable_Phi) <- 2.0

  expect_warning(
    result <- sim_var(N = 3, T_obs = 20, p = 3, K = 1,
                      family = "gaussian", Phi = unstable_Phi,
                      seed = 60, burnin = 10),
    "not VAR-stable"
  )

  # After rescaling, stability should hold
  Phi <- result$truth$Phi
  expect_true(bvarnet:::check_var_stability(Phi, 3, 1))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §7 No random effects mode (sd_alpha = 0)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("sd_alpha = 0 produces identical intercepts across subjects", {
  result <- sim_var(N = 10, T_obs = 20, p = 3, family = "gaussian",
                    sd_alpha = 0, seed = 70, burnin = 10)

  alpha_i <- result$truth$alpha_i
  # All rows should equal the population alpha
  for (i in seq_len(nrow(alpha_i))) {
    expect_equal(alpha_i[i, ], result$truth$alpha, tolerance = 1e-12)
  }
})


test_that("re_temporal = TRUE produces person-varying lag coefficients", {
  result <- sim_var(N = 5, T_obs = 20, p = 2, K = 1,
                    family = "gaussian", re_temporal = TRUE,
                    sd_phi = 0.3, seed = 71, burnin = 10)

  Phi_i <- result$truth$Phi_i
  # Check that at least two subjects differ in Phi
  expect_false(identical(Phi_i[1, , ], Phi_i[2, , ]))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §8 Covariate generation
# ═══════════════════════════════════════════════════════════════════════════════

test_that("covariates are included when q > 0", {
  result <- sim_var(N = 3, T_obs = 20, p = 2, q = 3,
                    family = "bernoulli", seed = 80, burnin = 10)
  df <- result$data

  x_cols <- paste0("x_", 1:3)
  expect_true(all(x_cols %in% names(df)))

  # truth should have gamma

  expect_true(!is.null(result$truth$gamma))
  expect_equal(dim(result$truth$gamma), c(3, 2))
})


test_that("no covariate columns when q = 0", {
  result <- sim_var(N = 3, T_obs = 20, p = 2, q = 0,
                    family = "bernoulli", seed = 81, burnin = 10)
  df <- result$data

  expect_false(any(grepl("^x_", names(df))))
  expect_null(result$truth$gamma)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §9 K > 1 (higher-order VAR)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("K = 2 produces correct Phi dimensions and valid output", {
  result <- sim_var(N = 3, T_obs = 30, p = 3, K = 2,
                    family = "gaussian", seed = 90, burnin = 10)

  expect_equal(dim(result$truth$Phi), c(6, 3))  # p*K x p
  expect_equal(nrow(result$data), 3 * 30)
  expect_true(all(is.finite(result$data$y_1)))
})


# ═══════════════════════════════════════════════════════════════════════════════
# §10 User-supplied parameters
# ═══════════════════════════════════════════════════════════════════════════════

test_that("user-supplied alpha is stored in truth", {
  my_alpha <- c(0.5, -0.5)
  result <- sim_var(N = 3, T_obs = 20, p = 2, family = "gaussian",
                    alpha = my_alpha, seed = 95, burnin = 10)
  expect_equal(result$truth$alpha, my_alpha)
})


test_that("user-supplied Phi (stable) is preserved", {
  Phi <- matrix(c(0.3, 0.1, -0.1, 0.4), 2, 2)
  result <- sim_var(N = 3, T_obs = 20, p = 2, K = 1,
                    family = "gaussian", Phi = Phi, seed = 96, burnin = 10)
  expect_equal(result$truth$Phi, Phi)
})


test_that("user-supplied sigma is preserved in gaussian truth", {
  my_sigma <- c(1.0, 2.0, 0.5)
  result <- sim_var(N = 3, T_obs = 20, p = 3, family = "gaussian",
                    sigma = my_sigma, seed = 97, burnin = 10)
  expect_equal(result$truth$sigma, my_sigma)
})


test_that("user-supplied kappa is preserved in ordinal truth", {
  my_kappa <- list(c(-2, -1, 0, 1), c(-1.5, -0.5, 0.5, 1.5))
  result <- sim_var(N = 3, T_obs = 20, p = 2, family = "ordinal",
                    C = 5, kappa = my_kappa, seed = 98, burnin = 10)
  expect_equal(result$truth$kappa, my_kappa)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §11 Internal helpers
# ═══════════════════════════════════════════════════════════════════════════════

test_that("build_companion returns correct dimensions", {
  p <- 3; K <- 2
  Phi <- matrix(runif(p * K * p, -0.3, 0.3), p * K, p)
  comp <- bvarnet:::build_companion(Phi, p, K)
  expect_equal(dim(comp), c(p * K, p * K))
})


test_that("build_companion for K=1 equals t(Phi)", {
  Phi <- matrix(c(0.3, 0.1, 0.05, 0.4), 2, 2)
  comp <- bvarnet:::build_companion(Phi, 2, 1)
  expect_equal(comp, t(Phi))
})


test_that("rescale_to_stable makes unstable Phi stable", {
  Phi <- matrix(0, 3, 3)
  diag(Phi) <- 1.5  # unstable
  Phi_stable <- bvarnet:::rescale_to_stable(Phi, 3, 1)
  expect_true(bvarnet:::check_var_stability(Phi_stable, 3, 1))
})


test_that("generate_default_kappa returns ordered cutpoints", {
  for (C in c(2, 3, 5, 10)) {
    kappa <- bvarnet:::generate_default_kappa(C)
    expect_length(kappa, C - 1)
    expect_true(!is.unsorted(kappa, strictly = TRUE))
  }
})


test_that("generate_response_binary returns 0/1 integers", {
  set.seed(200)
  eta <- rnorm(100)
  y <- bvarnet:::generate_response_binary(eta)
  expect_true(all(y %in% c(0L, 1L)))
  expect_length(y, 100)
})


test_that("generate_response_gaussian returns finite values", {
  set.seed(201)
  eta <- rnorm(50)
  sigma <- rep(1, 50)
  y <- bvarnet:::generate_response_gaussian(eta, sigma)
  expect_true(all(is.finite(y)))
  expect_length(y, 50)
})


test_that("generate_response_ordinal returns values in {1, ..., C}", {
  set.seed(202)
  p <- 5; C <- 4
  eta <- rnorm(p)
  kappa <- replicate(p, sort(rnorm(C - 1)), simplify = FALSE)
  y <- bvarnet:::generate_response_ordinal(eta, kappa, C, p)
  expect_true(all(y %in% 1:C))
  expect_length(y, p)
})


# ═══════════════════════════════════════════════════════════════════════════════
# §12 Stochastic property tests (wide tolerance)
# ═══════════════════════════════════════════════════════════════════════════════

test_that("bernoulli: large positive intercept → high marginal P(Y=1)", {
  # With very large intercepts, most responses should be 1
  result <- sim_var(N = 20, T_obs = 100, p = 2, family = "bernoulli",
                    alpha = c(5, 5), sd_alpha = 0, burnin = 100, seed = 300)
  y_cols <- paste0("y_", 1:2)
  for (col in y_cols) {
    prop <- mean(result$data[[col]])
    expect_gt(prop, 0.8)
  }
})


test_that("bernoulli: large negative intercept → low marginal P(Y=1)", {
  result <- sim_var(N = 20, T_obs = 100, p = 2, family = "bernoulli",
                    alpha = c(-5, -5), sd_alpha = 0, burnin = 100, seed = 301)
  y_cols <- paste0("y_", 1:2)
  for (col in y_cols) {
    prop <- mean(result$data[[col]])
    expect_lt(prop, 0.2)
  }
})


test_that("gaussian: zero Phi → means approx equal to intercept", {
  alpha_true <- c(3, -2, 0.5)
  Phi_zero <- matrix(0, 3, 3)
  result <- sim_var(N = 50, T_obs = 100, p = 3, K = 1,
                    family = "gaussian", alpha = alpha_true,
                    Phi = Phi_zero, sd_alpha = 0, burnin = 100, seed = 302)

  for (j in 1:3) {
    col <- paste0("y_", j)
    obs_mean <- mean(result$data[[col]])
    # Wide tolerance: within 0.5 of true intercept
    expect_equal(obs_mean, alpha_true[j], tolerance = 0.5)
  }
})


test_that("ordinal: extreme kappa shifts marginal distribution", {
  # Very negative kappa → most mass on higher categories
  kappa_low <- list(c(-10, -9, -8, -7), c(-10, -9, -8, -7))
  result_low <- sim_var(N = 20, T_obs = 100, p = 2, family = "ordinal",
                        C = 5, kappa = kappa_low, sd_alpha = 0,
                        burnin = 100, seed = 303)

  # Very positive kappa → most mass on lower categories
  kappa_high <- list(c(7, 8, 9, 10), c(7, 8, 9, 10))
  result_high <- sim_var(N = 20, T_obs = 100, p = 2, family = "ordinal",
                         C = 5, kappa = kappa_high, sd_alpha = 0,
                         burnin = 100, seed = 304)

  mean_low  <- mean(result_low$data$y_1)
  mean_high <- mean(result_high$data$y_1)

  # Low kappa → higher category mean, High kappa → lower category mean
  expect_gt(mean_low, mean_high)
})

# -----------------------------------------------------------------------------
# Savage-Dickey density ratio Bayes factor estimation
#
# Core functions:
#   - bf_table()          : primary export — tidy data frame of BFs
#   - savage_dickey()     : internal workhorse (named list per test)
#   - eval_prior_density(): analytical prior density evaluation
#   - get_phi_indices()   : resolve AR / CL / all phi parameter names
# -----------------------------------------------------------------------------


# ---- Prior density evaluation ------------------------------------------------

#' Evaluate the prior density at a point (internal)
#'
#' Computes the analytical density of a \code{bvarnet_prior} at \code{x}.
#'
#' @param prior A \code{bvarnet_prior} object.
#' @param x     Numeric scalar — the point at which to evaluate the density.
#'
#' @return Numeric scalar — the density.
#' @keywords internal
#' @noRd
eval_prior_density <- function(prior, x) {
  # TODO: if stationarity constraints are added to Phi, the effective prior
	#       becomes truncated and this analytical density must be corrected.
  switch(prior$family,
    normal    = stats::dnorm(x,  mean = prior$loc, sd = prior$scale),
    student_t = stats::dt((x - prior$loc) / prior$scale, df = prior$df) / prior$scale,
    cauchy    = stats::dcauchy(x, location = prior$loc, scale = prior$scale),
    stop("Unknown prior family: ", prior$family, call. = FALSE)
  )
}


#' Evaluate the joint prior density for independent priors (internal)
#'
#' @param prior_list A named list of \code{bvarnet_prior} objects keyed by
#'   parameter type (e.g., \code{"phi"}, \code{"beta"}).
#' @param param_types Character vector of prior types for each parameter.
#' @param null_vec Numeric vector of null values (same length as
#'   \code{param_types}).
#'
#' @return Numeric scalar — the product of marginal densities.
#' @keywords internal
#' @noRd
eval_joint_prior_density <- function(prior_list, param_types, null_vec) {
  prod(mapply(function(ptype, x) {
    eval_prior_density(prior_list[[ptype]], x)
  }, param_types, null_vec, USE.NAMES = FALSE))
}


# ---- Phi index resolution ----------------------------------------------------

#' Resolve Stan parameter names for phi sub-matrices (internal)
#'
#' Returns a character vector of Stan parameter names (e.g.,
#' \code{"phi[1,1]"}) for autoregressive, cross-lagged, or all effects.
#'
#' @param sd      The \code{standata} list from a \code{bvarnet} object.
#' @param lag     Integer; which lag block (1 to K). Default 1.
#' @param effect  One of \code{"ar"}, \code{"cl"}, \code{"all"}.
#'
#' @return Character vector of Stan parameter names.
#' @keywords internal
#' @noRd
get_phi_indices <- function(sd, lag = 1L, effect = c("ar", "cl", "all")) {
  effect <- match.arg(effect)
  p <- sd$p
  K <- sd$K
  if (lag < 1L || lag > K)
    stop(sprintf("'lag' must be between 1 and K = %d.", K), call. = FALSE)

  # phi is matrix[p*K, p] — phi[row, col]
  # For lag block k: rows (k-1)*p + 1 .. k*p
  # AR: row = (k-1)*p + j, col = j  (self-loops)
  # CL: everything else in that lag block
  offset <- (lag - 1L) * p
  nms    <- character(0)

  for (col in seq_len(p)) {
    for (row_within in seq_len(p)) {
      row <- offset + row_within
      is_ar <- (row_within == col)
      include <- switch(effect,
        ar  = is_ar,
        cl  = !is_ar,
        all = TRUE
      )
      if (include)
        nms <- c(nms, sprintf("phi[%d,%d]", row, col))
    }
  }
  nms
}


#' Resolve Stan parameter names for beta sub-groups (internal)
#'
#' @param sd   The \code{standata} list.
#' @param type One of \code{"intercepts"} or \code{"fe"} (non-intercept fixed effects).
#' @return Character vector of Stan parameter names.
#' @keywords internal
#' @noRd
get_beta_indices <- function(sd, type = c("intercepts", "fe")) {
  type  <- match.arg(type)
  p     <- sd$p
  n_fe  <- sd$n_fe
  has_intercept <- "Intercept" %in% colnames(sd$X)

  # Guard: ordinal models have no intercept in X

  if (type == "intercepts" && !has_intercept)
    stop(
      '`type = "intercepts"` is not valid for ordinal models: ',
      "beta row 1 is a covariate, not an intercept. ",
      "Cutpoints are stored in `kappa`.",
      call. = FALSE
    )

  nms <- character(0)
  if (type == "intercepts") {
    # Beta row 1 = intercept, across all p columns
    for (col in seq_len(p))
      nms <- c(nms, sprintf("beta[1,%d]", col))
  } else {
    # FE rows start at 2 when Intercept present, 1 otherwise (pure ordinal)
    fe_start <- if (has_intercept) 2L else 1L
    if (n_fe < fe_start)
      stop("No non-intercept fixed effects available.", call. = FALSE)
    for (col in seq_len(p)) {
      for (row in fe_start:n_fe) {
        nms <- c(nms, sprintf("beta[%d,%d]", row, col))
      }
    }
  }
  nms
}


#' Group Stan beta parameter names by predictor row (internal)
#'
#' Returns a named list — one element per predictor row — each element being a
#' character vector of Stan names \code{beta[row, 1:p]} for that row.
#'
#' @param sd   The \code{standata} list.
#' @param type One of \code{"fe"} or \code{"intercepts"}.
#' @return Named list of character vectors keyed by predictor name.
#' @keywords internal
#' @noRd
get_beta_indices_by_predictor <- function(sd, type = c("fe", "intercepts")) {
  type  <- match.arg(type)
  p     <- sd$p
  n_fe  <- sd$n_fe
  fe_names <- colnames(sd$X)
  if (is.null(fe_names)) fe_names <- paste0("fe", seq_len(n_fe))
  has_intercept <- "Intercept" %in% fe_names

  # Guard: ordinal models have no intercept in X
  if (type == "intercepts" && !has_intercept)
    stop(
      '`type = "intercepts"` is not valid for ordinal models: ',
      "beta row 1 is a covariate, not an intercept. ",
      "Cutpoints are stored in `kappa`.",
      call. = FALSE
    )

  fe_start <- if (has_intercept) 2L else 1L
  if (type == "fe" && n_fe < fe_start)
    stop("No non-intercept fixed effects available.", call. = FALSE)
  rows <- if (type == "fe") seq(fe_start, n_fe) else 1L

  stats::setNames(
    lapply(rows, function(r) {
      vapply(seq_len(p), function(col) sprintf("beta[%d,%d]", r, col),
             character(1L))
    }),
    fe_names[rows]
  )
}


#' Group Stan beta names by lag×predictor interaction term (internal)
#'
#' @param sd The \code{standata} list.
#' @return Named list; each element has \code{full} (all params),
#'   \code{by_lag} (list of per-lag-block param vectors), \code{ar}
#'   (AR-like interaction params where lagged outcome == target outcome),
#'   and \code{cl} (CL-like interaction params where lagged outcome !=
#'   target outcome).
#' @keywords internal
#' @noRd
get_lag_interaction_indices_by_term <- function(sd) {
  terms    <- sd$fe_interaction_terms
  fe_ic    <- sd$fe_interaction_colnames
  fe_names <- colnames(sd$X)
  p <- sd$p; K <- sd$K

  if (is.null(terms)) {
    stop("fe_interaction_terms metadata is missing from standata.",
         call. = FALSE)
  }
  if (length(terms) == 0L) return(list())

  out <- list()
  for (fac in terms) {
    has_lag <- any(fac == "lag")
    if (!has_lag) next   # plain x:x interactions handled by per-predictor plan

    suffix <- paste(fac[fac != "lag"], collapse = ":")

    # Use stored colnames; filter by suffix token
    term_ic   <- fe_ic[startsWith(fe_ic, "lag") & endsWith(fe_ic, paste0(":", suffix))]
    beta_rows <- match(term_ic, fe_names)

    if (length(term_ic) == 0L || any(is.na(beta_rows)))
      stop(sprintf("Lag-interaction columns for term '%s' not found in X.", suffix),
           call. = FALSE)

    lag_groups <- lapply(seq_len(K), function(k) {
      row_slice <- beta_rows[((k - 1L) * p + 1L):(k * p)]
      unlist(lapply(row_slice, function(r)
        vapply(seq_len(p), function(col) sprintf("beta[%d,%d]", r, col), character(1L))
      ))
    })
    names(lag_groups) <- paste0("lag", seq_len(K), ":", suffix)

    # AR/CL split: within each lag block, row_slice[j] maps to lagged
    # outcome y_j.  beta[row_for_y_j, col] is AR-like when j == col.
    ar_params <- character(0)
    cl_params <- character(0)
    for (k in seq_len(K)) {
      row_slice <- beta_rows[((k - 1L) * p + 1L):(k * p)]
      for (j in seq_len(p)) {
        r <- row_slice[j]
        for (col in seq_len(p)) {
          pname <- sprintf("beta[%d,%d]", r, col)
          if (j == col) ar_params <- c(ar_params, pname)
          else          cl_params <- c(cl_params, pname)
        }
      }
    }

    out[[suffix]] <- list(
      full   = unlist(lag_groups, use.names = FALSE),
      by_lag = lag_groups,
      ar     = ar_params,
      cl     = cl_params
    )
  }
  out
}





# ---- Density estimation internals -------------------------------------------

#' Compute the SDDR using logspline (internal)
#'
#' @param draws Numeric vector of posterior draws.
#' @param prior A \code{bvarnet_prior} object.
#' @param null  Numeric scalar — null value.
#'
#' @return Named list with elements \code{BF01}, \code{post_density},
#'   \code{prior_density}.
#' @keywords internal
#' @noRd
.compute_sddr_logspline <- function(draws, prior, null = 0) {
  # logspline with tryCatch — fall back to KDE on failure
  fit <- tryCatch(
    logspline::logspline(draws),
    error = function(e) NULL
  )

  if (!is.null(fit)) {
    post_den <- logspline::dlogspline(null, fit)
  } else {
    warning("logspline::logspline() failed; falling back to KDE (stats::density).",
            call. = FALSE)
    kde <- stats::density(draws, n = 4096)
    post_den <- stats::approx(kde$x, kde$y, xout = null, rule = 2)$y
  }

  prior_den <- eval_prior_density(prior, null)

  list(
    BF01          = post_den / prior_den,
    post_density  = post_den,
    prior_density = prior_den
  )
}


#' Compute the SDDR using a multivariate normal approximation (internal)
#'
#' @param draws_mat  Numeric matrix — S rows x d columns of posterior draws.
#' @param prior_list Named list of \code{bvarnet_prior} objects.
#' @param param_types Character vector of prior types for each column.
#' @param null_vec   Numeric vector of null values.
#'
#' @return Named list with elements \code{BF01}, \code{post_density},
#'   \code{prior_density}.
#' @keywords internal
#' @noRd
.compute_sddr_mvn <- function(draws_mat, prior_list, param_types, null_vec) {
  mu_hat    <- colMeans(draws_mat)
  Sigma_hat <- stats::cov(draws_mat)

  post_den  <- mvtnorm::dmvnorm(null_vec, mean = mu_hat, sigma = Sigma_hat)
  prior_den <- eval_joint_prior_density(prior_list, param_types, null_vec)

  list(
    BF01          = post_den / prior_den,
    post_density  = post_den,
    prior_density = prior_den
  )
}


# ---- Main workhorse ----------------------------------------------------------

#' Determine the prior type for a Stan parameter name
#' @keywords internal
#' @noRd
.param_type <- function(param_name) {
  if (grepl("^phi\\[",   param_name)) return("phi")
  if (grepl("^beta\\[",  param_name)) return("beta")
  if (grepl("^sigma\\[", param_name)) return("sigma")
  if (grepl("^sd_u\\[",  param_name)) return("sd_u")
  if (grepl("^kappa\\[", param_name)) return("kappa")
  stop(sprintf("Cannot determine prior type for parameter '%s'.", param_name),
       call. = FALSE)
}

#' Determine whether a parameter uses a half-prior
#' @keywords internal
#' @noRd
.is_half_prior <- function(param_name) {
  grepl("^(sigma|sd_u)\\[", param_name)
}


#' Compute the Savage-Dickey density ratio Bayes factor (internal)
#'
#' This is the internal workhorse called by \code{bf_table()}.
#' It returns a plain named list for a single test (one parameter or one joint
#' set of parameters).
#'
#' @param object     A \code{bvarnet} object.
#' @param params     Character vector of Stan parameter names.
#' @param null_value Numeric scalar or vector of null values (recycled if scalar).
#' @param method     One of \code{"auto"}, \code{"logspline"}, \code{"mvn"}.
#'
#' @return A named list with elements: \code{BF01}, \code{BF10}, \code{log_BF01},
#'   \code{post_density}, \code{prior_density}, \code{method}, \code{params},
#'   \code{null_value}.
#' @keywords internal
#' @noRd
savage_dickey <- function(object, params, null_value = 0,
                          method = c("auto", "logspline", "mvn")) {
  stopifnot(inherits(object, "bvarnet"))
  method    <- match.arg(method)

  # --- Validate params exist in draws ---
  all_par <- dimnames(object$draws)[[3]]
  missing <- setdiff(params, all_par)
  if (length(missing) > 0)
    stop(sprintf("Parameter(s) not found in draws: %s",
                 paste(missing, collapse = ", ")), call. = FALSE)

  # --- Recycle null_value ---
  d <- length(params)
  if (length(null_value) == 1L) null_value <- rep(null_value, d)
  stopifnot(length(null_value) == d)

  # --- Block half-prior params (sigma, sd_u) ---
  # The SDDR is not valid for parameters with half-priors: boundary at 0

  # violates the interior-null requirement at the natural null, and testing
  # at positive interior values has limited practical utility.
  half_flags <- vapply(params, .is_half_prior, logical(1), USE.NAMES = FALSE)
  if (any(half_flags)) {
    bad <- params[half_flags]
    stop(sprintf(
      paste0("Cannot compute SDDR for half-prior parameter(s): %s. ",
             "Parameters with half-priors (sigma, sd_u) are not supported ",
             "because the Savage-Dickey density ratio is not valid for ",
             "distributions bounded at zero.\n\n"),
      paste(bad, collapse = ", ")
    ), call. = FALSE)
  }

  # --- Check for low draw count ---
  draws_3d <- object$draws
  S_total  <- dim(draws_3d)[1] * dim(draws_3d)[2]
  if (S_total < 1000)
    warning("Very few posterior draws (S = ", S_total,
            "); BF estimates may be unreliable.", call. = FALSE)

  # --- Auto-select method ---
  if (method == "auto")
    method <- if (d == 1L) "logspline" else "mvn"

  # --- Extract draws ---
  param_types <- vapply(params, .param_type, character(1), USE.NAMES = FALSE)
  priors      <- object$priors   # bvarnet_priors object

  if (method == "logspline" && d == 1L) {

    draws_vec <- .extract_draws_raw(object, params)
    res <- .compute_sddr_logspline(
      draws_vec, prior = priors[[param_types]], null = null_value
    )

  } else if (method == "mvn" || (method == "logspline" && d > 1L)) {

    if (method == "logspline" && d > 1L) {
      warning("logspline is univariate; switching to MVN for joint test.",
              call. = FALSE)
      method <- "mvn"
    }
    draws_mat <- .extract_draws_raw(object, params)
    res <- .compute_sddr_mvn(
      draws_mat, priors, param_types, null_vec = null_value
    )

  } else {
    stop("Invalid method/dimension combination.", call. = FALSE)
  }

  list(
    BF01          = res$BF01,
    BF10          = 1 / res$BF01,
    log_BF01      = log(res$BF01),
    post_density  = res$post_density,
    prior_density = res$prior_density,
    method        = method,
    params        = params,
    null_value    = null_value
  )
}


#' Extract raw posterior draws for given Stan parameter names (internal)
#'
#' Returns a numeric vector (single param) or matrix (multiple params), flattening
#' the iter x chains dimensions.
#'
#' @param object A \code{bvarnet} object.
#' @param params Character vector of Stan parameter names.
#' @return Numeric vector or matrix.
#' @keywords internal
#' @noRd
.extract_draws_raw <- function(object, params) {
  draws_3d <- object$draws   # iter x chains x params
  idx <- match(params, dimnames(draws_3d)[[3]])
  chunk <- draws_3d[, , idx, drop = FALSE]
  # Flatten iter x chains
  dim(chunk) <- c(prod(dim(chunk)[1:2]), length(params))
  colnames(chunk) <- params
  if (length(params) == 1L) return(as.numeric(chunk))
  chunk
}


# ---- Parameter-block helpers for `parameter` mode ----------------------------

# ---- Variable-filtering helpers ----------------------------------------------

#' Validate variable names and classify into network/covariate (internal)
#'
#' @param sd       The \code{standata} list from a \code{bvarnet} object.
#' @param variable Character vector of variable names to look up.
#' @return A list with components \code{y_idx} (integer indices into Y columns,
#'   or \code{NULL}), \code{x_names} (character vector of matched covariate
#'   names, or \code{NULL}), and \code{has_y}/\code{has_x} logicals.
#' @keywords internal
#' @noRd
.classify_variable <- function(sd, variable) {
  y_names <- colnames(sd$Y)
  x_names <- colnames(sd$X)
  # Base covariates: exclude Intercept and interaction columns
  x_base <- x_names[!grepl(":", x_names) & x_names != "Intercept"]

  var_y <- intersect(variable, y_names)
  var_x <- intersect(variable, x_base)
  bad   <- setdiff(variable, c(y_names, x_base))
  if (length(bad) > 0L)
    stop(sprintf(
      "Unknown variable(s): %s. Available network variables: %s; covariates: %s.",
      paste(bad, collapse = ", "),
      paste(y_names, collapse = ", "),
      if (length(x_base)) paste(x_base, collapse = ", ") else "(none)"
    ), call. = FALSE)

  list(
    y_idx  = if (length(var_y)) match(var_y, y_names) else NULL,
    x_names = if (length(var_x)) var_x else NULL,
    has_y  = length(var_y) > 0L,
    has_x  = length(var_x) > 0L
  )
}

#' Filter phi indices to effects FROM specific variables (internal)
#'
#' Keeps only \code{phi[row, col]} entries where the lagged predictor
#' (\code{row_within}) is in \code{var_idx}.
#'
#' @param sd      The \code{standata} list.
#' @param lag     Integer; which lag block.
#' @param effect  One of \code{"ar"}, \code{"cl"}, \code{"all"}.
#' @param var_idx Integer vector of variable column positions.
#' @return Character vector of filtered Stan parameter names.
#' @keywords internal
#' @noRd
.filter_phi_by_variable <- function(sd, lag, effect, var_idx) {
  all_params <- get_phi_indices(sd, lag = lag, effect = effect)
  p <- sd$p
  keep <- vapply(all_params, function(pnm) {
    parts <- regmatches(pnm, regexec("^phi\\[(\\d+),(\\d+)\\]$", pnm))[[1]]
    row_i <- as.integer(parts[2])
    row_within <- ((row_i - 1L) %% p) + 1L
    row_within %in% var_idx
  }, logical(1L))
  all_params[keep]
}

#' Filter lag-interaction indices to effects FROM specific variables (internal)
#'
#' @param sd      The \code{standata} list.
#' @param var_idx Integer vector of variable column positions.
#' @return Same structure as \code{get_lag_interaction_indices_by_term()} but
#'   with only parameters where the lagged variable is in \code{var_idx}.
#' @keywords internal
#' @noRd
.filter_lag_interaction_by_variable <- function(sd, var_idx) {
  term_groups <- get_lag_interaction_indices_by_term(sd)
  p <- sd$p; K <- sd$K

  for (suffix in names(term_groups)) {
    tg <- term_groups[[suffix]]

    # Within each lag block of p rows, position j maps to lagged variable j.
    # Each position j generates p params (one per outcome col).
    # In by_lag[[k]] (length p*p), entries ((j-1)*p + 1):(j*p) → variable j.
    new_by_lag <- lapply(seq_len(K), function(k) {
      lg <- tg$by_lag[[k]]
      keep_idx <- integer(0)
      for (j in var_idx) {
        start <- (j - 1L) * p + 1L
        end   <- j * p
        if (end <= length(lg))
          keep_idx <- c(keep_idx, start:end)
      }
      lg[keep_idx]
    })
    names(new_by_lag) <- names(tg$by_lag)

    new_full <- unlist(new_by_lag, use.names = FALSE)
    # Reuse original AR/CL classification
    new_ar <- intersect(new_full, tg$ar)
    new_cl <- intersect(new_full, tg$cl)

    term_groups[[suffix]] <- list(
      full   = new_full,
      by_lag = new_by_lag,
      ar     = new_ar,
      cl     = new_cl
    )
  }

  # Remove terms that are now empty after filtering
  term_groups[vapply(term_groups, function(tg) length(tg$full) > 0L, logical(1L))]
}

#' Filter lag-interaction term groups to a specific set of covariates (internal)
#'
#' @param sd      The \code{standata} list.
#' @param x_names Character vector of covariate names to keep.
#' @return Filtered version of \code{get_lag_interaction_indices_by_term()},
#'   retaining only terms whose non-lag suffix matches \code{x_names}.
#' @keywords internal
#' @noRd
.filter_lag_interaction_by_covariate <- function(sd, x_names) {
  term_groups <- get_lag_interaction_indices_by_term(sd)
  # Term keys are the covariate suffix (e.g., "x_1" from c("lag","x_1"))
  term_groups[names(term_groups) %in% x_names]
}

#' Filter FE by_pred list to specific covariates (internal)
#'
#' @param by_pred Named list from \code{get_beta_indices_by_predictor()}.
#' @param x_names Character vector of covariate names to keep.
#' @return Filtered list retaining only predictors in \code{x_names}.
#' @keywords internal
#' @noRd
.filter_fe_by_covariate <- function(by_pred, x_names) {
  by_pred[names(by_pred) %in% x_names]
}


# ---- Primary export: bf_table() ----------------------------------------------#' Compute Bayes factor table for a bvarnet model
#'
#' Compute Savege-Dickey Bayes factors
#'
#' Computes Savage-Dickey density ratio Bayes factors for each (requested set of) parameter in the model.  
#' By default, all applicable parameters are tested and returned in a tidy data frame.  
#' The \code{type} argument controls which parameter groups are included; the \code{variable} argument can be used to filter to effects involving specific variables.
#' The \code{log_BF10} argument allows including the natural log of the Bayes factor in the output, and \code{round} controls numeric rounding of the results.
#'
#' @param object     A \code{bvarnet} object returned by \code{bvar()}.
#' @param type Character vector specifying which parameter groups to test.
#'   Use \code{"all"} (default) to include all applicable groups automatically.
#'   Available options:
#'   \describe{
#'     \item{\code{"ar"}}{Autoregressive effects (self-loops). Per-cell BFs
#'       for the lag specified by \code{lag}, plus a joint BF.}
#'     \item{\code{"cl"}}{Cross-lagged effects. Same structure as \code{"ar"}.}
#'     \item{\code{"intercepts"}}{Intercept parameters. Skipped automatically
#'       for ordinal outcomes.}
#'     \item{\code{"fe"}}{Non-intercept fixed effects (covariates).}
#'     \item{\code{"lag_fe"}}{Joint BFs for lag \eqn{\times} covariate
#'       interaction terms. Only available when the model was fitted with
#'       \code{fe_interactions} containing lag terms.}
#'     \item{\code{"temporal"}}{Joint BF for the entire temporal structure
#'       (all AR + CL parameters across all lags). When lag \eqn{\times}
#'       covariate interactions are present, additional omnibus rows are
#'       included.}
#'   }
#' @param lag        Integer; which lag block to use (default 1). Applies to
#'   \code{"ar"} and \code{"cl"} types.
#' @param null_value Numeric scalar; the null hypothesis value (default 0).
#' @param variable   Character vector or \code{NULL} (default).  One or more
#'   variable names.  When set, only effects involving these variables are
#'   included.
#' @param log_BF10 Logical; if \code{TRUE}, an additional \code{log_BF10}
#'   column (natural log of \code{BF10}) is appended to the output.
#'   Default is \code{FALSE}.
#' @param round Integer or \code{NULL}; number of decimal places to round
#'   numeric output columns.  Default is \code{5}.  Set to \code{NULL} to
#'   disable rounding.
#'
#' @return A data frame with columns: \code{type}, \code{predictor},
#'   \code{outcome}, \code{BF10} (and optionally \code{log_BF10}).
#'
#' @export
bf_table <- function(object,
                     type = "all",
                     lag = 1L,
                     null_value = 0,
                     variable = NULL,
                     log_BF10 = FALSE,
                     round = 5L) {
  stopifnot(inherits(object, "bvarnet"))

  # ---- Variable validation ----
  var_idx <- NULL
  var_x   <- NULL
  if (!is.null(variable)) {
    stopifnot(is.character(variable), length(variable) >= 1L)
    variable <- unique(variable)
    sd_tmp <- object$standata
    vclass <- .classify_variable(sd_tmp, variable)
    var_idx <- vclass$y_idx
    var_x   <- vclass$x_names

    # Cannot combine covariate-only variable with intercepts
    explicit_types <- if (!identical(type, "all")) type else character(0)
    if ("intercepts" %in% explicit_types)
      stop("'variable' cannot be combined with type = \"intercepts\".",
           call. = FALSE)
    # Cannot combine network-only variable with fe (no covariate to filter)
    if ("fe" %in% explicit_types && !vclass$has_x)
      stop("'variable' with type = \"fe\" requires at least one covariate name ",
           "(from x_cols). The variables you supplied are network variables; ",
           "use type = \"ar\", \"cl\", or \"temporal\" instead.",
           call. = FALSE)
    # Cannot combine covariate-only variable with ar/cl (phi is not filtered)
    if (any(c("ar", "cl") %in% explicit_types) && !vclass$has_y)
      stop("'variable' with type = \"ar\" or \"cl\" requires at least one ",
           "network variable (from colnames(standata$Y)). The variables you ",
           "supplied are covariates; use type = \"fe\", \"lag_fe\", or ",
           "\"temporal\" instead.",
           call. = FALSE)
  }

  # ---- Type mode (existing logic) ----
  sd <- object$standata
  nm <- get_param_names(sd)
  has_intercept <- "Intercept" %in% colnames(sd$X)
  # Minimum n_fe to have non-intercept FE: 2 with intercept, 1 without
  fe_min <- if (has_intercept) 2L else 1L

  # Resolve "all" to applicable types
  if (identical(type, "all")) {
    if (!is.null(var_idx) || !is.null(var_x)) {
      # Variable mode: start with types relevant to what was requested
      type <- character(0)
      if (!is.null(var_idx))
        type <- c("ar", "cl", "temporal")
      if (!is.null(var_x) && sd$n_fe >= fe_min)
        type <- c(type, "fe")
      if (!("temporal" %in% type) && !is.null(var_x))
        type <- c(type, "temporal")  # for interaction rows
    } else {
      type <- c("ar", "cl", "temporal")
      if (sd$n_fe >= fe_min)
        type <- c(type, "fe")
      if (has_intercept && any(object$family != "ordinal"))
        type <- c("intercepts", type)
    }
    lag_terms <- get_lag_interaction_indices_by_term(sd)
    if (length(lag_terms) > 0L) {
      # In variable mode, only add lag_fe if filtered terms would be non-empty
      if (!is.null(var_idx) || !is.null(var_x)) {
        filtered <- lag_terms
        if (!is.null(var_idx))
          filtered <- .filter_lag_interaction_by_variable(sd, var_idx)
        if (!is.null(var_x))
          filtered <- filtered[names(filtered) %in% var_x]
        if (length(filtered) > 0L)
          type <- c(type, "lag_fe")
      } else {
        type <- c(type, "lag_fe")
      }
    }
    type <- unique(type)
  }

  rows <- list()

  for (tp in type) {
    tp <- match.arg(tp, c("ar", "cl", "intercepts", "fe", "lag_fe", "temporal"))

    # ------------------------------------------------------------------
    # AR / CL — unchanged two-level structure
    # ------------------------------------------------------------------
    if (tp %in% c("ar", "cl")) {
      param_names <- if (!is.null(var_idx)) {
        .filter_phi_by_variable(sd, lag = lag, effect = tp, var_idx = var_idx)
      } else {
        switch(tp,
          ar = get_phi_indices(sd, lag = lag, effect = "ar"),
          cl = get_phi_indices(sd, lag = lag, effect = "cl")
        )
      }

      # Per-parameter BFs (logspline)
      for (pnm in param_names) {
        res <- savage_dickey(object, params = pnm, null_value = null_value,
                             method = "logspline")
        lab <- .param_label(pnm, nm)
        rows[[length(rows) + 1L]] <- data.frame(
          type          = .type_label(tp),
          predictor     = lab$predictor,
          outcome       = lab$outcome,
          BF01          = res$BF01,
          BF10          = res$BF10,
          log_BF01      = res$log_BF01,
          post_density  = res$post_density,
          prior_density = res$prior_density,
          method        = res$method,
          stringsAsFactors = FALSE
        )
      }

      # Joint BF (MVN)
      if (length(param_names) > 1L) {
        joint_res <- savage_dickey(object, params = param_names,
                                   null_value = null_value, method = "mvn")
        rows[[length(rows) + 1L]] <- data.frame(
          type          = paste0(.type_label(tp), " (joint)"),
          predictor     = paste0("all_", tp),
          outcome       = "\u2014",
          BF01          = joint_res$BF01,
          BF10          = joint_res$BF10,
          log_BF01      = joint_res$log_BF01,
          post_density  = joint_res$post_density,
          prior_density = joint_res$prior_density,
          method        = joint_res$method,
          stringsAsFactors = FALSE
        )
      }

    # ------------------------------------------------------------------
    # FE / INTERCEPTS — three-phase structure
    # ------------------------------------------------------------------
    } else if (tp %in% c("fe", "intercepts")) {
      param_names <- get_beta_indices(sd, type = tp)
      by_pred     <- get_beta_indices_by_predictor(sd, type = tp)
      type_base   <- .type_label(tp)

      # Filter to requested covariates when variable has x_names
      if (tp == "fe" && !is.null(var_x)) {
        by_pred <- .filter_fe_by_covariate(by_pred, var_x)
        param_names <- unlist(by_pred, use.names = FALSE)
      }

      # Filter out ordinal beta[1,j] sentinels for intercepts in mixed-family
      if (tp == "intercepts") {
        ord_indices <- which(object$family == "ordinal")
        if (length(ord_indices) > 0L) {
          sentinel_cols <- paste0("beta[1,", ord_indices, "]")
          param_names <- param_names[!param_names %in% sentinel_cols]
          by_pred <- lapply(by_pred, function(pvec) {
            pvec[!pvec %in% sentinel_cols]
          })
          by_pred <- by_pred[lengths(by_pred) > 0L]
        }
      }

      # Phase A — Per-cell BFs (logspline)
      for (pnm in param_names) {
        res <- savage_dickey(object, params = pnm, null_value = null_value,
                             method = "logspline")
        lab <- .param_label(pnm, nm)
        rows[[length(rows) + 1L]] <- data.frame(
          type          = type_base,
          predictor     = lab$predictor,
          outcome       = lab$outcome,
          BF01          = res$BF01,
          BF10          = res$BF10,
          log_BF01      = res$log_BF01,
          post_density  = res$post_density,
          prior_density = res$prior_density,
          method        = res$method,
          stringsAsFactors = FALSE
        )
      }

      # Phase B — Per-predictor joint BFs (MVN), only when p > 1
      for (pred_name in names(by_pred)) {
        pvec <- by_pred[[pred_name]]
        if (length(pvec) > 1L) {
          jres <- savage_dickey(object, params = pvec,
                                null_value = null_value, method = "mvn")
          rows[[length(rows) + 1L]] <- data.frame(
            type          = paste0(type_base, " (joint)"),
            predictor     = pred_name,
            outcome       = "\u2014",
            BF01          = jres$BF01,
            BF10          = jres$BF10,
            log_BF01      = jres$log_BF01,
            post_density  = jres$post_density,
            prior_density = jres$prior_density,
            method        = jres$method,
            stringsAsFactors = FALSE
          )
        }
      }

      # Phase C — Global joint-all BF (MVN), only when > 1 predictor row
      if (length(by_pred) > 1L) {
        all_params <- unlist(by_pred, use.names = FALSE)
        gres <- savage_dickey(object, params = all_params,
                              null_value = null_value, method = "mvn")
        rows[[length(rows) + 1L]] <- data.frame(
          type          = paste0(type_base, " (joint all)"),
          predictor     = paste0("all_", tp),
          outcome       = "\u2014",
          BF01          = gres$BF01,
          BF10          = gres$BF10,
          log_BF01      = gres$log_BF01,
          post_density  = gres$post_density,
          prior_density = gres$prior_density,
          method        = gres$method,
          stringsAsFactors = FALSE
        )
      }

    # ------------------------------------------------------------------
    # LAG_FE — grouped joint rows only
    # ------------------------------------------------------------------
    } else if (tp == "lag_fe") {
      term_groups <- get_lag_interaction_indices_by_term(sd)
      # Filter by network variable (source variable within interactions)
      if (!is.null(var_idx))
        term_groups <- .filter_lag_interaction_by_variable(sd, var_idx)
      # Filter by covariate (which interaction terms to include)
      if (!is.null(var_x))
        term_groups <- term_groups[names(term_groups) %in% var_x]

      if (length(term_groups) == 0L)
        stop(
          "No lag interaction columns found in standata$X. ",
          "Was the model fitted with fe_interactions containing 'lag' terms?",
          call. = FALSE
        )

      for (suffix in names(term_groups)) {
        tg <- term_groups[[suffix]]

        # Per-lag-block joint BFs
        for (lag_name in names(tg$by_lag)) {
          lg_params <- tg$by_lag[[lag_name]]
          if (length(lg_params) > 0L) {
            lres <- savage_dickey(object, params = lg_params,
                                  null_value = null_value,
                                  method = if (length(lg_params) == 1L) "logspline" else "mvn")
            rows[[length(rows) + 1L]] <- data.frame(
              type          = "Lag Interaction (per lag)",
              predictor     = lag_name,
              outcome       = "\u2014",
              BF01          = lres$BF01,
              BF10          = lres$BF10,
              log_BF01      = lres$log_BF01,
              post_density  = lres$post_density,
              prior_density = lres$prior_density,
              method        = lres$method,
              stringsAsFactors = FALSE
            )
          }
        }

        # Full-term omnibus joint BF (skip at K=1 — identical to per-lag row)
        if (length(tg$full) > 1L && sd$K > 1L) {
          ores <- savage_dickey(object, params = tg$full,
                                null_value = null_value, method = "mvn")
          rows[[length(rows) + 1L]] <- data.frame(
            type          = "Lag Interaction (joint)",
            predictor     = suffix,
            outcome       = "\u2014",
            BF01          = ores$BF01,
            BF10          = ores$BF10,
            log_BF01      = ores$log_BF01,
            post_density  = ores$post_density,
            prior_density = ores$prior_density,
            method        = ores$method,
            stringsAsFactors = FALSE
          )
        }
      }

    # ------------------------------------------------------------------
    # TEMPORAL — joint BF over entire phi (AR + CL, all lags)
    #   plus separate AR-only and CL-only joint BFs
    #   plus lag × covariate interactions if present
    # ------------------------------------------------------------------
    } else if (tp == "temporal") {
      K <- sd$K
      # When only covariates (no network vars), skip phi-only temporal rows
      covariate_only <- !is.null(var_x) && is.null(var_idx)

      all_phi <- character(0)
      all_ar  <- character(0)
      all_cl  <- character(0)
      if (!covariate_only) {
        for (k in seq_len(K)) {
          if (!is.null(var_idx)) {
            all_phi <- c(all_phi, .filter_phi_by_variable(sd, lag = k, effect = "all", var_idx = var_idx))
            all_ar  <- c(all_ar,  .filter_phi_by_variable(sd, lag = k, effect = "ar",  var_idx = var_idx))
            all_cl  <- c(all_cl,  .filter_phi_by_variable(sd, lag = k, effect = "cl",  var_idx = var_idx))
          } else {
            all_phi <- c(all_phi, get_phi_indices(sd, lag = k, effect = "all"))
            all_ar  <- c(all_ar,  get_phi_indices(sd, lag = k, effect = "ar"))
            all_cl  <- c(all_cl,  get_phi_indices(sd, lag = k, effect = "cl"))
          }
        }
      }

      # Base temporal BF (phi only — AR + CL combined)
      if (length(all_phi) > 0L) {
        if (length(all_phi) == 1L) {
          tres <- savage_dickey(object, params = all_phi,
                                null_value = null_value, method = "logspline")
        } else {
          tres <- savage_dickey(object, params = all_phi,
                                null_value = null_value, method = "mvn")
        }
        rows[[length(rows) + 1L]] <- data.frame(
          type          = "Temporal (joint)",
          predictor     = "all_phi",
          outcome       = "\u2014",
          BF01          = tres$BF01,
          BF10          = tres$BF10,
          log_BF01      = tres$log_BF01,
          post_density  = tres$post_density,
          prior_density = tres$prior_density,
          method        = tres$method,
          stringsAsFactors = FALSE
        )
      }

      # AR-only joint BF (skip at K=1 to avoid duplicating ar/cl type rows)
      if (length(all_ar) > 0L && K > 1L) {
        if (length(all_ar) == 1L) {
          ar_res <- savage_dickey(object, params = all_ar,
                                  null_value = null_value, method = "logspline")
        } else {
          ar_res <- savage_dickey(object, params = all_ar,
                                  null_value = null_value, method = "mvn")
        }
        rows[[length(rows) + 1L]] <- data.frame(
          type          = "Temporal AR (joint)",
          predictor     = "all_ar",
          outcome       = "\u2014",
          BF01          = ar_res$BF01,
          BF10          = ar_res$BF10,
          log_BF01      = ar_res$log_BF01,
          post_density  = ar_res$post_density,
          prior_density = ar_res$prior_density,
          method        = ar_res$method,
          stringsAsFactors = FALSE
        )
      }

      # CL-only joint BF (skip at K=1 or p=1)
      if (length(all_cl) > 0L && K > 1L) {
        if (length(all_cl) == 1L) {
          cl_res <- savage_dickey(object, params = all_cl,
                                  null_value = null_value, method = "logspline")
        } else {
          cl_res <- savage_dickey(object, params = all_cl,
                                  null_value = null_value, method = "mvn")
        }
        rows[[length(rows) + 1L]] <- data.frame(
          type          = "Temporal CL (joint)",
          predictor     = "all_cl",
          outcome       = "\u2014",
          BF01          = cl_res$BF01,
          BF10          = cl_res$BF10,
          log_BF01      = cl_res$log_BF01,
          post_density  = cl_res$post_density,
          prior_density = cl_res$prior_density,
          method        = cl_res$method,
          stringsAsFactors = FALSE
        )
      }

      # Lag × covariate interaction terms (if any)
      term_groups <- get_lag_interaction_indices_by_term(sd)
      if (!is.null(var_idx))
        term_groups <- .filter_lag_interaction_by_variable(sd, var_idx)
      if (!is.null(var_x))
        term_groups <- term_groups[names(term_groups) %in% var_x]
      if (length(term_groups) > 0L) {
        lag_int_params <- unlist(
          lapply(term_groups, `[[`, "full"), use.names = FALSE
        )

        # Per-term sub-tests (AR×interaction, CL×interaction)
        for (suffix in names(term_groups)) {
          tg_params <- term_groups[[suffix]]$full
          if (length(tg_params) > 0L) {
            # AR × interaction: beta params where lagged outcome == target
            ar_int <- term_groups[[suffix]]$ar
            if (length(ar_int) > 0L) {
              arx_res <- savage_dickey(
                object, params = ar_int,
                null_value = null_value,
                method = if (length(ar_int) == 1L) "logspline" else "mvn"
              )
              rows[[length(rows) + 1L]] <- data.frame(
                type          = "Temporal AR \u00d7 Interaction (joint)",
                predictor     = suffix,
                outcome       = "\u2014",
                BF01          = arx_res$BF01,
                BF10          = arx_res$BF10,
                log_BF01      = arx_res$log_BF01,
                post_density  = arx_res$post_density,
                prior_density = arx_res$prior_density,
                method        = arx_res$method,
                stringsAsFactors = FALSE
              )
            }
            # CL × interaction: beta params where lagged outcome != target
            cl_int <- term_groups[[suffix]]$cl
            if (length(cl_int) > 0L) {
              clx_res <- savage_dickey(
                object, params = cl_int,
                null_value = null_value,
                method = if (length(cl_int) == 1L) "logspline" else "mvn"
              )
              rows[[length(rows) + 1L]] <- data.frame(
                type          = "Temporal CL \u00d7 Interaction (joint)",
                predictor     = suffix,
                outcome       = "\u2014",
                BF01          = clx_res$BF01,
                BF10          = clx_res$BF10,
                log_BF01      = clx_res$log_BF01,
                post_density  = clx_res$post_density,
                prior_density = clx_res$prior_density,
                method        = clx_res$method,
                stringsAsFactors = FALSE
              )
            }
          }
        }

        # Full temporal + interactions omnibus (only when both phi and
        # interaction params exist; otherwise it duplicates another row)
        all_temporal <- c(all_phi, lag_int_params)
        if (length(all_phi) > 0L && length(lag_int_params) > 0L) {
          ares <- savage_dickey(object, params = all_temporal,
                                null_value = null_value, method = "mvn")
          rows[[length(rows) + 1L]] <- data.frame(
            type          = "Temporal + Interactions (joint)",
            predictor     = "all_temporal",
            outcome       = "\u2014",
            BF01          = ares$BF01,
            BF10          = ares$BF10,
            log_BF01      = ares$log_BF01,
            post_density  = ares$post_density,
            prior_density = ares$prior_density,
            method        = ares$method,
            stringsAsFactors = FALSE
          )
        }
      }
    }  }

  if (length(rows) == 0L) {
    out <- data.frame(
      type = character(0), predictor = character(0), outcome = character(0),
      BF10 = numeric(0), stringsAsFactors = FALSE
    )
    if (isTRUE(log_BF10)) out$log_BF10 <- numeric(0)
    return(out)
  }
  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  out <- out[, c("type", "predictor", "outcome", "BF10")]
  if (isTRUE(log_BF10)) out$log_BF10 <- log(out$BF10)
  if (!is.null(round)) {
    num_cols <- sapply(out, is.numeric)
    out[num_cols] <- lapply(out[num_cols], round, digits = round)
  }
  out
}


# ---- Label helpers -----------------------------------------------------------

#' Produce human-readable labels for a Stan parameter name (internal)
#' @keywords internal
#' @noRd
.param_label <- function(stan_name, nm) {
  # Parse "phi[r,c]" or "beta[r,c]"
  parts <- regmatches(stan_name, regexec("^(\\w+)\\[(\\d+),(\\d+)\\]$", stan_name))[[1]]
  if (length(parts) == 4L) {
    param <- parts[2]
    row_i <- as.integer(parts[3])
    col_i <- as.integer(parts[4])

    if (param == "phi") {
      predictor <- if (row_i <= length(nm$b)) nm$b[row_i] else stan_name
      outcome   <- if (col_i <= length(nm$y)) nm$y[col_i] else stan_name
    } else if (param == "beta") {
      predictor <- if (row_i <= length(nm$fe)) nm$fe[row_i] else stan_name
      outcome   <- if (col_i <= length(nm$y))  nm$y[col_i]  else stan_name
    } else {
      predictor <- stan_name
      outcome   <- stan_name
    }
  } else {
    predictor <- stan_name
    outcome   <- "\u2014"
  }

  list(predictor = predictor, outcome = outcome)
}

#' Human-readable type label (internal)
#' @keywords internal
#' @noRd
.type_label <- function(type) {
  switch(type,
    ar         = "Autoregressive",
    cl         = "Cross-lagged",
    intercepts = "Intercept",
    fe         = "Fixed Effect",
    type
  )
}

# ──────────────────────────────────────────────────────────────────────────────
# test-bayes_factor.R — Tests for savage_dickey(), bf_table(), and helpers
# ──────────────────────────────────────────────────────────────────────────────

# ── Layer 1: Analytical ground truth (no MCMC) ───────────────────────────────

test_that("eval_prior_density: normal prior", {
  pr <- prior("normal", loc = 0, scale = 1)
  expect_equal(eval_prior_density(pr, 0), dnorm(0, 0, 1))
  expect_equal(eval_prior_density(pr, 1.5), dnorm(1.5, 0, 1))
})

test_that("eval_prior_density: student_t prior", {
  pr <- prior("student_t", loc = 0, scale = 2, df = 5)
  expected <- dt(0 / 2, df = 5) / 2
  expect_equal(eval_prior_density(pr, 0), expected)
  expected2 <- dt(1.3 / 2, df = 5) / 2
  expect_equal(eval_prior_density(pr, 1.3), expected2)
})

test_that("eval_prior_density: cauchy prior", {
  pr <- prior("cauchy", loc = 0, scale = 1)
  expect_equal(eval_prior_density(pr, 0), dcauchy(0, 0, 1))
})

test_that("eval_joint_prior_density: product of independent marginals", {
  priors <- set_priors()
  types <- c("phi", "phi", "beta")
  null  <- c(0, 0, 0)
  expected <- eval_prior_density(priors$phi, 0) *
              eval_prior_density(priors$phi, 0) *
              eval_prior_density(priors$beta, 0)
  expect_equal(
    eval_joint_prior_density(priors, types, null),
    expected
  )
})

test_that("logspline SDDR matches analytical normal-normal BF", {
  set.seed(1)
  S <- 50000

  # Conjugate normal-normal: prior N(0,1), likelihood N(theta, 1), n=30, ybar=0.15
  # (moderate effect so BF is not too extreme for stable estimation)
  n_obs <- 30; ybar <- 0.15; sigma <- 1; prior_sd <- 1
  post_var  <- 1 / (n_obs / sigma^2 + 1 / prior_sd^2)
  post_mean <- post_var * (n_obs * ybar / sigma^2)
  draws <- rnorm(S, post_mean, sqrt(post_var))

  BF_true <- dnorm(0, post_mean, sqrt(post_var)) / dnorm(0, 0, 1)

  pr <- prior("normal", loc = 0, scale = 1)
  res <- .compute_sddr_logspline(draws, prior = pr, null = 0)
  expect_equal(res$BF01, BF_true, tolerance = 0.10)
})

test_that("MVN SDDR matches analytical bivariate normal-normal BF", {
  set.seed(2)
  S <- 30000

  # Two independent conjugate normals
  prior_sd <- c(1, 0.5)
  n_obs <- 100; ybar <- c(0.3, -0.2); sigma <- c(1, 1)

  post_var  <- 1 / (n_obs / sigma^2 + 1 / prior_sd^2)
  post_mean <- post_var * (n_obs * ybar / sigma^2)

  draws_mat <- cbind(
    rnorm(S, post_mean[1], sqrt(post_var[1])),
    rnorm(S, post_mean[2], sqrt(post_var[2]))
  )
  colnames(draws_mat) <- c("phi[1,1]", "phi[2,1]")

  # Analytical joint posterior density at (0, 0) — independent, so product
  post_den_true <- dnorm(0, post_mean[1], sqrt(post_var[1])) *
                   dnorm(0, post_mean[2], sqrt(post_var[2]))
  prior_den_true <- dnorm(0, 0, prior_sd[1]) * dnorm(0, 0, prior_sd[2])
  BF_true <- post_den_true / prior_den_true

  # Use two phi priors with different scales — need to pass them via prior_list
  priors <- list(phi = prior("normal", 0, 1))
  # Since both use "phi" type, the function uses the same prior for both — but
  # our analytical uses different scales.  So let's use same scale for both.
  draws_mat2 <- cbind(
    rnorm(S, post_mean[1], sqrt(post_var[1])),
    rnorm(S, post_mean[1], sqrt(post_var[1]))  # same marginal
  )
  colnames(draws_mat2) <- c("phi[1,1]", "phi[2,1]")

  # Equal-prior version
  post_var_eq  <- 1 / (n_obs / 1 + 1 / 1)
  post_mean_eq <- post_var_eq * (n_obs * 0.3)
  post_den_eq  <- dnorm(0, post_mean_eq, sqrt(post_var_eq))^2
  prior_den_eq <- dnorm(0, 0, 1)^2
  BF_eq <- post_den_eq / prior_den_eq

  res <- .compute_sddr_mvn(
    draws_mat2,
    prior_list  = list(phi = prior("normal", 0, 1)),
    param_types = c("phi", "phi"),
    null_vec    = c(0, 0)
  )
  expect_equal(res$BF01, BF_eq, tolerance = 0.10)
})


# ── Layer 2: Consistency across methods ──────────────────────────────────────

test_that("logspline and MVN give consistent single-parameter BFs", {
  set.seed(3)
  S <- 50000
  n_obs <- 30; ybar <- 0.15; prior_sd <- 1
  post_var  <- 1 / (n_obs + 1)
  post_mean <- post_var * (n_obs * ybar)
  draws <- rnorm(S, post_mean, sqrt(post_var))

  pr <- prior("normal", 0, 1)

  res_ls <- .compute_sddr_logspline(draws, prior = pr, null = 0)

  draws_mat <- matrix(draws, ncol = 1)
  colnames(draws_mat) <- "phi[1,1]"
  res_mvn <- .compute_sddr_mvn(draws_mat,
                                prior_list = list(phi = pr),
                                param_types = "phi",
                                null_vec = 0)

  expect_equal(res_ls$BF01, res_mvn$BF01, tolerance = 0.15)
})


# ── get_phi_indices tests ────────────────────────────────────────────────────

test_that("get_phi_indices returns correct AR indices for p=3, K=1", {
  sd <- list(p = 3L, K = 1L)
  ar <- get_phi_indices(sd, lag = 1, effect = "ar")
  expect_equal(ar, c("phi[1,1]", "phi[2,2]", "phi[3,3]"))
})

test_that("get_phi_indices returns correct CL indices for p=3, K=1", {
  sd <- list(p = 3L, K = 1L)
  cl <- get_phi_indices(sd, lag = 1, effect = "cl")
  expect_length(cl, 6)  # p*(p-1) = 6
  expect_false("phi[1,1]" %in% cl)
  expect_false("phi[2,2]" %in% cl)
  expect_false("phi[3,3]" %in% cl)
  expect_true("phi[2,1]" %in% cl)
  expect_true("phi[1,2]" %in% cl)
})

test_that("get_phi_indices returns all for p=2, K=1", {
  sd <- list(p = 2L, K = 1L)
  all <- get_phi_indices(sd, lag = 1, effect = "all")
  expect_length(all, 4)  # p*p = 4
})

test_that("get_phi_indices lag=2 uses correct offset", {
  sd <- list(p = 2L, K = 2L)
  ar2 <- get_phi_indices(sd, lag = 2, effect = "ar")
  expect_equal(ar2, c("phi[3,1]", "phi[4,2]"))
})

test_that("get_phi_indices errors on invalid lag", {
  sd <- list(p = 2L, K = 1L)
  expect_error(get_phi_indices(sd, lag = 2, effect = "ar"))
})


# ── get_beta_indices tests ───────────────────────────────────────────────────

test_that("get_beta_indices returns intercepts correctly", {
  X <- matrix(0, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  sd <- list(p = 3L, n_fe = 2L, X = X)
  ic <- get_beta_indices(sd, type = "intercepts")
  expect_equal(ic, c("beta[1,1]", "beta[1,2]", "beta[1,3]"))
})

test_that("get_beta_indices returns FE (non-intercept) correctly", {
  # With Intercept in X: FE starts at row 2
  X <- matrix(0, 10, 3, dimnames = list(NULL, c("Intercept", "x_1", "x_2")))
  sd <- list(p = 2L, n_fe = 3L, X = X)
  fe <- get_beta_indices(sd, type = "fe")
  expect_length(fe, 4)  # (3-1) * 2 = 4
  expect_true("beta[2,1]" %in% fe)
  expect_true("beta[3,2]" %in% fe)
  expect_false("beta[1,1]" %in% fe)

  # Without Intercept (pure ordinal): FE starts at row 1
  X_ord <- matrix(0, 10, 2, dimnames = list(NULL, c("x_1", "x_2")))
  sd_ord <- list(p = 2L, n_fe = 2L, X = X_ord)
  fe_ord <- get_beta_indices(sd_ord, type = "fe")
  expect_length(fe_ord, 4)  # 2 * 2 = 4
  expect_true("beta[1,1]" %in% fe_ord)
  expect_true("beta[2,2]" %in% fe_ord)
})


# ── savage_dickey() with mock bvarnet object ─────────────────────────────────

test_that("savage_dickey errors on missing param names", {
  mock <- make_mock_bvarnet("gaussian")
  expect_error(
    savage_dickey(mock, params = "phi[99,99]"),
    "not found in draws"
  )
})

test_that("savage_dickey errors on any half-prior param", {
  mock <- make_mock_bvarnet("gaussian")
  expect_error(
    savage_dickey(mock, params = "sigma[1]", null_value = 0),
    "Cannot compute SDDR"
  )
  # Also blocked at positive interior null values
  expect_error(
    savage_dickey(mock, params = "sigma[1]", null_value = 0.5),
    "Cannot compute SDDR"
  )
})

test_that("savage_dickey returns correct structure for univariate", {
  mock <- make_mock_bvarnet("gaussian")
  res <- savage_dickey(mock, params = "phi[1,1]", null_value = 0)
  expect_true(is.list(res))
  expect_named(res, c("BF01", "BF10", "log_BF01", "post_density",
                       "prior_density", "method", "params", "null_value"))
  expect_equal(res$method, "logspline")
  expect_equal(res$BF01 * res$BF10, 1)
  expect_equal(res$log_BF01, log(res$BF01))
})

test_that("savage_dickey returns correct structure for joint MVN", {
  mock <- make_mock_bvarnet("gaussian")
  res <- savage_dickey(mock, params = c("phi[1,1]", "phi[2,1]"),
                       null_value = 0, method = "mvn")
  expect_true(is.list(res))
  expect_equal(res$method, "mvn")
  expect_true(is.finite(res$BF01))
  expect_equal(res$BF01 * res$BF10, 1)
})



# ── bf_table() output shape tests ────────────────────────────────────────────

test_that("bf_table returns correct data frame shape for ar + cl", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = c("ar", "cl"))

  expect_s3_class(res, "data.frame")
  expect_true(all(c("type", "predictor", "outcome", "BF10")
                  %in% names(res)))

  # p=2, K=1 → ar: 2 + 1 joint + cl: 2 + 1 joint = 6
  expect_equal(nrow(res), 6)
  expect_true(all(res$BF10 > 0))
  expect_true(any(grepl("joint", res$type)))
})

test_that("bf_table returns correct rows for ar type (p=2, K=1)", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = "ar")

  # p=2: 2 AR params + 1 joint row = 3
  expect_equal(nrow(res), 3)
  expect_true(any(grepl("joint", res$type)))
})

test_that("bf_table returns correct rows for cl type (p=2, K=1)", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = "cl")

  # p=2: 2 CL params + 1 joint row = 3
  expect_equal(nrow(res), 3)
})

test_that("bf_table returns correct rows for intercepts", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = "intercepts")

  # p=2: 2 intercept params + 1 joint row = 3
  expect_equal(nrow(res), 3)
})

test_that("bf_table handles multiple types", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = c("ar", "cl"))

  # ar: 2 + 1 + cl: 2 + 1 = 6
  expect_equal(nrow(res), 6)
})

test_that("bf_table joint row BF01 is positive and finite", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = c("ar", "cl"))
  joint_rows <- res[grepl("joint", res$type), ]
  expect_equal(nrow(joint_rows), 2)  # one for AR, one for CL
  expect_true(all(is.finite(joint_rows$BF10)))
  expect_true(all(joint_rows$BF10 > 0))
})


# ── extract_param integration ────────────────────────────────────────────────

test_that("extract_param with bayes_factor = TRUE adds BF columns", {
  mock <- make_mock_bvarnet("gaussian")
  res <- extract_param(mock, bayes_factor = TRUE)
  expect_true("BF01" %in% names(res))
  expect_true("BF10" %in% names(res))

  # BFs should be computed for Intercept, Fixed Effect, Autoregressive, Cross-lagged rows
  bf_rows <- res[res$type %in% c("Intercept", "Fixed Effect", "Autoregressive", "Cross-lagged"), ]
  expect_true(all(!is.na(bf_rows$BF01)))
  expect_true(all(bf_rows$BF01 > 0))
})

test_that("extract_param with bayes_factor = FALSE has no BF columns", {
  mock <- make_mock_bvarnet("gaussian")
  res <- extract_param(mock, bayes_factor = FALSE)
  expect_false("BF01" %in% names(res))
})


# ── Edge cases ───────────────────────────────────────────────────────────────

test_that("savage_dickey warns on very few draws", {
  mock <- make_mock_bvarnet("gaussian", n_iter = 50L, n_chains = 2L)
  # 50 draws * 2 chains = 100 < 1000
  expect_warning(
    savage_dickey(mock, params = "phi[1,1]", null_value = 0),
    "Very few posterior draws"
  )
})

test_that("p = 1 case: AR and phi produce same params", {
  # Create a p=1 mock
  sd <- list(p = 1L, K = 1L)
  ar <- get_phi_indices(sd, lag = 1, effect = "ar")
  all <- get_phi_indices(sd, lag = 1, effect = "all")
  cl <- get_phi_indices(sd, lag = 1, effect = "cl")
  expect_equal(ar, all)
  expect_length(cl, 0)
})


# ── get_beta_indices_by_predictor tests ──────────────────────────────────────

test_that("get_beta_indices_by_predictor groups by predictor row", {
  X <- matrix(0, 10, 4,
              dimnames = list(NULL, c("Intercept", "x_1", "x_2", "x_1:x_2")))
  sd <- list(p = 3L, n_fe = 4L, X = X)
  by_pred <- get_beta_indices_by_predictor(sd, type = "fe")

  # 3 predictor rows (rows 2, 3, 4); each should have p = 3 beta names
  expect_length(by_pred, 3)
  expect_named(by_pred, c("x_1", "x_2", "x_1:x_2"))
  expect_equal(by_pred[["x_1"]], c("beta[2,1]", "beta[2,2]", "beta[2,3]"))
  expect_equal(by_pred[["x_2"]], c("beta[3,1]", "beta[3,2]", "beta[3,3]"))
})

test_that("get_beta_indices_by_predictor handles intercepts", {
  X <- matrix(0, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  sd <- list(p = 2L, n_fe = 2L, X = X)
  by_pred <- get_beta_indices_by_predictor(sd, type = "intercepts")

  expect_length(by_pred, 1)
  expect_named(by_pred, "Intercept")
  expect_equal(by_pred[["Intercept"]], c("beta[1,1]", "beta[1,2]"))
})

test_that("get_beta_indices_by_predictor errors for ordinal intercepts", {
  X <- matrix(0, 10, 2, dimnames = list(NULL, c("x_1", "x_2")))
  sd <- list(p = 2L, n_fe = 2L, X = X, C = 5L)
  expect_error(
    get_beta_indices_by_predictor(sd, type = "intercepts"),
    "not valid for ordinal"
  )
})

test_that("get_beta_indices errors for ordinal intercepts", {
  X <- matrix(0, 10, 2, dimnames = list(NULL, c("x_1", "x_2")))
  sd <- list(p = 2L, n_fe = 2L, X = X, C = 5L)
  expect_error(
    get_beta_indices(sd, type = "intercepts"),
    "not valid for ordinal"
  )
})


# ── bf_table three-level FE tests ────────────────────────────────────────────

test_that("bf_table fe: p=2, 2 predictors → 4 cell + 2 joint + 1 joint all = 7", {
  # Build mock with n_fe = 3 (Intercept + x_1 + x_2), p = 2
  p <- 2L; K <- 1L; n_fe <- 3L; n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(99L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, c("Intercept", "x_1", "x_2")))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "fe")
  expect_equal(nrow(res), 7)

  # Per-cell rows
  cell_rows <- res[res$type == "Fixed Effect", ]
  expect_equal(nrow(cell_rows), 4)  # 2 predictors * 2 outcomes

  # Per-predictor joint rows
  joint_rows <- res[res$type == "Fixed Effect (joint)", ]
  expect_equal(nrow(joint_rows), 2)
  expect_true(all(joint_rows$method == "mvn"))

  # Global joint-all row
  all_rows <- res[res$type == "Fixed Effect (joint all)", ]
  expect_equal(nrow(all_rows), 1)
  expect_equal(all_rows$predictor, "all_fe")
})

test_that("bf_table fe: p=1, 2 predictors → 2 cell + 0 joint + 1 joint all = 3", {
  # p=1 means Phase B is skipped (per-cell is already the joint)
  p <- 1L; K <- 1L; n_fe <- 3L; n_re <- 0L
  par_nms <- c("beta[1,1]", "beta[2,1]", "beta[3,1]",
               "phi[1,1]", "sigma[1]")

  set.seed(98L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, "y_1"))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, c("Intercept", "x_1", "x_2")))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, "lag1_y_1"))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "fe")
  # 2 per-cell + 0 per-predictor joint (p=1) + 1 joint-all (2 predictors) = 3
  expect_equal(nrow(res), 3)
  expect_equal(sum(res$type == "Fixed Effect (joint)"), 0)
  expect_equal(sum(res$type == "Fixed Effect (joint all)"), 1)
})

test_that("bf_table intercepts: p=2, 1 predictor row → 2 + 1 + 0 = 3", {
  # Phase C skipped because only 1 predictor row
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = "intercepts")
  expect_equal(nrow(res), 3)
  expect_equal(sum(res$type == "Intercept (joint)"), 1)
  expect_equal(sum(grepl("joint all", res$type)), 0)
})


# ── lag_fe type tests ────────────────────────────────────────────────────────

test_that("bf_table lag_fe: p=2, K=1, 1 interaction → 2 rows", {
  # Build a mock with lag×x_1 interaction
  p <- 2L; K <- 1L
  # X: Intercept, x_1, lag1_y_1:x_1, lag1_y_2:x_1
  n_fe <- 4L; n_re <- 0L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(97L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "lag_fe")
  # K=1: 1 per-lag joint only (omnibus suppressed since identical)
  expect_equal(nrow(res), 1)
  expect_true("Lag Interaction (per lag)" %in% res$type)
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table lag_fe: p=2, K=2 → 3 rows (2 per-lag + 1 omnibus)", {
  p <- 2L; K <- 2L
  fe_names <- c("Intercept", "x_1",
                "lag1_y_1:x_1", "lag1_y_2:x_1",
                "lag2_y_1:x_1", "lag2_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- sprintf("phi[%d,%d]",
                      rep(1:(p * K), times = p),
                      rep(1:p, each = p * K))
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(96L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL,
      c("lag1_y_1", "lag1_y_2", "lag2_y_1", "lag2_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1",
                                  "lag2_y_1:x_1", "lag2_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "lag_fe")
  expect_equal(nrow(res), 3)
  per_lag <- res[res$type == "Lag Interaction (per lag)", ]
  expect_equal(nrow(per_lag), 2)
  omnibus <- res[res$type == "Lag Interaction (joint)", ]
  expect_equal(nrow(omnibus), 1)
})

test_that("bf_table lag_fe errors when no lag interactions present", {
  mock <- make_mock_bvarnet("gaussian")
  expect_error(
    bf_table(mock, type = "lag_fe"),
    "No lag interaction columns"
  )
})




# ── to_stan_data metadata persistence ────────────────────────────────────────

test_that("to_stan_data stores fe_interaction_terms and colnames", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, q = 1, family = "bernoulli")
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), "x_1",
                     fe_interactions = list(c("lag", "x_1")), K = 1)

  expect_true(!is.null(sd$fe_interaction_terms))
  expect_true(is.list(sd$fe_interaction_terms))
  expect_length(sd$fe_interaction_terms, 1)
  expect_equal(sd$fe_interaction_terms[[1]], c("lag", "x_1"))

  expect_true(!is.null(sd$fe_interaction_colnames))
  expect_true(all(grepl(":x_1$", sd$fe_interaction_colnames)))
})

test_that("to_stan_data sets empty metadata when no fe_interactions", {
  df <- make_test_df(N = 3, T_obs = 20, p = 2, q = 1, family = "bernoulli")
  sd <- to_stan_data(df, "bernoulli", "id", "t",
                     paste0("y_", 1:2), "x_1", K = 1)
  expect_equal(sd$fe_interaction_terms, list())
  expect_null(sd$fe_interaction_colnames)
})


# ── bf_table() temporal joint BF ─────────────────────────────────────────────

test_that("bf_table temporal returns 1 row (joint only) for p=2, K=1", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = "temporal")

  expect_s3_class(res, "data.frame")
  # K=1: only combined joint (AR/CL sub-joints suppressed to avoid duplication)
  expect_equal(nrow(res), 1)
  expect_true("Temporal (joint)"    %in% res$type)
  expect_false("Temporal AR (joint)" %in% res$type)
  expect_false("Temporal CL (joint)" %in% res$type)

  combined <- res[res$type == "Temporal (joint)", ]
  expect_equal(combined$predictor, "all_phi")
  expect_equal(combined$outcome, "\u2014")
  expect_true(combined$BF10 > 0)
  expect_true(is.finite(combined$BF10))
})

test_that("bf_table temporal collects all phi params (p=2, K=1)", {
  mock <- make_mock_bvarnet("gaussian")
  sd   <- mock$standata
  # p=2, K=1 → 4 phi params total (2 AR + 2 CL)
  all_phi <- get_phi_indices(sd, lag = 1, effect = "all")
  expect_length(all_phi, 4)

  res <- bf_table(mock, type = "temporal")
  combined <- res[res$type == "Temporal (joint)", ]
  # Should return a single row for the joint test
  expect_equal(nrow(combined), 1L)
  expect_true(combined$BF10 > 0)
})

test_that("bf_table temporal can be combined with ar and cl", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock, type = c("ar", "cl", "temporal"))

  # p=2, K=1: ar(2+1) + cl(2+1) + temporal(1: combined only at K=1) = 7
  expect_equal(nrow(res), 7)
  expect_true("Temporal (joint)" %in% res$type)
  expect_true(any(grepl("Autoregressive", res$type)))
  expect_true(any(grepl("Cross-lagged", res$type)))
})

test_that("bf_table temporal works for all families", {
  for (fam in c("gaussian", "bernoulli", "ordinal")) {
    mock <- make_mock_bvarnet(fam)
    res  <- bf_table(mock, type = "temporal")
    # K=1: 1 row (combined joint only)
    expect_equal(nrow(res), 1, info = paste("family:", fam))
    expect_true(all(is.finite(res$BF10)), info = paste("family:", fam))
  }
})

test_that("bf_table temporal without lag interactions emits 1 row at K=1", {
  mock <- make_mock_bvarnet("gaussian")
  res  <- bf_table(mock, type = "temporal")
  # K=1, no lag interactions → combined joint only
  expect_equal(nrow(res), 1)
  expect_equal(res$type, "Temporal (joint)")
  expect_false("Temporal Interaction (joint)" %in% res$type)
  expect_false("Temporal + Interactions (joint)" %in% res$type)
})

test_that("bf_table temporal with lag interaction emits interaction rows", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(123L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "temporal")

  # K=1: 1 combined + 1 ARx + 1 CLx + 1 omnibus = 4
  # (Temporal AR/CL joints suppressed at K=1; Temporal Interaction joint
  #  removed as duplicate of lag_fe)
  expect_equal(nrow(res), 4)
  expect_true("Temporal (joint)" %in% res$type)
  expect_false("Temporal AR (joint)" %in% res$type)
  expect_false("Temporal CL (joint)" %in% res$type)
  expect_false("Temporal Interaction (joint)" %in% res$type)
  expect_true("Temporal AR \u00d7 Interaction (joint)" %in% res$type)
  expect_true("Temporal CL \u00d7 Interaction (joint)" %in% res$type)
  expect_true("Temporal + Interactions (joint)" %in% res$type)
  expect_equal(res$predictor[res$type == "Temporal (joint)"], "all_phi")
  expect_equal(res$predictor[res$type == "Temporal AR \u00d7 Interaction (joint)"], "x_1")
  expect_equal(res$predictor[res$type == "Temporal CL \u00d7 Interaction (joint)"], "x_1")
  expect_equal(res$predictor[res$type == "Temporal + Interactions (joint)"],
               "all_temporal")
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table temporal with 2 interaction terms emits per-term rows", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "x_2",
                "lag1_y_1:x_1", "lag1_y_2:x_1",
                "lag1_y_1:x_2", "lag1_y_2:x_2")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(456L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1"), c("lag", "x_2")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1",
                                  "lag1_y_1:x_2", "lag1_y_2:x_2")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "temporal")

  # K=1: 1 combined
  # + 2 AR×interaction + 2 CL×interaction
  # + 1 omnibus = 6
  # (Temporal AR/CL joints suppressed at K=1; Temporal Interaction joints
  #  removed as duplicate of lag_fe)
  expect_equal(nrow(res), 6)
  expect_false("Temporal AR (joint)" %in% res$type)
  expect_false("Temporal CL (joint)" %in% res$type)
  int_rows <- res[res$type == "Temporal Interaction (joint)", ]
  expect_equal(nrow(int_rows), 0)
  arx_rows <- res[res$type == "Temporal AR \u00d7 Interaction (joint)", ]
  expect_equal(nrow(arx_rows), 2)
  expect_setequal(arx_rows$predictor, c("x_1", "x_2"))
  clx_rows <- res[res$type == "Temporal CL \u00d7 Interaction (joint)", ]
  expect_equal(nrow(clx_rows), 2)
  expect_setequal(clx_rows$predictor, c("x_1", "x_2"))
})

test_that("bf_table temporal p=1: no CL row emitted", {
  # p=1 → only 1 AR param, 0 CL params → combined + AR only = 2
  p <- 1L; K <- 1L; n_fe <- 2L; n_re <- 0L
  par_nms <- c("beta[1,1]", "beta[2,1]", "phi[1,1]", "sigma[1]")

  set.seed(789L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, "y_1"))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, "lag1_y_1"))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(p = p, K = K, n_fe = n_fe, n_re = n_re,
                     Y = Y, X = X, B = B, Z = Z,
                     fe_interaction_terms = list()),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "temporal")
  # K=1: combined joint only (AR sub-joint suppressed at K=1)
  expect_equal(nrow(res), 1)
  expect_true("Temporal (joint)"    %in% res$type)
  expect_false("Temporal AR (joint)" %in% res$type)
  expect_false("Temporal CL (joint)" %in% res$type)
  expect_true(all(res$BF10 > 0))
})


# ── bf_table() type = "all" auto-detection ───────────────────────────────────

test_that("bf_table type='all' works for gaussian (no interactions)", {
  mock <- make_mock_bvarnet("gaussian")
  res <- bf_table(mock)  # default type = "all"

  expect_s3_class(res, "data.frame")
  # intercepts + ar + cl + fe + temporal (no lag_fe)
  # intercepts: 2 + 1 joint = 3
  # ar: 2 + 1 joint = 3
  # cl: 2 + 1 joint = 3
  # fe: 2 + 1 joint = 3 (1 predictor, p=2 → joint; no global joint-all since 1 pred)
  # temporal: 1 at K=1 (combined joint only)
  expect_true(any(grepl("Intercept", res$type)))
  expect_true(any(grepl("Autoregressive", res$type)))
  expect_true(any(grepl("Cross-lagged", res$type)))
  expect_true(any(grepl("Fixed Effect", res$type)))
  expect_true(any(grepl("Temporal", res$type)))
  # No lag interaction rows
  expect_false(any(grepl("Lag Interaction", res$type)))
  expect_true(all(res$BF10 > 0))
})

test_that("bf_table type='all' skips intercepts for ordinal", {
  mock <- make_mock_bvarnet("ordinal")
  res <- bf_table(mock)
  expect_false(any(grepl("^Intercept", res$type)))
  expect_true(any(grepl("Autoregressive", res$type)))
})

test_that("bf_table type='fe' works for pure ordinal (no intercept in X)", {
  mock <- make_mock_bvarnet("ordinal")
  res <- bf_table(mock, type = "fe")
  # Pure ordinal: n_fe=1 (x_1), p=2 → 2 per-cell rows
  per_cell <- res[res$type == "Fixed Effect", ]
  expect_equal(nrow(per_cell), 2L)
  expect_true(all(per_cell$predictor == "x_1"))
  expect_true(all(is.finite(per_cell$BF10) & per_cell$BF10 > 0))
})

test_that("bf_table type='all' includes FE for pure ordinal", {
  mock <- make_mock_bvarnet("ordinal")
  res <- bf_table(mock)
  expect_true(any(res$type == "Fixed Effect"))
})

test_that("bf_table type='all' includes lag_fe when interactions exist", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(111L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock)
  expect_true(any(grepl("Lag Interaction", res$type)))
  expect_true(any(grepl("Temporal", res$type)))
  expect_true(any(grepl("Intercept", res$type)))
  expect_true(all(res$BF10 > 0))
})


# ── Layer 5: bf_table() variable mode ─────────────────────────────────────────

test_that("bf_table(variable = 'y_1', type = 'ar') returns only y_1 self-loop", {
  mock <- make_mock_bvarnet("bernoulli")
  res  <- bf_table(mock, variable = "y_1", type = "ar")

  # p=2, K=1 → only phi[1,1] (y_1 self-loop) + no joint (single param)
  expect_equal(nrow(res), 1L)
  expect_equal(res$type[1], "Autoregressive")
  expect_equal(res$outcome[1], "y_1")
  expect_true(res$BF10[1] > 0)
})

test_that("bf_table(variable = 'y_1', type = 'cl') returns CL effects from y_1", {
  mock <- make_mock_bvarnet("bernoulli")
  res  <- bf_table(mock, variable = "y_1", type = "cl")

  # p=2, K=1 → phi[1,2] (y_1 → y_2), no joint (single param)
  expect_equal(nrow(res), 1L)
  expect_equal(res$type[1], "Cross-lagged")
  expect_equal(res$outcome[1], "y_2")
})

test_that("bf_table(variable = 'y_1') with type = 'all' skips fe/intercepts", {
  mock <- make_mock_bvarnet("bernoulli")
  res  <- bf_table(mock, variable = "y_1")

  # Should have AR + CL + temporal rows, no Intercept or Fixed Effect
  expect_false(any(grepl("Intercept", res$type)))
  expect_false(any(grepl("Fixed Effect", res$type)))
  expect_true(any(res$type == "Autoregressive"))
  expect_true(any(res$type == "Cross-lagged"))
  expect_true(any(grepl("Temporal", res$type)))
  expect_true(all(res$BF10 > 0))
})

test_that("bf_table(variable = c('y_1', 'y_2')) returns effects from both", {
  mock <- make_mock_bvarnet("bernoulli")
  res  <- bf_table(mock, variable = c("y_1", "y_2"), type = "ar")

  # Both self-loops: phi[1,1] and phi[2,2], plus a joint
  expect_equal(nrow(res), 3L)  # 2 per-cell + 1 joint
  expect_true("Autoregressive (joint)" %in% res$type)
})

test_that("bf_table(variable = 'nonexistent') errors informatively", {
  mock <- make_mock_bvarnet("bernoulli")
  expect_error(
    bf_table(mock, variable = "nonexistent"),
    "Unknown variable"
  )
})

test_that("bf_table(network variable + type = 'fe') errors", {
  mock <- make_mock_bvarnet("bernoulli")
  expect_error(
    bf_table(mock, variable = "y_1", type = "fe"),
    "covariate name"
  )
})

test_that("bf_table(variable + type = 'intercepts') errors", {
  mock <- make_mock_bvarnet("bernoulli")
  expect_error(
    bf_table(mock, variable = "y_1", type = "intercepts"),
    "intercepts"
  )
})

test_that("bf_table returns 0-row data frame when variable+type yields no params", {
  # p=1 → no CL params at all; variable="y_1", type="cl" → empty
  p <- 1L; K <- 1L; n_fe <- 2L; n_re <- 0L
  par_nms <- c("beta[1,1]", "beta[2,1]", "phi[1,1]", "sigma[1]")
  set.seed(42L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))
  Y <- matrix(0, 10, 1, dimnames = list(NULL, "y_1"))
  X <- matrix(0, 10, 2, dimnames = list(NULL, c("Intercept", "x_1")))
  B <- matrix(0, 10, 1, dimnames = list(NULL, "lag1_y_1"))
  Z <- matrix(0, 10, 0)
  mock <- structure(list(
    draws = draws,
    standata = list(p = p, K = K, n_fe = n_fe, n_re = n_re,
                    Y = Y, X = X, B = B, Z = Z,
                    fe_interaction_terms = list()),
    priors = set_priors(),
    family = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, variable = "y_1", type = "cl")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), 0L)
  expect_equal(ncol(res), 4L)
})

test_that("bf_table(variable) joint rows only span filtered params", {
  mock <- make_mock_bvarnet("bernoulli")
  # variable = "y_1", type = "ar" → 1 param, no joint
  res1 <- bf_table(mock, variable = "y_1", type = "ar")
  expect_equal(nrow(res1), 1L)
  expect_false(any(grepl("joint", res1$type)))

  # variable = c("y_1", "y_2"), type = "cl" → 2 params + 1 joint
  res2 <- bf_table(mock, variable = c("y_1", "y_2"), type = "cl")
  # y_1→y_2 and y_2→y_1, plus joint
  expect_equal(nrow(res2), 3L)
  expect_true("Cross-lagged (joint)" %in% res2$type)
})

test_that("bf_table(variable) temporal joints scope to variable", {
  mock <- make_mock_bvarnet("bernoulli")
  # variable = "y_1" → temporal should only have phi from y_1
  res <- bf_table(mock, variable = "y_1", type = "temporal")

  # p=2, K=1, variable=y_1 → all_phi: phi[1,1], phi[1,2] (2 params)
  # all_ar: phi[1,1] (1), all_cl: phi[1,2] (1)
  # Should get: Temporal (joint), Temporal AR (joint), Temporal CL (joint)
  expect_true("Temporal (joint)" %in% res$type)
  expect_true(all(res$BF10 > 0))
})

test_that("bf_table(variable + type='lag_fe') filters to variable interactions", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(444L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  # variable = "y_1" → only lag1_y_1:x_1 interactions (1 param per outcome)
  res <- bf_table(mock, variable = "y_1", type = "lag_fe")
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) >= 1L)
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table(variable) with K=2 filters across lags", {
  p <- 2L; K <- 2L
  fe_names <- c("Intercept", "x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- sprintf("phi[%d,%d]",
                      rep(1:(p * K), times = p),
                      rep(1:p, each = p * K))
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(555L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL,
      c("lag1_y_1", "lag1_y_2", "lag2_y_1", "lag2_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(p = p, K = K, n_fe = n_fe, n_re = n_re,
                     Y = Y, X = X, B = B, Z = Z,
                     fe_interaction_terms = list()),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  # variable = "y_1", type = "ar", lag = 1 → phi[1,1] only
  res_ar1 <- bf_table(mock, variable = "y_1", type = "ar", lag = 1L)
  expect_equal(nrow(res_ar1), 1L)

  # variable = "y_1", type = "ar", lag = 2 → phi[3,1] only (row 3 = lag2 y_1)
  res_ar2 <- bf_table(mock, variable = "y_1", type = "ar", lag = 2L)
  expect_equal(nrow(res_ar2), 1L)

  # variable = "y_1", type = "temporal" → phi from y_1 across both lags
  # phi[1,1], phi[1,2], phi[3,1], phi[3,2] = 4 params
  res_t <- bf_table(mock, variable = "y_1", type = "temporal")
  expect_true("Temporal (joint)" %in% res_t$type)
  # K=2 → should also have AR/CL sub-joints
  expect_true("Temporal AR (joint)" %in% res_t$type)
  expect_true("Temporal CL (joint)" %in% res_t$type)
  expect_true(all(is.finite(res_t$BF01) & res_t$BF01 > 0))
})

test_that("bf_table(variable + temporal) with K=1 interactions filters correctly", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(666L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  # variable="y_1" + temporal with interactions
  res <- bf_table(mock, variable = "y_1", type = "temporal")
  expect_s3_class(res, "data.frame")
  expect_true("Temporal (joint)" %in% res$type)
  # Should have omnibus row combining phi + interactions
  expect_true("Temporal + Interactions (joint)" %in% res$type)
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})


# ── Layer 5b: bf_table() covariate variable mode ─────────────────────────────

test_that("bf_table(variable = covariate, type = 'fe') filters to that covariate", {
  mock <- make_mock_bvarnet("gaussian")
  # mock has fe: Intercept, x_1  →  fe type has just x_1
  res <- bf_table(mock, variable = "x_1", type = "fe")
  expect_s3_class(res, "data.frame")
  # p=2, 1 predictor "x_1": 2 per-cell + 1 joint = 3
  expect_equal(nrow(res), 3L)
  expect_true(all(grepl("Fixed Effect", res$type)))
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table(variable = covariate, type = 'all') auto-selects fe + lag_fe", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "x_2", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(777L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  # variable = "x_1" (covariate only) → should get fe rows for x_1 only,
  # lag_fe rows for x_1 interactions, and temporal interaction rows
  res <- bf_table(mock, variable = "x_1")
  expect_s3_class(res, "data.frame")
  # Should include FE rows for x_1 (not x_2)
  fe_rows <- res[grepl("^Fixed Effect", res$type), ]
  expect_true(nrow(fe_rows) > 0)
  expect_true(all(fe_rows$predictor %in% c("x_1", "all_fe")))
  # Should include lag interaction rows
  expect_true(any(grepl("Lag Interaction", res$type)))
  # Should NOT include intercept or AR/CL/temporal phi rows
  expect_false(any(grepl("^Intercept", res$type)))
  expect_false(any(grepl("Autoregressive", res$type)))
  expect_false(any(grepl("Cross-lagged", res$type)))
  expect_false("Temporal (joint)" %in% res$type)
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table(variable = mixed network + covariate) combines both", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(888L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  # Mix network var (y_1) + covariate (x_1)
  res <- bf_table(mock, variable = c("y_1", "x_1"))
  expect_s3_class(res, "data.frame")
  # Should have AR rows (from y_1 phi filtering)
  expect_true(any(grepl("Autoregressive", res$type)))
  # Should have FE rows (from x_1 covariate filtering)
  expect_true(any(grepl("Fixed Effect", res$type)))
  # Should have lag interaction rows
  expect_true(any(grepl("Lag Interaction", res$type)))
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
})

test_that("bf_table(variable = unknown) errors with available names", {
  mock <- make_mock_bvarnet("gaussian")
  expect_error(
    bf_table(mock, variable = "zzz"),
    "Unknown variable"
  )
})

test_that("bf_table(variable = covariate, type = 'ar') errors", {
  mock <- make_mock_bvarnet("gaussian")
  expect_error(
    bf_table(mock, variable = "x_1", type = "ar"),
    "network variable"
  )
  expect_error(
    bf_table(mock, variable = "x_1", type = "cl"),
    "network variable"
  )
})

test_that("mixed-family type='all' does not crash on ordinal intercepts", {
  mock <- make_mock_bvarnet(c("gaussian", "ordinal"))
  # Must not error (ordinal beta[1,j] sentinel would cause savage_dickey crash)
  res <- bf_table(mock)
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) > 0)
  expect_true(all(is.finite(res$BF10) & res$BF10 > 0))
  # Intercept rows should exist for gaussian but not include ordinal sentinel
  int_rows <- res[grepl("^Intercept", res$type), ]
  expect_true(nrow(int_rows) > 0)
})

test_that("K=1 type='all' has no duplicate AR/CL joint rows", {

  mock <- make_mock_bvarnet("gaussian")
  res  <- bf_table(mock)
  # Temporal block at K=1 should only have Temporal (joint), not AR/CL sub-joints
  # (those would be duplicates of the AR (joint) / CL (joint) rows)
  expect_false("Temporal AR (joint)" %in% res$type)
  expect_false("Temporal CL (joint)" %in% res$type)
  # But the regular AR/CL joints should still exist
  expect_true("Autoregressive (joint)" %in% res$type)
  expect_true("Cross-lagged (joint)" %in% res$type)
})

test_that("type='all' with interactions has no duplicate interaction joints", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(222L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock)
  # Temporal Interaction (joint) should NOT appear (duplicate of lag_fe)
  expect_false("Temporal Interaction (joint)" %in% res$type)
  # But lag_fe rows should exist
  expect_true(any(grepl("Lag Interaction", res$type)))
})

test_that("K=1 lag_fe has no duplicate omnibus row", {
  p <- 2L; K <- 1L
  fe_names <- c("Intercept", "x_1", "lag1_y_1:x_1", "lag1_y_2:x_1")
  n_fe <- length(fe_names); n_re <- 0L
  beta_nm <- sprintf("beta[%d,%d]", rep(1:n_fe, times = p), rep(1:p, each = n_fe))
  phi_nm  <- c("phi[1,1]", "phi[2,1]", "phi[1,2]", "phi[2,2]")
  par_nms <- c(beta_nm, phi_nm, "sigma[1]", "sigma[2]")

  set.seed(333L)
  draws <- array(rnorm(40 * 2 * length(par_nms)),
                 dim = c(40L, 2L, length(par_nms)),
                 dimnames = list(NULL, NULL, par_nms))

  Y <- matrix(0, 10, p, dimnames = list(NULL, c("y_1", "y_2")))
  X <- matrix(0, 10, n_fe, dimnames = list(NULL, fe_names))
  B <- matrix(0, 10, p * K, dimnames = list(NULL, c("lag1_y_1", "lag1_y_2")))
  Z <- matrix(0, 10, 0)

  mock <- structure(list(
    draws     = draws,
    standata  = list(
      p = p, K = K, n_fe = n_fe, n_re = n_re, Y = Y, X = X, B = B, Z = Z,
      fe_interaction_terms    = list(c("lag", "x_1")),
      fe_interaction_colnames = c("lag1_y_1:x_1", "lag1_y_2:x_1")
    ),
    priors    = set_priors(),
    family    = "gaussian"
  ), class = "bvarnet")

  res <- bf_table(mock, type = "lag_fe")
  # K=1: only per-lag row, no omnibus (would be identical)
  expect_equal(nrow(res), 1)
  expect_equal(res$type, "Lag Interaction (per lag)")
  expect_false("Lag Interaction (joint)" %in% res$type)
})

---
title: "Mixed-Model"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Mixed-Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




``` r
library(bvarnet)
library(qgraph)
```

## Mixed-family VAR

`bvarnet` allows combining different outcome families in a single VAR model
by passing a named vector to the `family` argument. This is useful when the
variables in the network are measured on different scales.

In this example we fit a mixed-family VAR(1) with three types of outcome:

- **ordinal**: `stress_level` (5-point Likert items)
- **binary**: `happyornot_bin` (binarised daily average)
- **gaussian**: `sleep_duration` (hours)

## Load Data
Again, we start with loading the data

``` r
data(studentlife)
```


### Priors

For a mixed-family model we specify priors for all parameter types that can appear. Here that means we need to specify priors for the following parameters: `intercept`, `phi` (temporal), `beta` (fixed effects), `kappa` (ordinal thresholds), and `sigma` (gaussian residual SD).


``` r
priors <- set_priors(
  intercept = prior(family = "normal", loc = 0, scale = 1),
  phi = prior(family = "normal", loc = 0, scale = 0.5),
  beta = prior(family = "normal", loc = 0, scale = 1),
  kappa = prior(family = "normal", loc = 0, scale = 1),
  sigma = prior(family = "normal", loc = 0, scale = 1)
)
```

### Estimation

We pass a named character vector to `family` so that each outcome gets the
appropriate likelihood.


``` r
fit_mixed <- bvar(
  id_col   = "id",
  time_col = "day",
  y_cols   = c("stress_level", "happyornot", "sleep_duration"),
  x_cols   = NULL,
  re_cols  = NULL,
  re_temporal = FALSE,
  K        = 1,
  data     = studentlife,
  family   = c(stress_level = "ordinal",
               happyornot = "bernoulli",
               sleep_duration = "gaussian"),
  priors   = priors,
  iter     = 4000,
  warmup   = 2000,
  chains   = 4,
  cores    = 4,
  seed     = 1337
)
```

### Results


``` r
print(fit_mixed)
#> BVAR Network fit
#> ======================================== 
#> Family:      mixed (stress_level=ordinal, happyornot=bernoulli, sleep_duration=gaussian)
#> Outcomes (p): 3 
#> Lags (K):     1 
#> Fixed eff.:   1 
#> Observations: 87 
#> Rhat max:    1.001
#> Divergences: 1  WARNING: check model/priors.
#> Priors:
#>   intercept ~ Normal(0, 1)
#>   beta   ~ Normal(0, 1)
#>   phi    ~ Normal(0, 0.5)
#>   sigma  ~ Half-Normal(0, 1)
#>   kappa  ~ Normal(0, 1)
#> Total time:  1.2 sec
#> ========================================
```


``` r
summary(fit_mixed)
#> BVAR Network Summary
#> ================================================== 
#> Family: mixed (stress_level=ordinal, happyornot=bernoulli, sleep_duration=gaussian) | p=3 | K=1 | n=87
#> Rhat max: 1.001 | Divergences: 1
#>   WARNING: divergent transitions detected — check model/priors.
#> 
#> --- Intercept ---
#>  predictor outcome        mean   median q5     q95    rhat ess_bulk ess_tail
#>  Intercept happyornot     -1.167 -1.160 -1.734 -0.626 1    11176.10 9116.660
#>  Intercept sleep_duration  5.644  5.648  5.297  5.986 1    12937.83 9771.961
#> 
#> 
#> --- Autoregressive ---
#>  predictor           outcome        mean  median q5     q95   rhat ess_bulk  ess_tail
#>  lag1_stress_level   stress_level   0.196 0.194   0.033 0.367 1     8462.812 8610.870
#>  lag1_happyornot     happyornot     0.441 0.440  -0.257 1.136 1    12339.308 9618.399
#>  lag1_sleep_duration sleep_duration 0.226 0.225   0.108 0.345 1    10963.284 9073.396
#> 
#> 
#> --- Cross-lagged ---
#>  predictor           outcome        mean   median q5     q95    rhat  ess_bulk  ess_tail 
#>  lag1_happyornot     stress_level    0.188  0.192 -0.258  0.627 1.000 11502.547  9541.818
#>  lag1_sleep_duration stress_level   -0.104 -0.102 -0.189 -0.023 1.000  7852.874  8120.769
#>  lag1_stress_level   happyornot     -0.234 -0.230 -0.650  0.175 1.000  9856.815  9572.424
#>  lag1_sleep_duration happyornot      0.067  0.067 -0.120  0.248 1.000  9073.367  9094.861
#>  lag1_stress_level   sleep_duration -0.198 -0.199 -0.453  0.053 1.000 11200.448  9478.212
#>  lag1_happyornot     sleep_duration -0.292 -0.297 -0.868  0.285 1.001 13926.942 10143.977
#> 
#> 
#> --- Residual SD ---
#>  predictor      outcome mean  median q5    q95  rhat ess_bulk ess_tail
#>  sleep_duration sigma   1.418 1.411  1.249 1.61 1    13626.98 10426.3 
#> 
#> 
#> --- Threshold ---
#>  predictor               outcome mean   median q5     q95   rhat  ess_bulk ess_tail 
#>  kappa(stress_level, c1) —       -0.085 -0.067 -0.458 0.232 1.000  5995.39  4848.848
#>  kappa(stress_level, c2) —        0.243  0.243 -0.034 0.515 1.000 15220.91 12517.771
#>  kappa(stress_level, c3) —        0.484  0.476  0.192 0.803 1.000 15744.65 13041.185
#>  kappa(stress_level, c4) —        0.992  0.955  0.505 1.606 1.001 11753.41 11016.804
#> 
#> 
#> ==================================================
#> Use extract_param() or extract_param(fit, type = "...") for the full parameter table.
#> Use extract_network_matrix() for the temporal network matrix.
```

The temporal network can be extracted and inspected as usual:


``` r
nw_mat <- extract_network_matrix(fit_mixed)
qgraph(nw_mat)
```

![plot of chunk plot-network-mixed](figure/plot-network-mixed-1.png)

As every other model, the mixed model can be extended by random effects (`Vignette(Random-Effects)`), hypothesis tests can be performed (`Vignette(Hypothesis-Testing)`).

---
title: "bvarnet"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{bvarnet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



## Introduction

This package implements methods to analyse (multilevel) Vector Autoregression commonly used for the analysis of intensive longitudinal datasets like EMA data.

In the following vignette we will introduce the basic functionality of the package. For more elaborated usecases, we refer to the additional vignettes

- `vignette("Hypothesis-Testing")`
- `vignette("Mixed-Model")`
- `vignette("Missing-Data")`
- `vignette("Random-Effects")`
- `vignette("MCMC-Diagnostics")`


## Setup

First, lets load the package:

``` r
library(bvarnet)
library(qgraph)
```

## Data

Now, we can load the example data:

``` r
data(studentlife)
```

There is some missing data in the dataset. The models default options handle this by themselves. For a further elaboration on this, you can read `vignette("Missing-Data")`.

Currently, `bvarnet` allowes for the following outcome variables:

- binary (`family = "bernoulli")`
- ordinal (`family = "ordinal")` (adjacent category model)
- continuous (`family = "gaussian")`

For the following tutorial we will investigate the temporal network for the variables `anxious`, `calm`, `conventional`, `critical`, and `dependable`. All these variables are on a ordinal scale. Therefore we will use the adjacent category ordinal model to estimate this model.

## Prior Specification

First we need to specify the prior distributions that we want to use on the parameters.
For this we can use the `set_priors()` function. As we are running a ordinal model, without covariates, we have to specify priors on the temporal structure (denoted by the $\Phi$ matrix) and on the category threshold parameters ($\kappa$) which is specific to the adjacent category ordinal model.


``` r
priors <- set_priors(phi = prior(family = "normal",
                                 loc = 0,
                                 scale = 0.5),
                     kappa = prior(family = "normal",
                                   loc = 0,
                                   scale = 1))

priors
#> bvarnet prior specification:
#>   phi    ~ Normal(0, 0.5)
#>   kappa  ~ Normal(0, 1)
```

Currently, three prior families are implemented: normal, student-t and chauchy. For standart deviations and random effects all of these priors are automatically scaled to half-priors. 
For each distribution, the location and scale parameters can be specified. For the student-t distribution, the degrees of freedom can also be specified.

## Model Estimation


``` r
fit <- bvar(
  id_col = "id",
  time_col = "day",
  y_cols = c("anxious", "calm", "conventional", "critical", "dependable"),
  x_cols = NULL,
  re_cols = NULL,
  re_temporal = FALSE,
  K = 1,
  data = studentlife,
  family = c("ordinal"),
  priors = priors,
  iter = 4000,
  warmup = 1000,
  chains = 4,
  cores = 4,
  seed = 1337
)
```

If you want to learn how to extend this model to a multilevel model, check out the `Vignette(Random-Effects)`.

## Stan Warnings

Most models will give a warning when initiating sampling, that looks like this:

```text
Chain 1 Informational Message: The current Metropolis proposal is about to be rejected because of the following issue:
Chain 1 Exception: categorical_logit_glm_lpmf: Intercept[3] is -inf, but must be finite! (in '/var/folders/n5/38kfmkv55hq8bnd03344d2b40000gn/T/RtmpkjgDq1/model-a60f79e9a37c.stan', line 153, column 8 to column 94)
Chain 1 If this warning occurs sporadically, such as for highly constrained variable types like covariance matrices, then the sampler is fine,
Chain 1 but if this warning occurs often then your model may be either severely ill-conditioned or misspecified.
Chain 1
```

This warning is printed by Stan when a proposed draw briefly violates internal constraints (here, producing a non-finite intercept), so that proposal is rejected. As long as it occurs only occasionally and the chains run, converge, and pass standard diagnostics, this warning is expected and can be safely ignored.

## Model Output

We can print an overview of the model we just ran using the `print(fit)` functions:

``` r
print(fit)
#> BVAR Network fit
#> ======================================== 
#> Family:      ordinal
#> Outcomes (p): 5 
#> Lags (K):     1 
#> Fixed eff.:   0 
#> Observations: 147 
#> Rhat max:    1.001
#> Divergences: 1  WARNING: check model/priors.
#> Priors:
#>   beta   ~ Normal(0, 1)  (default)
#>   phi    ~ Normal(0, 0.5)
#>   kappa  ~ Normal(0, 1)
#> Total time:  17.7 sec
#> ========================================
```

Here we get information about the number of variables and lags, the number of observations and some first indications if the model did converge.

### Summary

To further inspect the model parameters we can use the `summary(fit)` function:


``` r
summary(fit)
#> BVAR Network Summary
#> ================================================== 
#> Family: ordinal | p=5 | K=1 | n=147
#> Rhat max: 1.001 | Divergences: 1
#>   WARNING: divergent transitions detected — check model/priors.
#> 
#> --- Autoregressive ---
#>  predictor         outcome      mean  median q5     q95   rhat  ess_bulk ess_tail
#>  lag1_anxious      anxious      0.190 0.191  -0.064 0.441 1.000 19633.01 11959.17
#>  lag1_calm         calm         0.173 0.173  -0.057 0.409 1.001 15187.45 12033.26
#>  lag1_conventional conventional 0.522 0.522   0.252 0.801 1.000 18655.99 12661.32
#>  lag1_critical     critical     0.273 0.270   0.066 0.492 1.001 24042.28 12349.77
#>  lag1_dependable   dependable   0.459 0.456   0.228 0.697 1.000 19660.02 12414.55
#> 
#> 
#> --- Cross-lagged ---
#>  predictor         outcome      mean   median q5     q95    rhat ess_bulk ess_tail
#>  lag1_calm         anxious      -0.295 -0.293 -0.534 -0.064 1    16400.52 11459.45
#>  lag1_conventional anxious       0.109  0.110 -0.145  0.363 1    19919.94 12271.96
#>  lag1_critical     anxious       0.059  0.059 -0.142  0.266 1    29249.05 11530.32
#>  lag1_dependable   anxious       0.049  0.047 -0.156  0.259 1    18623.56 12379.02
#>  lag1_anxious      calm         -0.080 -0.080 -0.331  0.168 1    19087.17 12676.98
#>  lag1_conventional calm         -0.157 -0.157 -0.416  0.100 1    17990.79 12297.06
#>  lag1_critical     calm         -0.073 -0.072 -0.279  0.137 1    30605.37 12311.23
#>  lag1_dependable   calm          0.110  0.108 -0.096  0.318 1    18954.26 13189.49
#>  lag1_anxious      conventional -0.172 -0.174 -0.438  0.096 1    19308.78 12656.36
#>  lag1_calm         conventional -0.102 -0.100 -0.343  0.136 1    15993.48 13042.57
#> 
#> ... 10 more rows. Use extract_temporal(fit, effect = "cl") for full output.
#> 
#> --- Threshold ---
#>  predictor               outcome mean   median q5     q95    rhat ess_bulk ess_tail
#>  kappa(anxious, c1)      —       -0.987 -0.986 -1.396 -0.581 1    17267.83 10548.39
#>  kappa(calm, c1)         —       -1.201 -1.168 -1.711 -0.796 1    17288.56 10950.22
#>  kappa(conventional, c1) —       -0.986 -0.967 -1.385 -0.648 1    19490.40 12274.73
#>  kappa(critical, c1)     —        0.089  0.097 -0.234  0.380 1    16714.72 12274.38
#>  kappa(dependable, c1)   —       -1.514 -1.475 -2.193 -0.969 1    13656.79 10635.92
#>  kappa(anxious, c2)      —        0.442  0.448  0.125  0.737 1    15943.52 12871.09
#>  kappa(calm, c2)         —       -0.843 -0.835 -1.179 -0.539 1    18624.66 13571.76
#>  kappa(conventional, c2) —       -0.686 -0.687 -0.976 -0.388 1    18078.73 13500.06
#>  kappa(critical, c2)     —        0.529  0.528  0.287  0.775 1    17260.78 14724.44
#>  kappa(dependable, c2)   —       -0.832 -0.833 -1.218 -0.444 1    17188.28 10192.67
#> 
#> ... 10 more rows. Use extract_param(fit, type = "Threshold") for full output.
#> 
#> ==================================================
#> Use extract_param() or extract_param(fit, type = "...") for the full parameter table.
#> Use extract_network_matrix() for the temporal network matrix.
```

### Extracting Parameters

Here we can see, that we can not see all category threshold parameters ($\kappa$). To inspect them completely we have to extract them using `extract_param(fit, type = "Threshold")`:


``` r
params <- extract_param(fit, type = "Threshold")
params
#>         type               predictor outcome        mean     median         q5         q95      rhat ess_bulk ess_tail
#> 26 Threshold      kappa(anxious, c1)       — -0.98739947 -0.9860493 -1.3959425 -0.58128709 1.0002899 17267.83 10548.39
#> 27 Threshold         kappa(calm, c1)       — -1.20056112 -1.1684911 -1.7108714 -0.79625675 1.0001173 17288.56 10950.22
#> 28 Threshold kappa(conventional, c1)       — -0.98550582 -0.9670843 -1.3851722 -0.64823757 1.0000965 19490.40 12274.73
#> 29 Threshold     kappa(critical, c1)       —  0.08918956  0.0971121 -0.2338354  0.38037924 1.0000410 16714.72 12274.38
#> 30 Threshold   kappa(dependable, c1)       — -1.51421608 -1.4750196 -2.1928847 -0.96885779 0.9999901 13656.79 10635.93
#> 31 Threshold      kappa(anxious, c2)       —  0.44160030  0.4480043  0.1246503  0.73744380 1.0001864 15943.52 12871.09
#> 32 Threshold         kappa(calm, c2)       — -0.84343255 -0.8350261 -1.1789986 -0.53859744 0.9999530 18624.66 13571.76
#> 33 Threshold kappa(conventional, c2)       — -0.68574365 -0.6871833 -0.9757996 -0.38764781 1.0000669 18078.73 13500.06
#> 34 Threshold     kappa(critical, c2)       —  0.52905562  0.5279551  0.2874228  0.77458323 1.0002312 17260.78 14724.44
#> 35 Threshold   kappa(dependable, c2)       — -0.83224176 -0.8331054 -1.2180161 -0.44414506 1.0004728 17188.28 10192.67
#> 36 Threshold      kappa(anxious, c3)       —  0.91591078  0.9039044  0.5767963  1.29100115 0.9999471 16410.18 13830.48
#> 37 Threshold         kappa(calm, c3)       — -0.35622581 -0.3623932 -0.6523983 -0.04375198 0.9999353 15096.12 13654.72
#> 38 Threshold kappa(conventional, c3)       —  0.61041320  0.6068505  0.2562171  0.97513834 1.0001069 15027.75 12162.08
#> 39 Threshold     kappa(critical, c3)       —  0.73137454  0.7238967  0.4781784  1.01258300 1.0000583 18194.09 14135.84
#> 40 Threshold   kappa(dependable, c3)       —  0.12394548  0.1239607 -0.2146176  0.46524592 1.0001516 12629.71 10121.86
#> 41 Threshold      kappa(anxious, c4)       —  1.64429481  1.6027258  1.0616271  2.36308576 1.0000596 15643.85 11786.75
#> 42 Threshold         kappa(calm, c4)       —  1.28145587  1.2735726  0.8468702  1.73513969 0.9999143 21851.63 11834.63
#> 43 Threshold kappa(conventional, c4)       —  2.21871737  2.2052999  1.5672800  2.91469544 1.0002161 18781.33 10564.61
#> 44 Threshold     kappa(critical, c4)       —  1.06503664  1.0262242  0.6818237  1.57819682 0.9998645 19805.16 13097.76
#> 45 Threshold   kappa(dependable, c4)       —  1.08776225  1.0817278  0.6448819  1.56455443 1.0003547 14400.05 10104.37
```

## Network Visualization

If we are interested in the temporal network structure, we can inspect this using either the `extract_temporal(fit)`, or the `extract_network_matrix(fit)` functions. Here we will use the `extract_network_matrix(fit)` to plot the network:


``` r
nw_mat <- extract_network_matrix(fit)
qgraph(nw_mat)
```

![plot of chunk plot-network](figure/plot-network-1.png)

Lastly, if you want to perform hypothesis tests on your estimated parameters we refer you to the `Vignette(Hypothesis-Testing)`.

---
title: "Missing-Data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Missing-Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



## Setup

First, lets load the package:

``` r
library(bvarnet)
```


## Data

Now, we can load the example data:

``` r
data(studentlife)
```


## Missing Data in `bvarnet`

Inspecting the dataset, we can see that there are quite a lot of missing values:

``` r
head(studentlife)
#> # A tibble: 6 × 76
#>      id   day happy happyornot   sad sadornot social_number stress_level sleep_hour sleep_quality difficult_stay_awake
#>   <dbl> <dbl> <dbl>      <int> <dbl>    <dbl>         <dbl>        <dbl>      <dbl>         <dbl>                <int>
#> 1     0     1    NA         NA    NA       NA         NA               3       7.5           1.5                     0
#> 2     0     2    NA         NA    NA       NA          4               2       5.33          2.33                    0
#> 3     0     4    NA         NA    NA       NA          2               4       9             2                       0
#> 4     0     5    NA         NA    NA       NA          2              NA       3             4                       0
#> 5     0     6    NA         NA    NA       NA         NA               1      NA            NA                      NA
#> 6     0     7    NA         NA    NA       NA          2.67            2       5             3                       1
#> # ℹ 65 more variables: anxious <dbl>, calm <dbl>, conventional <dbl>, critical <dbl>, dependable <dbl>, disorganized <dbl>,
#> #   enthusiastic <dbl>, open_to_experiences <dbl>, reserved <dbl>, sympathetic <dbl>, act_running_ep_0 <dbl>,
#> #   act_running_ep_1 <dbl>, act_running_ep_2 <dbl>, act_running_ep_3 <dbl>, act_running_ep_4 <dbl>, act_still_ep_0 <dbl>,
#> #   act_still_ep_1 <dbl>, act_still_ep_2 <dbl>, act_still_ep_3 <dbl>, act_still_ep_4 <dbl>, act_unknown_ep_0 <dbl>,
#> #   act_unknown_ep_1 <dbl>, act_unknown_ep_2 <dbl>, act_unknown_ep_3 <dbl>, act_unknown_ep_4 <dbl>, act_walking_ep_0 <dbl>,
#> #   act_walking_ep_1 <dbl>, act_walking_ep_2 <dbl>, act_walking_ep_3 <dbl>, act_walking_ep_4 <dbl>, act_on_foot_ep_0 <dbl>,
#> #   act_on_foot_ep_1 <dbl>, act_on_foot_ep_2 <dbl>, act_on_foot_ep_3 <dbl>, act_on_foot_ep_4 <dbl>, …
```

If we look at the subset of variables that we use in the `Vignette("bvarnet")` c("anxious", "calm", "conventional", "critical", "dependable"), we see there is approximately $90\%$ of missing data:

``` r
sum(is.na(studentlife[c("anxious", "calm", "conventional", "critical", "dependable")])) / (5*nrow(studentlife))
#> [1] 0.8988494
```


## The Skipping-Lag Mechanism

Currently, `bvarnet` handles missing data through a listwise deletion and skipping lag-mechanism. 
Listwise deletion is a computational simple solution to the missing data problem, but does rely on the assumption that data are *missing completely at random (MCAR)*. Violating this assumption may lead to biased parameter estimation.
In longitudinal settings such as like the Vector Autoregressive models, listwise deletion can introduce irregular gaps in the observed time series. This violates the modeling assumption of no missing time points and equal length of time gaps.
To address this issue, we introduce a skipping-lag mechanism. Specifically, lagged effects are only estimated when the time difference between two observations does not exceed a threshold determined by the autoregressive order K. That is, for a VAR(K) model, lagged relationships are only included if the effective time gap corresponds to at most K time steps.
In other words, the model excludes the estimation of temporal parameters across large time gaps, thereby preserving the interpretation of lagged effects as local (short-term) dependencies.

## Comparison: With and Without Lag Skipping

In the `bvar` function the skipping-lag mechanism can be turned on and off by using the `skip_lag` argument. To illustrate this, we can fit the model once using the lag skipping mechanism, and once without:

``` r
fit_no_skip_lag <- bvar(
  id_col = "id",
  time_col = "day",
  y_cols = c("anxious", "calm", "conventional", "critical", "dependable"),
  x_cols = NULL,
  re_temporal = FALSE,
  K = 1,
  skip_lag = FALSE,
  data = studentlife,
  family = c("ordinal"),
  priors = set_priors(),
  iter = 4000,
  warmup = 1000,
  chains = 4,
  cores = 4,
  seed = 1337
)

fit_skip_lag <- bvar(
  id_col = "id",
  time_col = "day",
  y_cols = c("anxious", "calm", "conventional", "critical", "dependable"),
  x_cols = NULL,
  re_temporal = FALSE,
  K = 1,
  skip_lag = TRUE,
  data = studentlife,
  family = c("ordinal"),
  priors = set_priors(),
  iter = 4000,
  warmup = 1000,
  chains = 4,
  cores = 4,
  seed = 1337
)
```

## Model Output

We can print an overview of the models we just ran using the `print(fit)` functions:

``` r
print(fit_no_skip_lag)
#> BVAR Network fit
#> ======================================== 
#> Family:      ordinal
#> Outcomes (p): 5 
#> Lags (K):     1 
#> Fixed eff.:   0 
#> Observations: 67 
#> Rhat max:    1.001
#> Divergences: 4  WARNING: check model/priors.
#> Priors:       beta ~ Normal(0, 1), phi ~ Normal(0, 0.5), kappa ~ Normal(0, 2) (all defaults)
#> Total time:  13.0 sec
#> ========================================
print(fit_skip_lag)
#> BVAR Network fit
#> ======================================== 
#> Family:      ordinal
#> Outcomes (p): 5 
#> Lags (K):     1 
#> Fixed eff.:   0 
#> Observations: 147 
#> Rhat max:    1.001
#> Divergences: 2  WARNING: check model/priors.
#> Priors:       beta ~ Normal(0, 1), phi ~ Normal(0, 0.5), kappa ~ Normal(0, 2) (all defaults)
#> Total time:  20.0 sec
#> ========================================
```

Here we get information about the number of variables and lags, the number of observations and some first indications if the model did converge.

To further inspect the model parameters we can use the `summary(fit)` function:


``` r
summary(fit_no_skip_lag)
#> BVAR Network Summary
#> ================================================== 
#> Family: ordinal | p=5 | K=1 | n=67
#> Rhat max: 1.001 | Divergences: 4
#>   WARNING: divergent transitions detected — check model/priors.
#> 
#> --- Autoregressive ---
#>  predictor         outcome      mean  median q5     q95   rhat ess_bulk ess_tail
#>  lag1_anxious      anxious      0.339 0.338   0.040 0.651 1    14313.98 12663.29
#>  lag1_calm         calm         0.272 0.271  -0.032 0.583 1    12597.37 11566.30
#>  lag1_conventional conventional 0.704 0.699   0.387 1.037 1    14261.66 11256.07
#>  lag1_critical     critical     0.375 0.371   0.132 0.628 1    17694.93 12596.36
#>  lag1_dependable   dependable   0.509 0.503   0.256 0.782 1    15301.30 12277.34
#> 
#> 
#> --- Cross-lagged ---
#>  predictor         outcome      mean   median q5     q95    rhat ess_bulk ess_tail
#>  lag1_calm         anxious      -0.302 -0.300 -0.608 -0.003 1    10483.74 10555.98
#>  lag1_conventional anxious       0.214  0.213 -0.090  0.522 1    15024.71 12494.95
#>  lag1_critical     anxious       0.131  0.130 -0.110  0.375 1    18724.81 11340.99
#>  lag1_dependable   anxious       0.114  0.114 -0.129  0.364 1    14973.68 12577.59
#>  lag1_anxious      calm         -0.115 -0.116 -0.426  0.195 1    13347.20 11867.03
#>  lag1_conventional calm         -0.226 -0.224 -0.549  0.091 1    13782.69 12198.55
#>  lag1_critical     calm         -0.110 -0.111 -0.363  0.146 1    18142.55 12199.66
#>  lag1_dependable   calm          0.180  0.177 -0.073  0.442 1    15069.37 11928.87
#>  lag1_anxious      conventional -0.156 -0.153 -0.472  0.157 1    13342.76 12063.64
#>  lag1_calm         conventional -0.040 -0.040 -0.333  0.253 1    11344.55 12031.13
#> 
#> ... 10 more rows. Use extract_temporal(fit, effect = "cl") for full output.
#> 
#> --- Threshold ---
#>  predictor               outcome mean   median q5     q95    rhat  ess_bulk ess_tail
#>  kappa(anxious, c1)      —       -0.699 -0.689 -2.162  0.738 1.000 12338.40 12029.46
#>  kappa(calm, c1)         —       -2.297 -2.259 -4.094 -0.655 1.000 12290.58 10066.12
#>  kappa(conventional, c1) —       -0.639 -0.631 -2.081  0.774 1.000 14940.93 11881.86
#>  kappa(critical, c1)     —       -0.004  0.005 -1.379  1.347 1.000 13376.88 11823.21
#>  kappa(dependable, c1)   —       -1.497 -1.482 -2.988 -0.071 1.000 13894.65 11128.82
#>  kappa(anxious, c2)      —        1.569  1.569  0.188  2.938 1.000 17380.14 11741.22
#>  kappa(calm, c2)         —       -1.036 -1.021 -2.468  0.355 1.000 21368.65 12745.46
#>  kappa(conventional, c2) —       -0.146 -0.144 -1.519  1.231 1.001 16691.08 13160.89
#>  kappa(critical, c2)     —        0.945  0.939 -0.384  2.291 1.000 15379.49 12143.76
#>  kappa(dependable, c2)   —       -0.631 -0.622 -1.978  0.699 1.001 18780.65 12187.75
#> 
#> ... 10 more rows. Use extract_param(fit, type = "Threshold") for full output.
#> 
#> ==================================================
#> Use extract_param() or extract_param(fit, type = "...") for the full parameter table.
#> Use extract_network_matrix() for the temporal network matrix.
summary(fit_skip_lag)
#> BVAR Network Summary
#> ================================================== 
#> Family: ordinal | p=5 | K=1 | n=147
#> Rhat max: 1.001 | Divergences: 2
#>   WARNING: divergent transitions detected — check model/priors.
#> 
#> --- Autoregressive ---
#>  predictor         outcome      mean  median q5     q95   rhat ess_bulk ess_tail
#>  lag1_anxious      anxious      0.203 0.203  -0.054 0.460 1    15617.37 12599.58
#>  lag1_calm         calm         0.176 0.174  -0.058 0.415 1    13025.39 11190.21
#>  lag1_conventional conventional 0.558 0.554   0.280 0.843 1    14346.16 11205.97
#>  lag1_critical     critical     0.278 0.277   0.075 0.492 1    18103.80 11283.74
#>  lag1_dependable   dependable   0.476 0.473   0.247 0.714 1    15253.27 12152.74
#> 
#> 
#> --- Cross-lagged ---
#>  predictor         outcome      mean   median q5     q95    rhat ess_bulk ess_tail
#>  lag1_calm         anxious      -0.308 -0.307 -0.556 -0.069 1    12276.01 12256.25
#>  lag1_conventional anxious       0.113  0.112 -0.145  0.375 1    15288.24 12209.56
#>  lag1_critical     anxious       0.065  0.065 -0.142  0.273 1    21335.42 11693.19
#>  lag1_dependable   anxious       0.054  0.052 -0.158  0.267 1    14645.31 12404.76
#>  lag1_anxious      calm         -0.085 -0.085 -0.338  0.168 1    15499.73 13047.83
#>  lag1_conventional calm         -0.162 -0.162 -0.423  0.096 1    14847.92 12191.17
#>  lag1_critical     calm         -0.077 -0.077 -0.286  0.131 1    20946.54 12223.59
#>  lag1_dependable   calm          0.116  0.115 -0.096  0.328 1    15982.24 11988.57
#>  lag1_anxious      conventional -0.184 -0.184 -0.462  0.095 1    14348.25 12403.62
#>  lag1_calm         conventional -0.105 -0.105 -0.349  0.134 1    12989.52 12059.56
#> 
#> ... 10 more rows. Use extract_temporal(fit, effect = "cl") for full output.
#> 
#> --- Threshold ---
#>  predictor               outcome mean   median q5     q95    rhat  ess_bulk ess_tail 
#>  kappa(anxious, c1)      —       -1.033 -1.029 -1.470 -0.613 1.000 11123.68  9566.673
#>  kappa(calm, c1)         —       -1.300 -1.266 -1.869 -0.848 1.000 14564.11 10916.416
#>  kappa(conventional, c1) —       -1.033 -1.012 -1.471 -0.674 1.000 14373.14 11229.523
#>  kappa(critical, c1)     —        0.086  0.095 -0.239  0.379 1.000 12541.34 10745.366
#>  kappa(dependable, c1)   —       -1.726 -1.680 -2.521 -1.081 1.000 10748.70 10274.783
#>  kappa(anxious, c2)      —        0.459  0.464  0.139  0.765 1.000 13762.57 12456.461
#>  kappa(calm, c2)         —       -0.884 -0.875 -1.244 -0.552 1.000 18423.96 13368.980
#>  kappa(conventional, c2) —       -0.707 -0.710 -1.008 -0.397 1.000 15460.49 10840.830
#>  kappa(critical, c2)     —        0.548  0.547  0.298  0.806 1.000 16603.34 12957.191
#>  kappa(dependable, c2)   —       -0.859 -0.860 -1.263 -0.456 1.001 10048.09  6937.794
#> 
#> ... 10 more rows. Use extract_param(fit, type = "Threshold") for full output.
#> 
#> ==================================================
#> Use extract_param() or extract_param(fit, type = "...") for the full parameter table.
#> Use extract_network_matrix() for the temporal network matrix.
```

### Why does `skip_lag = FALSE` produce fewer observations?

When `skip_lag = FALSE`, any row whose previous observation is not exactly one time step away lacks a valid lag. Because the autoregressive component cannot be computed for that row, the row is dropped entirely from the likelihood. This is the conservative choice: every observation that remains has a genuine lagged value, so estimates of the temporal coefficients ($\phi$) are unbiased (given the model).

When `skip_lag = TRUE` (the default), those same rows are kept. The contribution of the autoregressive effect is set to zero, while the outcome is still modeled through the intercept, fixed effects, and (if present) random effects. Oly the temporal component is "switched off" for that row.

### Practical implications

The two approaches trade off sample size against potential bias in the autoregressive estimates:

* **`skip_lag = FALSE`** — Every row has a valid lag, so $\phi$ estimates reflect true short-term dynamics. However, in datasets with many irregular gaps (such as the present one), a large share of observations is discarded, reducing statistical power for all parameters.
* **`skip_lag = TRUE`** — All observations contribute to the estimation of the intercept, fixed effects, and residual variance. Because zero-filled lags are treated as "no temporal information" rather than "the lag was zero", rows with gaps gently pull $\phi$ toward zero. In datasets with many gaps this can lead to a slight weakening of autoregressive estimates.

Currently, more sophisticated missing data handling (e.g. imputation) has to be implemented by the users themselves.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{format.bvarnet_prior}
\alias{format.bvarnet_prior}
\title{Format a bvarnet_prior for printing}
\usage{
\method{format}{bvarnet_prior}(x, half = FALSE, ...)
}
\arguments{
\item{x}{A \code{bvarnet_prior} object.}

\item{half}{Logical; if \code{TRUE} prepends "Half-" to indicate a
half-prior (used for positive-constrained parameters like sd_u and sigma).}

\item{...}{Ignored.}
}
\value{
A character string.
}
\description{
Format a bvarnet_prior for printing
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{extract_draws}
\alias{extract_draws}
\title{Extract raw posterior draws for a single parameter block}
\usage{
extract_draws(object, parameter = c("beta", "phi", "sd_u", "sigma", "kappa"))
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{\link{bvar}}.}

\item{parameter}{Character. One of \code{"beta"}, \code{"phi"},
\code{"sd_u"}, \code{"sigma"}, or \code{"kappa"}.}
}
\value{
A numeric matrix with one row per posterior draw and one column
per Stan parameter element.
}
\description{
Returns an \code{(iterations * chains)} by \code{params} matrix with
Stan-indexed column names (e.g. \code{"beta[1,1]"}, \code{"phi[2,3]"}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{get_default_priors}
\alias{get_default_priors}
\title{Get the default prior specification for a given model family}
\usage{
get_default_priors(family = NULL, has_re = TRUE)
}
\arguments{
\item{family}{Character (optional). One of \code{"bernoulli"},
\code{"ordinal"}, \code{"gaussian"}. When \code{NULL} (the default),
all parameter priors are shown.}

\item{has_re}{Logical. Does the model include random effects?
Default \code{TRUE}. When \code{FALSE}, the \code{sd_u} prior is
omitted.}
}
\value{
A \code{bvarnet_priors} object.
}
\description{
Returns a \code{bvarnet_priors} object showing the default priors that
apply to a particular model configuration. Parameters irrelevant to
the chosen family or model structure are omitted, so the returned object
reflects what the sampler will actually use.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{print.bvarnet}
\alias{print.bvarnet}
\title{Print a bvarnet model object}
\usage{
\method{print}{bvarnet}(x, ...)
}
\arguments{
\item{x}{A \code{bvarnet} object.}

\item{...}{Ignored.}
}
\value{
\code{x} invisibly.
}
\description{
Displays a brief summary of the fitted model: family, dimensions,
Rhat, divergences, chain return codes, priors, and total sampling time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{print.bvarnet_priors}
\alias{print.bvarnet_priors}
\title{Print a bvarnet_priors specification}
\usage{
\method{print}{bvarnet_priors}(x, ...)
}
\arguments{
\item{x}{A \code{bvarnet_priors} object.}

\item{...}{Ignored.}
}
\value{
\code{x} invisibly.
}
\description{
Shows only the priors explicitly set by the user.  When no priors have
been overridden (all defaults), a compact note is printed instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_factor.R
\name{bf_table}
\alias{bf_table}
\title{Compute Savege-Dickey Bayes factors}
\usage{
bf_table(
  object,
  type = "all",
  lag = 1L,
  null_value = 0,
  variable = NULL,
  log_BF10 = FALSE,
  round = 5L
)
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{bvar()}.}

\item{type}{Character vector specifying which parameter groups to test.
Use \code{"all"} (default) to include all applicable groups automatically.
Available options:
\describe{
\item{\code{"ar"}}{Autoregressive effects (self-loops). Per-cell BFs
for the lag specified by \code{lag}, plus a joint BF.}
\item{\code{"cl"}}{Cross-lagged effects. Same structure as \code{"ar"}.}
\item{\code{"intercepts"}}{Intercept parameters. Skipped automatically
for ordinal outcomes.}
\item{\code{"fe"}}{Non-intercept fixed effects (covariates).}
\item{\code{"lag_fe"}}{Joint BFs for lag \eqn{\times} covariate
interaction terms. Only available when the model was fitted with
\code{fe_interactions} containing lag terms.}
\item{\code{"temporal"}}{Joint BF for the entire temporal structure
(all AR + CL parameters across all lags). When lag \eqn{\times}
covariate interactions are present, additional omnibus rows are
included.}
}}

\item{lag}{Integer; which lag block to use (default 1). Applies to
\code{"ar"} and \code{"cl"} types.}

\item{null_value}{Numeric scalar; the null hypothesis value (default 0).}

\item{variable}{Character vector or \code{NULL} (default).  One or more
variable names.  When set, only effects involving these variables are
included.}

\item{log_BF10}{Logical; if \code{TRUE}, an additional \code{log_BF10}
column (natural log of \code{BF10}) is appended to the output.
Default is \code{FALSE}.}

\item{round}{Integer or \code{NULL}; number of decimal places to round
numeric output columns.  Default is \code{5}.  Set to \code{NULL} to
disable rounding.}
}
\value{
A data frame with columns: \code{type}, \code{predictor},
\code{outcome}, \code{BF10} (and optionally \code{log_BF10}).
}
\description{
Computes Savage-Dickey density ratio Bayes factors for each (requested set of) parameter in the model.
By default, all applicable parameters are tested and returned in a tidy data frame.
The \code{type} argument controls which parameter groups are included; the \code{variable} argument can be used to filter to effects involving specific variables.
The \code{log_BF10} argument allows including the natural log of the Bayes factor in the output, and \code{round} controls numeric rounding of the results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractors.R
\name{extract_network_matrix}
\alias{extract_network_matrix}
\title{Extract a network matrix of temporal coefficients}
\usage{
extract_network_matrix(
  object,
  lag = 1L,
  stat = c("mean", "median", "q5", "q95")
)
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{\link{bvar}}.}

\item{lag}{Integer. Which lag block. Default 1.}

\item{stat}{Character. Summary statistic to fill the matrix with:
\code{"mean"} (default), \code{"median"}, \code{"q5"}, or \code{"q95"}.}
}
\value{
A named \code{p x p} numeric matrix. Element \code{[i, j]}
gives the effect of variable \code{i} (lagged) on variable \code{j}
(outcome). Row and column names are the outcome variable names.
}
\description{
Returns a named \code{p x p} matrix of posterior summary statistics for
the VAR lag coefficients at a chosen lag, suitable for network
visualisation (e.g., with \pkg{igraph} or \pkg{qgraph}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_bvarnet.R
\name{compare_to_truth}
\alias{compare_to_truth}
\title{Compare fitted model parameters to simulation truth}
\usage{
compare_to_truth(
  fit,
  truth,
  ci_width = 0.9,
  bayes_factor = FALSE,
  null_value = 0
)
}
\arguments{
\item{fit}{A fitted \code{bvarnet} object (output from \code{bvar()}).}

\item{truth}{The \code{truth} component from \code{sim_var()} output.}

\item{ci_width}{Numeric. Width of the credible interval (default 0.90).}

\item{bayes_factor}{Logical; if \code{TRUE}, compute Savage-Dickey BFs for
beta and phi parameters and append \code{BF01}, \code{BF10}, and
\code{bf_correct} columns.  \code{bf_correct} is \code{TRUE} when BF01 > 1
for true null parameters (true value == \code{null_value}) and BF10 > 1
for true non-null parameters.  Default \code{FALSE}.}

\item{null_value}{Numeric scalar; the null hypothesis value for Bayes
factor computation (default 0).  Only used when \code{bayes_factor = TRUE}.}
}
\value{
A data frame with columns: parameter, node, true_value,
post_mean, post_sd, ci_lower, ci_upper, covered (logical), and optionally
BF01, BF10, bf_correct.
}
\description{
Extracts posterior summaries from a fitted \code{bvarnet} object and
compares them to the true parameter values used for data generation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{set_priors}
\alias{set_priors}
\title{Build a prior specification object for \code{bvar()}}
\usage{
set_priors(
  intercept = NULL,
  beta = NULL,
  phi = NULL,
  sd_u = NULL,
  kappa = NULL,
  sigma = NULL
)
}
\arguments{
\item{intercept}{Prior for the intercept. Only applies to gaussian and bernoulli models;
for ordinal models the intercept is absorbed into the kappa (threshold parameter).}

\item{beta}{Prior for fixed-effect regression coefficients (slopes).}

\item{phi}{Prior for lag coefficients.}

\item{sd_u}{Prior for random-effect standard deviations (half-prior).}

\item{kappa}{Prior for ordinal cut-points (ordinal models only).}

\item{sigma}{Prior for residual standard deviation (gaussian models only;
half-prior).}
}
\value{
A \code{bvarnet_priors} S3 object.
}
\description{
Returns a \code{bvarnet_priors} object containing a \code{bvarnet_prior}
for every model parameter type.  Any argument left as \code{NULL} uses the
package default.
Available prior distributions are:
\itemize{
\item normal(loc, scale)
\item student_t(loc, scale, df)
\item cauchy(loc, scale)
For standart deviations and random effects, the prior is automatically converted to a half-prior (truncated at \code{loc}) in the Stan code, so the printed format reflects this.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bvar.R
\name{bvar}
\alias{bvar}
\title{Fit a Bayesian multilevel VAR network model}
\usage{
bvar(
  id_col,
  time_col,
  y_cols,
  x_cols = NULL,
  center_x = FALSE,
  fe_interactions = NULL,
  re_interactions = NULL,
  re_cols = NULL,
  re_temporal = FALSE,
  K = 1,
  na_action = c("listwise"),
  skip_lag = TRUE,
  data,
  family = c("bernoulli", "ordinal", "gaussian"),
  priors = set_priors(),
  iter = 4000,
  warmup = 1000,
  chains = 4,
  cores = 1,
  seed = NULL,
  adapt_delta = NULL,
  max_treedepth = NULL,
  save_data = FALSE
)
}
\arguments{
\item{id_col}{Character. Name of the subject/group identifier column.}

\item{time_col}{Character. Name of the time column.}

\item{y_cols}{Character vector. Names of the outcome columns.}

\item{x_cols}{Character vector or NULL. Names of the covariate columns.}

\item{center_x}{Logical. Grand-mean centre covariates before fitting?
Default \code{FALSE}.}

\item{fe_interactions}{List or NULL. Fixed-effect interaction terms to add
to the design matrix. Each element is a character vector of column names
to interact, or \code{c("lag", "x")} to interact all lag columns with
a covariate.}

\item{re_interactions}{List or NULL. Random-effect interaction terms.}

\item{re_cols}{Character vector. Columns from X and/or "Intercept" to include as random slopes.}

\item{re_temporal}{Logical. Include random slopes on lag predictors?
Default \code{FALSE}.}

\item{K}{Integer. AR order. Default 1.}

\item{na_action}{Character. Missing-data strategy; currently only
\code{"listwise"}.}

\item{skip_lag}{Logical. If \code{TRUE} (default), rows with irregular time
gaps have their lag set to zero rather than being dropped.}

\item{data}{Data frame in long format.}

\item{family}{Character scalar or vector. Observation model per node.
A scalar is recycled to all \code{y_cols}. A vector of length
\code{length(y_cols)} (named or positional) specifies per-node families.
Valid values: \code{"bernoulli"}, \code{"ordinal"}, \code{"gaussian"}.}

\item{priors}{A \code{bvarnet_priors} object from \code{set_priors()}.
Defaults to \code{set_priors()} (package defaults).}

\item{iter}{Integer. Number of post-warmup iterations per chain. Default 4000.}

\item{warmup}{Integer. Number of warmup iterations per chain. Default 1000.}

\item{chains}{Integer. Number of MCMC chains. Default 4.}

\item{cores}{Integer. Number of chains to run in parallel. Default 1.}

\item{seed}{Integer or NULL. RNG seed.}

\item{adapt_delta}{Numeric in (0, 1). Target average proposal acceptance
probability during warmup adaptation. Higher values (e.g., 0.95–0.99)
reduce divergences at the cost of slower sampling. Default \code{NULL}
(CmdStan default of 0.8).}

\item{max_treedepth}{Integer. Maximum depth of the NUTS binary tree.
Increasing this allows the sampler to take more leapfrog steps per
iteration, which can help with difficult posteriors (e.g., funnels in
hierarchical logistic models) but increases computation. Default
\code{NULL} (CmdStan default of 10).}

\item{save_data}{Logical. If \code{TRUE}, store the preprocessed (sorted,
listwise-deleted) estimation data in the \code{data_used} slot of the
returned object for reproducibility and downstream analyses.
Default \code{FALSE}.}
}
\value{
A \code{bvarnet} object (a named list) with slots:
\code{draws}, \code{convergence}, \code{diagnostics}, \code{timing},
\code{metadata}, \code{return_codes}, \code{family}, \code{standata},
\code{priors}. If \code{save_data = TRUE}, also includes
\code{data_used} (the cleaned estimation data frame).
}
\description{
The \code{bvar} function estimates the posterior distribution of the specified Bayesian (Multilevel) Vector Autoregression.
}
\examples{
\dontrun{
# Run bvar on studentlife data
data(studentlife, package = "bvarnet")
fit <- bvar(
  id_col = "id",
  time_col = "time",
  y_cols = c("anxious", "calm", "conventional", "critical", "dependable"),
  re_temporal = TRUE,
  K = 1,
  data = studentlife,
  family = "ordinal",
  priors = set_priors(),
  seed = 1337)

summary(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{print.bvarnet_prior}
\alias{print.bvarnet_prior}
\title{Print a bvarnet_prior}
\usage{
\method{print}{bvarnet_prior}(x, ...)
}
\arguments{
\item{x}{A \code{bvarnet_prior} object.}

\item{...}{Passed to \code{format.bvarnet_prior()}.}
}
\value{
\code{x} invisibly.
}
\description{
Print a bvarnet_prior
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/studentlife.R
\docType{data}
\name{studentlife}
\alias{studentlife}
\title{StudentLife Data}
\format{
\subsection{\code{studentlife}}{

A data frame with 1912 rows and 72 columns:
}
}
\source{
https://openesmdata.org/datasets/0004_wang/

https://doi.org/10.1145/2632048.2632054
}
\usage{
studentlife
}
\description{
Assessing mental health, academic performance and behavioral trends of college students using smartphones.
Wang, R., Chen, F., Chen, Z., Li, T., Harari, G., Tignor, S., Zhou, X., Ben-Zeev, D., & Campbell, A. T. (2014).
StudentLife: assessing mental health, academic performance and behavioral trends of college students using smartphones.
Proceedings of the 2014 ACM International Joint Conference on Pervasive and Ubiquitous Computing, 3–14. https://doi.org/10.1145/2632048.2632054
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_param.R
\name{extract_param}
\alias{extract_param}
\title{Extract labelled parameter summaries from a fitted bvarnet model}
\usage{
extract_param(object, bayes_factor = FALSE, null_value = 0, type = NULL)
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{bvar()}.}

\item{bayes_factor}{Logical; if \code{TRUE}, append \code{BF01} and
\code{BF10} columns computed via the Savage-Dickey density ratio for
beta and phi parameters.  Default \code{FALSE}.}

\item{null_value}{Numeric scalar; the null hypothesis value for Bayes
factor computation (default 0).  Only used when \code{bayes_factor = TRUE}.}

\item{type}{Character vector or \code{NULL} (default). If supplied, only
rows matching the given type(s) are returned.  Valid values are:
\code{"Intercept"}, \code{"Fixed Effect"}, \code{"Autoregressive"},
\code{"Cross-lagged"}, \code{"Random Effect SD"}, \code{"Residual SD"},
\code{"Threshold"}.}
}
\value{
A data frame with columns: \code{type}, \code{predictor},
\code{outcome}, \code{mean}, \code{median}, \code{q5}, \code{q95},
\code{rhat}, \code{ess_bulk}, \code{ess_tail}, and optionally
\code{BF01}, \code{BF10}.
}
\description{
Returns a single flat data frame with posterior summaries (mean, median,
5th/95th percentiles) and convergence diagnostics (Rhat, ESS) for all
model parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bvarnet-package.R
\docType{package}
\name{bvarnet-package}
\alias{bvarnet}
\alias{bvarnet-package}
\title{bvarnet: Bayesian Estimation of Dynamic VAR Models using STAN}
\description{
Bayesian estimation of multilevel Vector Autoregression (VAR) models using Stan. Supports Gaussian, Binary, and Ordinal (adjacent category) outcome variables with random effects and customizable priors.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://flo1met.github.io/bvarnet/}
  \item \url{https://github.com/flo1met/bvarnet}
  \item Report bugs at \url{https://github.com/flo1met/bvarnet/issues}
}

}
\author{
\strong{Maintainer}: Florian Metwaly \email{f.j.metwaly@uva.nl} (\href{https://orcid.org/0009-0001-8357-7870}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_bvarnet.R
\name{sim_var}
\alias{sim_var}
\title{Simulate data from a multilevel VAR model}
\usage{
sim_var(
  N,
  T_obs,
  p,
  K = 1L,
  family = c("bernoulli", "ordinal", "gaussian"),
  alpha = NULL,
  gamma = NULL,
  Phi = NULL,
  sigma = NULL,
  kappa = NULL,
  q = 0L,
  x_gen = NULL,
  sd_alpha = 0.5,
  sd_phi = 0.2,
  sd_gamma = NULL,
  re_temporal = FALSE,
  C = 5L,
  burnin = 500L,
  seed = NULL
)
}
\arguments{
\item{N}{Integer. Number of subjects (groups).}

\item{T_obs}{Integer. Number of time points per subject.}

\item{p}{Integer. Number of outcome nodes.}

\item{K}{Integer. AR order (default 1).}

\item{family}{Character. One of \code{"bernoulli"}, \code{"ordinal"},
\code{"gaussian"}.}

\item{alpha}{Numeric vector of length \code{p}. Population intercepts
(on logit scale for bernoulli, identity for gaussian). For ordinal,
this is absorbed into kappa and should be left NULL. Generated if NULL.}

\item{gamma}{Matrix \code{q x p}. Population covariate effects. Generated
if NULL and \code{q > 0}.}

\item{Phi}{Matrix \code{(p*K) x p}. Population lag coefficients. Generated
if NULL.}

\item{sigma}{Numeric vector of length \code{p}. Residual SD per node
(gaussian only). Generated if NULL.}

\item{kappa}{List of \code{p} ordered vectors, each of length \code{C-1}.
Cutpoints per node (ordinal only). Generated if NULL.}

\item{q}{Integer. Number of covariates (default 0).}

\item{x_gen}{Function \code{f(N, T_obs)} returning an \code{N x T_obs x q}
array of covariates. If NULL, default generation is used.}

\item{sd_alpha}{Numeric. SD of random intercepts (scalar or p-vector).
Default 0.5. Set to 0 to simulate a fixed-effects-only model with no
between-person variation in intercepts.}

\item{sd_phi}{Numeric. SD of random lag coefficients (scalar or matrix).
Default 0.2.}

\item{sd_gamma}{Numeric or NULL. SD of random covariate slopes. NULL means
no random slopes on covariates.}

\item{re_temporal}{Logical. Include random slopes on lag predictors?
Default FALSE.}

\item{C}{Integer. Number of ordinal categories (ordinal only, default 5).}

\item{burnin}{Integer. Number of time points to discard as warmup before
recording data (default 500). The VAR process is simulated for
\code{burnin + T_obs} time points per subject, and the first
\code{burnin} are discarded. This allows the process to reach its
stationary distribution before data collection begins.}

\item{seed}{Integer or NULL. RNG seed.}
}
\value{
A list with two components:
\describe{
\item{data}{A long-format data frame with columns \code{id}, \code{t},
\code{y_1}, ..., \code{y_p}, and optionally \code{x_1}, ..., \code{x_q}.}
\item{truth}{A list of true generating parameters.}
}
}
\description{
Generates data from the generative model implied by each Stan model family.
Useful for testing parameter recovery and model validation.
}
\details{
To simulate a VAR without any random effects (i.e. all subjects share
identical parameters), set \code{sd_alpha = 0}, \code{re_temporal = FALSE}
(the default), and \code{sd_gamma = NULL} (the default).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractors.R
\name{extract_random_effects}
\alias{extract_random_effects}
\title{Extract random-effect summaries}
\usage{
extract_random_effects(object, what = c("sd", "mean_u", "draws_u"))
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{\link{bvar}}.}

\item{what}{Character. What to extract:
\describe{
\item{\code{"sd"}}{Data frame of random-effect SD summaries
(from \code{extract_param}).}
\item{\code{"mean_u"}}{3D array \code{[node, subject, re]} of
posterior means of subject-level effects.}
\item{\code{"draws_u"}}{4D array \code{[draw, node, subject, re]}
of full posterior draws.}
}}
}
\value{
Depends on \code{what}; see above.
}
\description{
Returns random-effect standard deviations (group-level variance),
subject-level posterior means, or the full posterior draws of the
subject-level random effects \code{u}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_priors.R
\name{prior}
\alias{prior}
\title{Construct a single prior distribution}
\usage{
prior(family, loc = 0, scale = 1, df = 7)
}
\arguments{
\item{family}{Character. One of \code{"normal"}, \code{"student_t"},
\code{"cauchy"}.}

\item{loc}{Location parameter (default 0).}

\item{scale}{Scale parameter (default 1). Must be > 0.}

\item{df}{Degrees of freedom for \code{"student_t"} (default 7). Must be0 when \code{family = "student_t"}.}
}
\value{
A \code{bvarnet_prior} S3 object.
}
\description{
Builds a \code{bvarnet_prior} object specifying the prior family and its
parameters.  Supported families in Phase 1 are \code{"normal"},
\code{"student_t"}, and \code{"cauchy"}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.bvarnet}
\alias{summary.bvarnet}
\title{Summary method for bvarnet objects}
\usage{
\method{summary}{bvarnet}(object, bayes_factor = FALSE, null_value = 0, ...)
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{\link{bvar}}.}

\item{bayes_factor}{Logical; if \code{TRUE}, append Savage-Dickey BF
columns. Default \code{FALSE}.}

\item{null_value}{Numeric scalar; null hypothesis value for BF computation.
Default 0.}

\item{...}{Ignored.}
}
\value{
An object of class \code{"summary.bvarnet"} (a list) with elements:
\describe{
\item{table}{Data frame from \code{extract_param()}.}
\item{family}{Model family.}
\item{p}{Number of outcome variables.}
\item{K}{AR order.}
\item{n}{Number of observations.}
\item{rhat_max}{Maximum Rhat across all parameters.}
\item{n_divergences}{Total divergent transitions.}
}
}
\description{
Returns a labelled posterior summary table grouped by parameter type,
with convergence diagnostics and optional Bayes factors. Wraps
\code{\link{extract_param}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.summary.bvarnet}
\alias{print.summary.bvarnet}
\title{Print a bvarnet summary}
\usage{
\method{print}{summary.bvarnet}(x, digits = 3, max_rows = 10, ...)
}
\arguments{
\item{x}{A \code{summary.bvarnet} object.}

\item{digits}{Number of decimal digits for numeric columns. Default 3.}

\item{max_rows}{Maximum number of rows to print per parameter group.
Default 10.}

\item{...}{Ignored.}
}
\value{
\code{x} invisibly.
}
\description{
Pretty-prints the output of \code{\link{summary.bvarnet}}, grouping
parameters by type and displaying convergence information.
Each group is truncated to \code{max_rows} rows; use \code{extract_param()}
or dedicated extractors to see full output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractors.R
\name{extract_temporal}
\alias{extract_temporal}
\title{Extract temporal (VAR lag) effects}
\usage{
extract_temporal(
  object,
  lag = NULL,
  effect = c("all", "ar", "cl"),
  bayes_factor = FALSE,
  null_value = 0
)
}
\arguments{
\item{object}{A \code{bvarnet} object returned by \code{\link{bvar}}.}

\item{lag}{Integer or \code{NULL}. If specified, only effects from this
lag are returned. Default \code{NULL} (all lags).}

\item{effect}{Character. One of \code{"all"} (default), \code{"ar"}
(autoregressive only), or \code{"cl"} (cross-lagged only).}

\item{bayes_factor}{Logical; if \code{TRUE}, append BF columns.
Default \code{FALSE}.}

\item{null_value}{Numeric; null hypothesis for BF. Default 0.}
}
\value{
A data frame with columns \code{type}, \code{predictor},
\code{outcome}, \code{mean}, \code{median}, \code{q5}, \code{q95},
\code{rhat}, \code{ess_bulk}, \code{ess_tail}, and optionally
\code{BF01}, \code{BF10}.
}
\description{
Returns a data frame of autoregressive and/or cross-lagged parameter
summaries with convergence diagnostics, filtered by lag and effect type.
}
